@@ -2,7 +2,7 @@
  * Copyright (C) 2007 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
- *      Pawe Cesar Sanjuan Szklarz <paweld2@gmail.com>
+ *      Paweł Cesar Sanjuan Szklarz <paweld2@gmail.com>
  *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
@@ -36,17 +36,20 @@
 /**
  * gda_xslt_register
  *
- * FIXME: ADD COMMENT
+ * Register the xslt extension to libxslt
  */
 void
 gda_xslt_register (void)
 {
+	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
 	static int init = 0;
+
+	g_static_mutex_lock (&init_mutex);
 	if (!init) {
 		int init_res;
 		init = 1;
 		init_res =
-			xsltRegisterExtModule (GDA_XSLT_EXTENSION_URI,
+			xsltRegisterExtModule (BAD_CAST GDA_XSLT_EXTENSION_URI,
 					       gda_xslt_extension_init,
 					       gda_xslt_extension_shutdown);
 		if (init_res != 0) {
@@ -54,12 +57,13 @@ gda_xslt_register (void)
 				 init_res);
 		}
 	}
+	g_static_mutex_unlock (&init_mutex);
 }
 
 /**
  * gda_xslt_set_execution_context
  *
- * FIXME: ADD COMMENT
+ * Set the internal gda-xslt context to the xsltTransformationContext.
  */
 void
 gda_xslt_set_execution_context (xsltTransformContextPtr tcxt,
@@ -70,13 +74,15 @@ gda_xslt_set_execution_context (xsltTransformContextPtr tcxt,
 
 /**
  * gda_xslt_create_context_simple
+ * @cnc: a #GdaConnection
+ * @error: a place to store errors, or %NULL
  *
- * FIXME: ADD COMMENT
+ * create a gda-xslt context with the Gdaconnection and not predefined queries
  *
  * Returns:
  */
 GdaXsltExCont *
-gda_xslt_create_context_simple (GdaConnection * cnc, GError ** error)
+gda_xslt_create_context_simple (GdaConnection *cnc, G_GNUC_UNUSED GError **error)
 {
 	GdaXsltExCont *local = NULL;
 
@@ -91,7 +97,7 @@ gda_xslt_create_context_simple (GdaConnection * cnc, GError ** error)
 /**
  * gda_xslt_finalize_context
  * 
- * FIXME: ADD COMMENT
+ * clear the gda-xslt context
  *
  * Returns:
  */
