@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2006 - 2008 The GNOME Foundation.
+ * Copyright (C) 2006 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -121,9 +121,11 @@ struct _GdaDataModelImportPrivate {
 	GSList              *columns;
 	GdaDataModel        *random_access_model; /* data is imported into this model if random access is required */
 	GSList              *errors; /* list of errors as GError structures */
-	GdaSet    *options;
+	GdaSet              *options;
 
 	gint                 iter_row;
+	gboolean             init_done;
+	gboolean             strict;
 };
 
 /* properties */
@@ -135,6 +137,7 @@ enum
 	PROP_DATA_STRING,
 	PROP_XML_NODE,
 	PROP_OPTIONS,
+	PROP_STRICT
 };
 
 #define CSV_TITLE_BUFFER_SIZE 255
@@ -174,7 +177,7 @@ static void         add_error (GdaDataModelImport *model, const gchar *err);
 static GObjectClass *parent_class = NULL;
 
 /**
- * gda_data_model_import_get_type
+ * gda_data_model_import_get_type:
  *
  * Returns: the #GType of GdaDataModelImport.
  */
@@ -194,7 +197,8 @@ gda_data_model_import_get_type (void)
 			NULL,
 			sizeof (GdaDataModelImport),
 			0,
-			(GInstanceInitFunc) gda_data_model_import_init
+			(GInstanceInitFunc) gda_data_model_import_init,
+			0
 		};
 
 		static const GInterfaceInfo data_model_info = {
@@ -223,29 +227,69 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 	/* properties */
 	object_class->set_property = gda_data_model_import_set_property;
         object_class->get_property = gda_data_model_import_get_property;
+	/**
+	 * GdaDataModelImport:random-access:
+	 *
+	 * Defines if the data model will be accessed randomly or through a cursor. If set to %FALSE,
+	 * access will have to be done using a cursor.
+	 */
 	g_object_class_install_property (object_class, PROP_RANDOM_ACCESS,
                                          g_param_spec_boolean ("random-access", NULL, "Random access to the data model "
 							       "is possible",
 							       FALSE,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * GdaDataModelImport:filename:
+	 *
+	 * Name of the file to import.
+	 */
 	g_object_class_install_property (object_class, PROP_FILENAME,
                                          g_param_spec_string ("filename", NULL, "File to import", NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * GdaDataModelImport:data-string:
+	 *
+	 * Data to import, as a string.
+	 */
 	g_object_class_install_property (object_class, PROP_DATA_STRING,
                                          g_param_spec_string ("data-string", NULL, "String to import", NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * GdaDataModelImport:xml-node:
+	 *
+	 * Data to import, as a pointer to an XML node (a #xmlNodePtr).
+	 */
 	g_object_class_install_property (object_class, PROP_XML_NODE,
                                          g_param_spec_pointer ("xml-node", NULL, "XML node to import from",
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * GdaDataModelImport:options:
+	 *
+	 * Data model options.
+	 */
 	g_object_class_install_property (object_class, PROP_OPTIONS,
                                          g_param_spec_object ("options", NULL, "Options to configure the import",
                                                                GDA_TYPE_SET,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * GdaDataModelImport:strict:
+	 *
+	 * Defines the behaviour in case the imported data contains recoverable errors (usually too
+	 * many or too few data per row). If set to %TRUE, an error will be reported and the import
+	 * will stop, and if set to %FALSE, then the error will be reported but the import will not stop.
+	 *
+	 * Since: 4.2.1
+	 */
+	g_object_class_install_property (object_class, PROP_STRICT,
+                                         g_param_spec_boolean ("strict", NULL, "Consider missing or too much values an error",
+							       FALSE,
+							       G_PARAM_READABLE | G_PARAM_WRITABLE |
+							       G_PARAM_CONSTRUCT));
 
 	/* virtual functions */
 	object_class->dispose = gda_data_model_import_dispose;
@@ -281,7 +325,7 @@ gda_data_model_import_data_model_init (GdaDataModelIface *iface)
 }
 
 static void
-gda_data_model_import_init (GdaDataModelImport *model, GdaDataModelImportClass *klass)
+gda_data_model_import_init (GdaDataModelImport *model, G_GNUC_UNUSED GdaDataModelImportClass *klass)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL_IMPORT (model));
 	model->priv = g_new0 (GdaDataModelImportPrivate, 1);
@@ -301,12 +345,14 @@ gda_data_model_import_init (GdaDataModelImport *model, GdaDataModelImportClass *
 	model->priv->data_length = 0;
 
 	model->priv->iter_row = -1;
+	model->priv->init_done = FALSE;
+	model->priv->strict = FALSE;
 }
 
 static void
 csv_free_stored_rows (GdaDataModelImport *model)
 {
-	gint i;
+	gsize i;
 	g_assert (model->priv->format == FORMAT_CSV);
 	for (i = 0; i < model->priv->extract.csv.rows_read->len; i++) {
 		GSList *list = g_array_index (model->priv->extract.csv.rows_read,
@@ -375,7 +421,6 @@ gda_data_model_import_dispose (GObject *object)
 			if (model->priv->extract.xml.reader) {
 				xmlFreeTextReader (model->priv->extract.xml.reader);
 				model->priv->extract.xml.reader = NULL;
-				/* xmlCleanupParser (); to be included? */
 			}
 			break;
 		case FORMAT_CSV:
@@ -500,7 +545,7 @@ gda_data_model_import_set_property (GObject *object,
 			model->priv->src.mapped.filename = g_strdup (g_value_get_string (value));
 
 			/* file opening */
-			model->priv->src.mapped.fd = open (model->priv->src.mapped.filename, O_RDONLY);
+			model->priv->src.mapped.fd = open (model->priv->src.mapped.filename, O_RDONLY); /* Flawfinder: ignore */
 			if (model->priv->src.mapped.fd < 0) {
 				/* error */
 				add_error (model, strerror(errno));
@@ -527,18 +572,59 @@ gda_data_model_import_set_property (GObject *object,
 					return;
 				}
 #else
-				HANDLE view = CreateFileMapping((HANDLE)model->priv->src.mapped.fd, 
+				HANDLE hFile = CreateFile (model->priv->src.mapped.filename,
+				                           GENERIC_READ,
+										   FILE_SHARE_READ,
+										   NULL,
+										   OPEN_EXISTING,
+										   FILE_ATTRIBUTE_NORMAL,
+										   NULL);
+				if (!hFile) {
+						LPVOID lpMsgBuf;
+						DWORD dw = GetLastError();
+						FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
+									   FORMAT_MESSAGE_FROM_SYSTEM |
+									   FORMAT_MESSAGE_IGNORE_INSERTS,
+									   NULL,
+									   dw,
+									   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+									   (LPTSTR) &lpMsgBuf,
+									   0, NULL);
+						add_error (model, (gchar *)lpMsgBuf);
+						return;
+				}
+				HANDLE view = CreateFileMapping(hFile, 
 								NULL, PAGE_READONLY|SEC_COMMIT, 0,0 , NULL);
 				if (!view) {
 					/* error */
-					add_error (model, sys_errlist [errno]);
+					LPVOID lpMsgBuf;
+					DWORD dw = GetLastError();
+					FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
+					               FORMAT_MESSAGE_FROM_SYSTEM |
+								   FORMAT_MESSAGE_IGNORE_INSERTS,
+								   NULL,
+								   dw,
+								   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+								   (LPTSTR) &lpMsgBuf,
+								   0, NULL);
+					add_error (model, (gchar *)lpMsgBuf);
 					return;
 				}
 				model->priv->src.mapped.start = MapViewOfFile(view, FILE_MAP_READ, 0, 0, 
 									      model->priv->src.mapped.length);
 				if (!model->priv->src.mapped.start) {
 					/* error */
-					add_error (model, sys_errlist [errno]);
+					LPVOID lpMsgBuf;
+					DWORD dw = GetLastError();
+					FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
+					               FORMAT_MESSAGE_FROM_SYSTEM |
+								   FORMAT_MESSAGE_IGNORE_INSERTS,
+								   NULL,
+								   dw,
+								   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+								   (LPTSTR) &lpMsgBuf,
+								   0, NULL);
+					add_error (model, (gchar *)lpMsgBuf);
 					return;
 				}
 #endif
@@ -564,7 +650,7 @@ gda_data_model_import_set_property (GObject *object,
 			break;
                 }
 		case PROP_OPTIONS:
-                        if(model->priv->options)
+                        if (model->priv->options)
                           g_object_unref(model->priv->options);
 
 			model->priv->options = g_value_get_object (value);
@@ -577,60 +663,68 @@ gda_data_model_import_set_property (GObject *object,
 					g_object_ref (model->priv->options);
 			}
 			return;
+		case PROP_STRICT:
+			model->priv->strict = g_value_get_boolean (value);
+			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
 
-	/* here we now have a valid data to analyse, try to determine the real kind of data 
+	/* here we now have a valid data to analyze, try to determine the real kind of data 
 	 * (CVS text of XML) */
-	if (model->priv->format != FORMAT_XML_NODE) {
-		g_assert (model->priv->data_start);
+	if (model->priv->format != FORMAT_XML_NODE
+	    && model->priv->data_start) {
 		if (!strncmp (model->priv->data_start, "<?xml", 5)) 
 			model->priv->format = FORMAT_XML_DATA;
 		else 
 			model->priv->format = FORMAT_CSV;
 	}
 
-	/* analyse common options and init. */
-	switch (model->priv->format) {
-	case FORMAT_XML_DATA:
+	/* analyze common options and init.
+	 * WARNING when adding properties: we need to avoid double initialization here... */
+	if (! model->priv->init_done) {
+		model->priv->init_done = TRUE;
+		switch (model->priv->format) {
+		case FORMAT_XML_DATA:
 			init_xml_import (model);
 			break;
 			
-	case FORMAT_CSV:
-		model->priv->extract.csv.quote = '"';
-		if (model->priv->options) {
-			const gchar *option;
-			option = find_option_as_string (model, "ENCODING");
-			if (option) 
-				model->priv->extract.csv.encoding = g_strdup (option);
-			option = find_option_as_string (model, "SEPARATOR");
-			if (option)
-				model->priv->extract.csv.delimiter = *option;
+		case FORMAT_CSV:
 			model->priv->extract.csv.quote = '"';
-			option = find_option_as_string (model, "QUOTE");
-			if (option)
-				model->priv->extract.csv.quote = *option;
+			if (model->priv->options) {
+				const gchar *option;
+				option = find_option_as_string (model, "ENCODING");
+				if (option) 
+					model->priv->extract.csv.encoding = g_strdup (option);
+				option = find_option_as_string (model, "SEPARATOR");
+				if (option)
+					model->priv->extract.csv.delimiter = *option;
+				model->priv->extract.csv.quote = '"';
+				option = find_option_as_string (model, "QUOTE");
+				if (option)
+					model->priv->extract.csv.quote = *option;
+			}
+			init_csv_import (model);
+			break;
+			
+		case FORMAT_XML_NODE:
+			model->priv->random_access = TRUE;
+			init_node_import (model);
+			break;
+		default:
+			g_assert_not_reached ();
 		}
-		init_csv_import (model);
-		break;
 		
-	case FORMAT_XML_NODE:
-		model->priv->random_access = TRUE;
-		init_node_import (model);
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-
-	/* for random access, create a new GdaDataModelArray model and copy the contents from this model */
-	if (model->priv->random_access && model->priv->columns && !model->priv->random_access_model) {
-		GdaDataModel *ramodel;
-
-		ramodel = gda_data_access_wrapper_new ((GdaDataModel *) model);		
-		model->priv->random_access_model = ramodel;
+		/* for random access, create a new GdaDataModelArray model and copy the contents
+		   from this model */
+		if (model->priv->random_access && model->priv->columns && !model->priv->random_access_model) {
+			GdaDataModel *ramodel;
+			
+			ramodel = gda_data_access_wrapper_new ((GdaDataModel *) model);		
+			model->priv->random_access_model = ramodel;
+		}
 	}
 }
 
@@ -660,18 +754,21 @@ gda_data_model_import_get_property (GObject *object,
 			else
 				g_value_set_string (value, model->priv->src.string);
 			break;
+		case PROP_STRICT:
+			g_value_set_boolean (value, model->priv->strict);
+			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
 }
 
 /**
- * gda_data_model_import_new_file
+ * gda_data_model_import_new_file:
  * @filename: the file to import data from
  * @random_access: TRUE if random access will be required
- * @options: list of importing options
+ * @options: (transfer none) (allow-none): importing options
  *
  * Creates a new #GdaDataModel object which contains the data stored within the @filename file.
  *
@@ -689,7 +786,9 @@ gda_data_model_import_get_property (GObject *object,
  *   <listitem><para>Other formats: no option</para></listitem>
  * </itemizedlist>
  *
- * Returns: a pointer to the newly created #GdaDataModel.
+ * Note: after the creation, please use gda_data_model_import_get_errors() to check any error.
+ *
+ * Returns: (transfer full): a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
 gda_data_model_import_new_file   (const gchar *filename, gboolean random_access, GdaSet *options)
@@ -707,17 +806,17 @@ gda_data_model_import_new_file   (const gchar *filename, gboolean random_access,
 }
 
 /**
- * gda_data_model_import_new_mem
- * @data: a string containng the data to import
+ * gda_data_model_import_new_mem:
+ * @data: a string containing the data to import
  * @random_access: TRUE if random access will be required
- * @options: list of importing options, see gda_data_model_import_new_file() for more information
+ * @options: (transfer none) (allow-none): importing options, see gda_data_model_import_new_file() for more information
  *
  * Creates a new #GdaDataModel object which contains the data stored in the @data string. 
  *
  * Important note: the @data string is not copied for memory efficiency reasons and should not
  * therefore be altered in any way as long as the returned data model exists.
  *
- * Returns: a pointer to the newly created #GdaDataModel.
+ * Returns: (transfer full): a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
 gda_data_model_import_new_mem (const gchar *data, gboolean random_access, GdaSet *options)
@@ -733,13 +832,13 @@ gda_data_model_import_new_mem (const gchar *data, gboolean random_access, GdaSet
 }
 
 /**
- * gda_data_model_import_new_xml_node
- * @node: an XML node corresponding to a &lt;data-array&gt; tag
+ * gda_data_model_import_new_xml_node:
+ * @node: (transfer none): an XML node corresponding to a &lt;data-array&gt; tag
  *
  * Creates a new #GdaDataModel and loads the data in @node. The resulting data model
  * can be accessed in a random way.
  *
- * Returns: a pointer to the newly created #GdaDataModel.
+ * Returns: (transfer full): a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
 gda_data_model_import_new_xml_node (xmlNodePtr node)
@@ -956,7 +1055,7 @@ csv_parser_field_read_cb (char *s, size_t len, void *data)
 }
 
 static void 
-csv_parser_row_read_cb (char c, void *data) 
+csv_parser_row_read_cb (G_GNUC_UNUSED char c, void *data)
 {
 	CsvParserData *pdata = (CsvParserData* ) data;
 	GSList *row;
@@ -1170,12 +1269,16 @@ init_xml_import (GdaDataModelImport *model)
 			}
 			str = (gchar*)xmlTextReaderGetAttribute (reader, (xmlChar*)"size");
 			if (str) {
-				spec->size = atoi (str);
+				spec->size = atoi (str); /* Flawfinder: ignore */
+				if (spec->size < 0)
+					spec->size = 0;
 				xmlFree (str);
 			}
 			str = (gchar*)xmlTextReaderGetAttribute (reader, (xmlChar*)"scale");
 			if (str) {
-				spec->scale = atoi (str);
+				spec->scale = atoi (str); /* Flawfinder: ignore */
+				if (spec->scale < 0)
+					spec->scale = 0;
 				xmlFree (str);
 			}
 			str = (gchar*)xmlTextReaderGetAttribute (reader, (xmlChar*)"pkey");
@@ -1488,12 +1591,16 @@ init_node_import (GdaDataModelImport *model)
 			}
 			str = (gchar*)xmlGetProp (cur, (xmlChar*)"size");
 			if (str) {
-				spec->size = atoi (str);
+				spec->size = atoi (str); /* Flawfinder: ignore */
+				if (spec->size < 0)
+					spec->size = 0;
 				xmlFree (str);
 			}
 			str = (gchar*)xmlGetProp (cur, (xmlChar*)"scale");
 			if (str) {
-				spec->scale = atoi (str);
+				spec->scale = atoi (str); /* Flawfinder: ignore */
+				if (spec->scale < 0)
+					spec->scale = 0;
 				xmlFree (str);
 			}
 			str = (gchar*)xmlGetProp (cur, (xmlChar*)"pkey");
@@ -1587,13 +1694,13 @@ add_error (GdaDataModelImport *model, const gchar *err)
 }
 
 /**
- * gda_data_model_import_get_errors
+ * gda_data_model_import_get_errors:
  * @model: a #GdaDataModelImport object
  *
  * Get the list of errors which @model has to report. The returned list is a list of
  * #GError structures, and must not be modified
  *
- * Returns: the list of errors (which must not be modified), or %NULL
+ * Returns: (transfer none) (element-type GObject.Error): the list of errors (which must not be modified), or %NULL
  */
 GSList *
 gda_data_model_import_get_errors (GdaDataModelImport *model)
@@ -1605,7 +1712,7 @@ gda_data_model_import_get_errors (GdaDataModelImport *model)
 }
 
 /**
- * gda_data_model_import_clean_errors
+ * gda_data_model_import_clean_errors:
  * @model: a #GdaDataModelImport object
  *
  * Clears the history of errors @model has to report
@@ -1712,7 +1819,7 @@ gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row, GEr
 }
 
 static GdaValueAttribute
-gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, G_GNUC_UNUSED gint row)
 {
 	GdaValueAttribute flags = 0;
 	GdaDataModelImport *imodel;
@@ -1762,15 +1869,26 @@ add_error_too_few_values (GdaDataModelImport *model)
 
 	switch (model->priv->format){
 	case FORMAT_CSV:
-		str = g_strdup_printf (_("Row at line %d does not have enough values, "
-					 "completed with NULL values"),
-				       model->priv->extract.csv.text_line);
+		if (model->priv->strict)
+			str = g_strdup_printf (_("Row at line %d does not have enough values"),
+					       model->priv->extract.csv.text_line > 1 ?
+					       model->priv->extract.csv.text_line - 1 :
+					       model->priv->extract.csv.text_line);
+		else
+			str = g_strdup_printf (_("Row at line %d does not have enough values, "
+						 "completed with NULL values"),
+					       model->priv->extract.csv.text_line > 1 ?
+					       model->priv->extract.csv.text_line - 1 :
+					       model->priv->extract.csv.text_line);
 		add_error (model, str);
 		g_free (str);
 		break;
 	default:
-		add_error (model, ("Row does not have enough values, "
-				    "completed with NULL values"));
+		if (model->priv->strict)
+			add_error (model, ("Row does not have enough values"));
+		else
+			add_error (model, ("Row does not have enough values, "
+					   "completed with NULL values"));
 		break;
 	}
 }
@@ -1816,6 +1934,9 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 		break;
 
 	case FORMAT_CSV:
+		if (! imodel->priv->extract.csv.rows_read)
+			return FALSE;
+
 		if (gda_data_model_iter_is_valid (iter) &&
 		    (imodel->priv->extract.csv.rows_read->len > 0)) {
 			/* get rid of row pointer by iter */
@@ -1862,11 +1983,16 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 			}
 		}
 		if (plist || vlist) {
-			allok = FALSE;
+			if (imodel->priv->strict)
+				allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				for (; plist; plist = plist->next)
-					gda_holder_force_invalid (GDA_HOLDER (plist->data));
+				for (; plist; plist = plist->next) {
+					if (imodel->priv->strict)
+						gda_holder_force_invalid (GDA_HOLDER (plist->data));
+					else
+						gda_holder_set_value (GDA_HOLDER (plist->data), NULL, NULL);
+				}
 			}
 			else
 				add_error_too_many_values (imodel);
@@ -1935,11 +2061,16 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 			}
 		}
 		if (plist || vlist) {
-			allok = FALSE;
+			if (imodel->priv->strict)
+				allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				for (; plist; plist = plist->next)
-					gda_holder_force_invalid (GDA_HOLDER (plist->data));
+				for (; plist; plist = plist->next) {
+					if (imodel->priv->strict)
+						gda_holder_force_invalid (GDA_HOLDER (plist->data));
+					else
+						gda_holder_set_value (GDA_HOLDER (plist->data), NULL, NULL);
+				}
 			}
 			else 
 				add_error_too_many_values (imodel);
