@@ -1,6 +1,6 @@
 /* gda-meta-struct.c
  *
- * Copyright (C) 2008 Vivien Malerba
+ * Copyright (C) 2008 - 2011 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -100,7 +100,8 @@ gda_meta_struct_get_type (void) {
 			NULL,
 			sizeof (GdaMetaStruct),
 			0,
-			(GInstanceInitFunc) gda_meta_struct_init
+			(GInstanceInitFunc) gda_meta_struct_init,
+			0
 		};
 		
 		g_static_mutex_lock (&registering);
@@ -152,12 +153,13 @@ static void
 gda_meta_struct_init (GdaMetaStruct *mstruct) {
 	mstruct->priv = g_new0 (GdaMetaStructPrivate, 1);
 	mstruct->priv->store = NULL;
+	mstruct->priv->db_objects = NULL;
 	mstruct->priv->index = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
 
 /**
- * gda_meta_struct_new
+ * gda_meta_struct_new:
  * @store: a #GdaMetaStore from which the new #GdaMetaStruct object will fetch information
  * @features: the kind of extra information the new #GdaMetaStruct object will compute
  *
@@ -165,7 +167,7 @@ gda_meta_struct_init (GdaMetaStruct *mstruct) {
  * the more features, the more time it takes to run. Features such as table's columns, each column's attributes, etc
  * are not optional and will always be computed.
  *
- * Returns: the newly created #GdaMetaStruct object
+ * Returns: (transfer full): the newly created #GdaMetaStruct object
  */
 GdaMetaStruct *
 gda_meta_struct_new (GdaMetaStore *store, GdaMetaStructFeature features) 
@@ -240,6 +242,7 @@ gda_meta_struct_set_property (GObject *object,
 			mstruct->priv->features = g_value_get_uint (value);
 			break;
 		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
@@ -263,20 +266,21 @@ gda_meta_struct_get_property (GObject *object,
 			g_value_set_uint (value, mstruct->priv->features);
 			break;
 		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
 		}
 	}
 }
 
 static void
-meta_store_changed_cb (GdaMetaStore *store, GSList *changes, GdaMetaStruct *mstruct)
+meta_store_changed_cb (GdaMetaStore *store, G_GNUC_UNUSED GSList *changes, GdaMetaStruct *mstruct)
 {
 	/* for now we mark ALL the db objects as outdated */
 	meta_store_reset_cb (store, mstruct);
 }
 
 static void
-meta_store_reset_cb (GdaMetaStore *store, GdaMetaStruct *mstruct)
+meta_store_reset_cb (G_GNUC_UNUSED GdaMetaStore *store, GdaMetaStruct *mstruct)
 {
 	/* mark ALL the db objects as outdated */
 	GSList *list;
@@ -311,11 +315,12 @@ compute_view_dependencies (GdaMetaStruct *mstruct, GdaMetaDbObject *view_dbobj,
 				tmp_obj = gda_meta_struct_complement (m2,
 								      GDA_META_DB_VIEW, NULL, NULL, vname, NULL);
 			gda_value_free (vname);
-			g_object_unref (m2);
 
-			if (!tmp_obj) 
+			if (!tmp_obj) {
 				/* could not find dependency */
+				g_object_unref (m2);
 				continue;
+			}
 
 			/* the dependency exists, and is identified by tmp_obj->obj_catalog, tmp_obj->obj_schema 
 			 * and tmp_obj->obj_name */
@@ -336,6 +341,7 @@ compute_view_dependencies (GdaMetaStruct *mstruct, GdaMetaDbObject *view_dbobj,
 						       tmp_obj->obj_schema, tmp_obj->obj_name);
 				g_hash_table_insert (mstruct->priv->index, str, ref_obj);
 			}
+			g_object_unref (m2);
 			g_assert (ref_obj);
 			gda_value_free (catalog);
 			gda_value_free (schema);
@@ -414,13 +420,13 @@ prepare_sql_identifier_for_compare (gchar *str)
 }
 
 /**
- * gda_meta_struct_complement
+ * gda_meta_struct_complement:
  * @mstruct: a #GdaMetaStruct object
  * @type: the type of object to add (which can be GDA_META_DB_UNKNOWN)
- * @catalog: the catalog the object belongs to (as a G_TYPE_STRING GValue), or %NULL
- * @schema: the schema the object belongs to (as a G_TYPE_STRING GValue), or %NULL
+ * @catalog: (allow-none): the catalog the object belongs to (as a G_TYPE_STRING GValue), or %NULL
+ * @schema: (allow-none): the schema the object belongs to (as a G_TYPE_STRING GValue), or %NULL
  * @name: the object's name (as a G_TYPE_STRING GValue), not %NULL
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Creates a new #GdaMetaDbObject structure in @mstruct to represent the database object (of type @type)
  * which can be uniquely identified as @catalog.@schema.@name.
@@ -442,7 +448,7 @@ prepare_sql_identifier_for_compare (gchar *str)
  * For more information, see the <link linkend="information_schema:sql_identifiers">
  * meta data section about SQL identifiers</link>.
  *
- * Returns: the #GdaMetaDbObject corresponding to the database object if no error occurred, or %NULL
+ * Returns: (transfer none): the #GdaMetaDbObject corresponding to the database object if no error occurred, or %NULL
  */
 GdaMetaDbObject *
 gda_meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
@@ -545,6 +551,263 @@ gda_meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 	return dbo;
 }
 
+static GdaMetaForeignKeyPolicy
+policy_string_to_value (const gchar *string)
+{
+	if (!string)
+		return GDA_META_FOREIGN_KEY_UNKNOWN;
+	if (*string == 'C')
+		return GDA_META_FOREIGN_KEY_CASCADE;
+	else if (*string == 'R')
+		return GDA_META_FOREIGN_KEY_RESTRICT;
+	else if (*string == 'N') {
+		if (!strcmp (string + 1, "ONE"))
+			return GDA_META_FOREIGN_KEY_NONE;
+		else
+			return GDA_META_FOREIGN_KEY_NO_ACTION;
+	}
+	else if (*string == 'S') {
+		if (!strcmp (string + 1, "ET NULL"))
+			return GDA_META_FOREIGN_KEY_SET_NULL;
+		else
+			return GDA_META_FOREIGN_KEY_SET_DEFAULT;
+	} 
+	return GDA_META_FOREIGN_KEY_UNKNOWN;
+}
+
+/*
+ * Find if there is already a declared foreign for @dbo to @ref_dbo using the columns
+ * listed in @columns
+ *
+ * @columns contains 4 columns, with no NULL value, and with nrows>0
+ *  - @dbo's column name (string)
+ *  - @dbo's column name ordinal position (int)
+ *  - @ref_dbo's column name (string)
+ *  - @ref_dbo's column name ordinal position (int)
+ */
+static GdaMetaTableForeignKey *
+find_real_foreign_key (GdaMetaTable *table, GdaMetaDbObject *ref_dbo, GdaDataModel *columns)
+{
+	GSList *list;
+	gint nrows = -1;
+	for (list = table->fk_list; list; list = list->next) {
+		GdaMetaTableForeignKey *fk = (GdaMetaTableForeignKey*) list->data;
+		gint i;
+		gboolean *mapping;
+		if (fk->depend_on != ref_dbo)
+			continue;
+
+		if (nrows == -1)
+			nrows = gda_data_model_get_n_rows (columns);
+
+		if (nrows != fk->cols_nb)
+			continue;
+
+		mapping = g_new0 (gboolean, nrows);
+		for (i = 0; i < nrows; i++) {
+			const GValue *cvalues[4];
+			const gchar *fk_colname, *ref_colname;
+			gint fk_pos, ref_pos;
+			gint j;
+			for (j = 0; j < 4; j++) {
+				cvalues[j] = gda_data_model_get_value_at (columns, j, i, NULL);
+				if (!cvalues[j]) {
+					g_free (mapping);
+					goto out;
+				}
+			}
+			fk_colname = g_value_get_string (cvalues[0]);
+			fk_pos = g_value_get_int (cvalues[1]);
+			ref_colname = g_value_get_string (cvalues[2]);
+			ref_pos = g_value_get_int (cvalues[3]);
+			if (!fk_colname || !*fk_colname ||
+			    !ref_colname || !*ref_colname) {
+				g_free (mapping);
+				goto out;
+			}
+
+			for (j = 0; j < nrows; j++) {
+				if (mapping [j]) /* row already mapped? */
+					continue;
+				if ((fk_pos == fk->fk_cols_array[j]) &&
+				    (ref_pos == fk->ref_pk_cols_array[j]) &&
+				    !strcmp (fk_colname, fk->fk_names_array[j]) &&
+				    !strcmp (ref_colname, fk->ref_pk_names_array[j])) {
+					mapping [j] = TRUE;
+					break;
+				}
+			}
+			if (j == nrows)
+				/* i'th row has not been mapped */
+				break;
+		}
+		g_free (mapping);
+		if (i == nrows)
+			/* all the rows have been mapped */
+			return fk;
+		else
+			continue;
+	}
+ out:
+	return NULL;
+}
+
+static gboolean
+add_declared_foreign_keys (GdaMetaStruct *mstruct, GdaMetaTable *mt, GError **error)
+{
+	const gchar *sql1 = "SELECT DISTINCT constraint_name, ref_table_catalog, ref_table_schema, ref_table_name FROM __declared_fk WHERE table_catalog = ##tc::string AND table_schema = ##ts::string AND table_name = ##tname::string ORDER BY constraint_name, ref_table_catalog, ref_table_schema, ref_table_name";
+	const gchar *sql2 = "select de.column_name, c.ordinal_position, de.ref_column_name, rc.ordinal_position FROM __declared_fk de LEFT JOIN _columns c ON (de.table_catalog=c.table_catalog AND de.table_schema=c.table_schema AND de.table_name=c.table_name AND de.column_name=c.column_name) LEFT JOIN _columns rc ON (de.ref_table_catalog=rc.table_catalog AND de.ref_table_schema=rc.table_schema AND de.ref_table_name=rc.table_name AND de.ref_column_name=rc.column_name) WHERE de.constraint_name=##cname::string AND de.table_catalog=##tc::string AND de.table_schema=##ts::string AND de.table_name=##tname::string";
+	GdaDataModel *model, *cmodel = NULL;
+	gint i, nrows;
+	GValue *v1, *v2, *v3;
+	GdaMetaDbObject *dbo;
+
+	dbo = (GdaMetaDbObject*) mt;
+	g_value_set_string ((v1 = gda_value_new (G_TYPE_STRING)), dbo->obj_catalog);
+	g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)), dbo->obj_schema);
+	g_value_set_string ((v3 = gda_value_new (G_TYPE_STRING)), dbo->obj_name);
+	model = gda_meta_store_extract (mstruct->priv->store, sql1, 
+					error, "tc", v1, "ts", v2, "tname", v3, NULL);
+	if (!model) 
+		goto onerror;
+
+	nrows = gda_data_model_get_n_rows (model);
+	for (i = 0; i < nrows; i++) {
+		GdaMetaDbObject *ref_dbo = NULL;
+		GdaMetaTableForeignKey *tfk = NULL;
+		const GValue *fk_catalog, *fk_schema, *fk_tname, *fk_name;
+		gboolean ignore = FALSE;
+
+		fk_name = gda_data_model_get_value_at (model, 0, i, error);
+		if (!fk_name) goto onerror;		
+		fk_catalog = gda_data_model_get_value_at (model, 1, i, error);
+		if (!fk_catalog) goto onerror;
+		fk_schema = gda_data_model_get_value_at (model, 2, i, error);
+		if (!fk_schema) goto onerror;
+		fk_tname = gda_data_model_get_value_at (model, 3, i, error);
+		if (!fk_tname) goto onerror;
+
+		/* get columns */
+		cmodel = gda_meta_store_extract (mstruct->priv->store, sql2, 
+						 error, "tc", v1, "ts", v2, "tname", v3,
+						 "cname", fk_name, NULL);
+		if (!cmodel) 
+			goto onerror;
+
+		/* create new FK structure */
+		tfk = g_new0 (GdaMetaTableForeignKey, 1);
+		tfk->meta_table = dbo;
+		tfk->declared = (gpointer) 0x01;
+		tfk->fk_name = g_value_dup_string (fk_name);
+		tfk->on_update_policy = GINT_TO_POINTER (GDA_META_FOREIGN_KEY_NONE);
+		tfk->on_delete_policy = GINT_TO_POINTER (GDA_META_FOREIGN_KEY_NONE);
+		
+		/* ignore if some columns are not found in the schema
+		 * (maybe wrong or outdated FK decl) */
+		gint j, cnrows;
+		cnrows = gda_data_model_get_n_rows (cmodel);
+		if (cnrows == 0)
+			ignore = TRUE;
+		else {
+			tfk->cols_nb = cnrows;
+			tfk->fk_cols_array = g_new0 (gint, cnrows);
+			tfk->fk_names_array = g_new0 (gchar *, cnrows);
+			tfk->ref_pk_cols_array = g_new0 (gint, cnrows);
+			tfk->ref_pk_names_array = g_new0 (gchar *, cnrows);
+		}
+		for (j = 0; j < cnrows; j++) {
+			const GValue *ov;
+			ov = gda_data_model_get_value_at (cmodel, 0, j, error);
+			if (!ov) goto onerror;
+			if (G_VALUE_TYPE (ov) != G_TYPE_STRING) {
+				ignore = TRUE;
+				break;
+			}
+			tfk->fk_names_array [j] = g_value_dup_string (ov);
+
+			ov = gda_data_model_get_value_at (cmodel, 1, j, error);
+			if (!ov) goto onerror;
+			if (G_VALUE_TYPE (ov) != G_TYPE_INT) {
+				ignore = TRUE;
+				break;
+			}
+			tfk->fk_cols_array [j] = g_value_get_int (ov);
+
+			ov = gda_data_model_get_value_at (cmodel, 2, j, error);
+			if (!ov) goto onerror;
+			if (G_VALUE_TYPE (ov) != G_TYPE_STRING) {
+				ignore = TRUE;
+				break;
+			}
+			tfk->ref_pk_names_array [j] = g_value_dup_string (ov);
+
+			ov = gda_data_model_get_value_at (cmodel, 3, j, error);
+			if (!ov) goto onerror;
+			if (G_VALUE_TYPE (ov) != G_TYPE_INT) {
+				ignore = TRUE;
+				break;
+			}
+			tfk->ref_pk_cols_array [j] = g_value_get_int (ov);
+		}
+
+		/* ignore if there is already an implemented FK */
+		if (! ignore) {
+			ref_dbo = _meta_struct_get_db_object (mstruct, fk_catalog,
+							      fk_schema, fk_tname);
+			if (ref_dbo && find_real_foreign_key (mt, ref_dbo, cmodel))
+				ignore = TRUE;
+		}
+		if (ignore) {
+			/* ignore this FK end move on to the next one */
+			if (tfk)
+				gda_meta_table_foreign_key_free (tfk);
+			continue;
+		}
+
+		tfk->depend_on = ref_dbo;
+		if (!tfk->depend_on) {
+			gchar *str;
+			tfk->depend_on = g_new0 (GdaMetaDbObject, 1);
+			tfk->depend_on->obj_type = GDA_META_DB_UNKNOWN;
+			tfk->depend_on->obj_catalog = g_strdup (g_value_get_string (fk_catalog));
+			tfk->depend_on->obj_schema = g_strdup (g_value_get_string (fk_schema));
+			tfk->depend_on->obj_name = g_strdup (g_value_get_string (fk_tname));
+			mstruct->priv->db_objects = g_slist_append (mstruct->priv->db_objects, tfk->depend_on);
+			str = g_strdup_printf ("%s.%s.%s", g_value_get_string (fk_catalog), 
+					       g_value_get_string (fk_schema), 
+					       g_value_get_string (fk_tname));
+			g_hash_table_insert (mstruct->priv->index, str, tfk->depend_on);
+		}
+		else if (tfk->depend_on->obj_type == GDA_META_DB_TABLE) {
+			GdaMetaTable *dot = GDA_META_TABLE (tfk->depend_on);
+			dot->reverse_fk_list = g_slist_prepend (dot->reverse_fk_list, tfk);
+		}
+
+		dbo->depend_list = g_slist_append (dbo->depend_list, tfk->depend_on);
+		mt->fk_list = g_slist_prepend (mt->fk_list, tfk);
+	}
+	g_object_unref (model);
+	if (cmodel)
+		g_object_unref (cmodel);
+
+	gda_value_free (v1);
+	gda_value_free (v2);
+	gda_value_free (v3);
+
+	return TRUE;
+
+ onerror:
+	if (model)
+		g_object_unref (model);
+	if (cmodel)
+		g_object_unref (cmodel);
+	gda_value_free (v1);
+	gda_value_free (v2);
+	gda_value_free (v3);
+
+	return FALSE;
+}
+
 static GdaMetaDbObject *
 _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 			 const GValue *icatalog, const GValue *ischema, const GValue *iname, 
@@ -823,7 +1086,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 
 		/* foreign keys */
 		if (mstruct->priv->features & GDA_META_STRUCT_FEATURE_FOREIGN_KEYS) { 
-			sql = "SELECT ref_table_catalog, ref_table_schema, ref_table_name, constraint_name, ref_constraint_name FROM _referential_constraints WHERE table_catalog = ##tc::string AND table_schema = ##ts::string AND table_name = ##tname::string";
+			sql = "SELECT ref_table_catalog, ref_table_schema, ref_table_name, constraint_name, ref_constraint_name, update_rule, delete_rule, constraint_name FROM _referential_constraints WHERE table_catalog = ##tc::string AND table_schema = ##ts::string AND table_name = ##tname::string";
 			model = gda_meta_store_extract (mstruct->priv->store, sql, error, 
 							"tc", icatalog, 
 							"ts", ischema, 
@@ -834,29 +1097,37 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 			nrows = gda_data_model_get_n_rows (model);
 			for (i = 0; i < nrows; i++) {
 				GdaMetaTableForeignKey *tfk;
-				const GValue *fk_catalog, *fk_schema, *fk_name;
-				
+				const GValue *fk_catalog, *fk_schema, *fk_tname, *fk_name;
+				const GValue *upd_policy, *del_policy;
+
 				fk_catalog = gda_data_model_get_value_at (model, 0, i, error);
 				if (!fk_catalog) goto onfkerror;
 				fk_schema = gda_data_model_get_value_at (model, 1, i, error);
 				if (!fk_schema) goto onfkerror;
-				fk_name = gda_data_model_get_value_at (model, 2, i, error);
+				fk_tname = gda_data_model_get_value_at (model, 2, i, error);
+				if (!fk_tname) goto onfkerror;
+				upd_policy = gda_data_model_get_value_at (model, 5, i, error);
+				if (!upd_policy) goto onfkerror;
+				del_policy = gda_data_model_get_value_at (model, 6, i, error);
+				if (!del_policy) goto onfkerror;
+				fk_name = gda_data_model_get_value_at (model, 7, i, error);
 				if (!fk_name) goto onfkerror;
 
 				tfk = g_new0 (GdaMetaTableForeignKey, 1);
 				tfk->meta_table = dbo;
-				tfk->depend_on = _meta_struct_get_db_object (mstruct, fk_catalog, fk_schema, fk_name);
+				tfk->fk_name = g_value_dup_string (fk_name);
+				tfk->depend_on = _meta_struct_get_db_object (mstruct, fk_catalog, fk_schema, fk_tname);
 				if (!tfk->depend_on) {
 					gchar *str;
 					tfk->depend_on = g_new0 (GdaMetaDbObject, 1);
 					tfk->depend_on->obj_type = GDA_META_DB_UNKNOWN;
 					tfk->depend_on->obj_catalog = g_strdup (g_value_get_string (fk_catalog));
 					tfk->depend_on->obj_schema = g_strdup (g_value_get_string (fk_schema));
-					tfk->depend_on->obj_name = g_strdup (g_value_get_string (fk_name));
+					tfk->depend_on->obj_name = g_strdup (g_value_get_string (fk_tname));
 					mstruct->priv->db_objects = g_slist_append (mstruct->priv->db_objects, tfk->depend_on);
 					str = g_strdup_printf ("%s.%s.%s", g_value_get_string (fk_catalog), 
 							       g_value_get_string (fk_schema), 
-							       g_value_get_string (fk_name));
+							       g_value_get_string (fk_tname));
 					g_hash_table_insert (mstruct->priv->index, str, tfk->depend_on);
 				}
 				else if (tfk->depend_on->obj_type == GDA_META_DB_TABLE) {
@@ -864,7 +1135,19 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 					dot->reverse_fk_list = g_slist_prepend (dot->reverse_fk_list, tfk);
 				}
 				dbo->depend_list = g_slist_append (dbo->depend_list, tfk->depend_on);
-				
+
+				if (G_VALUE_TYPE (upd_policy) == G_TYPE_STRING)
+					tfk->on_update_policy = GINT_TO_POINTER (policy_string_to_value (g_value_get_string (upd_policy)));
+				else
+					tfk->on_update_policy = GINT_TO_POINTER (GDA_META_FOREIGN_KEY_UNKNOWN);
+
+				if (G_VALUE_TYPE (upd_policy) == G_TYPE_STRING)
+					tfk->on_delete_policy = GINT_TO_POINTER (policy_string_to_value (g_value_get_string (del_policy)));
+				else
+					tfk->on_delete_policy = GINT_TO_POINTER (GDA_META_FOREIGN_KEY_UNKNOWN);
+
+				tfk->declared = NULL;
+
 				/* FIXME: compute @cols_nb, and all the @*_array members (ref_pk_cols_array must be
 				 * initialized with -1 values everywhere */
 				sql = "SELECT k.column_name, c.ordinal_position FROM _key_column_usage k INNER JOIN _columns c ON (c.table_catalog = k.table_catalog AND c.table_schema = k.table_schema AND c.table_name=k.table_name AND c.column_name=k.column_name) WHERE k.table_catalog = ##tc::string AND k.table_schema = ##ts::string AND k.table_name = ##tname::string AND k.constraint_name = ##cname::string ORDER BY k.ordinal_position";
@@ -885,9 +1168,9 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 				ref_pk_cols = gda_meta_store_extract (mstruct->priv->store, sql, error, 
 								      "tc", fk_catalog,
 								      "ts", fk_schema,
-								      "tname", fk_name,
+								      "tname", fk_tname,
 								      "cname", cvalue, NULL);
-				/*g_print ("tname=%s cvalue=%s\n", gda_value_stringify (fk_name),
+				/*g_print ("tname=%s cvalue=%s\n", gda_value_stringify (fk_tname),
 				  gda_value_stringify (cvalue));*/
 				
 				if (fk_cols && ref_pk_cols) {
@@ -897,7 +1180,13 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 					if (fk_nrows != ref_pk_nrows) {
 						/*gda_data_model_dump (fk_cols, stdout);
 						  gda_data_model_dump (ref_pk_cols, stdout);*/
-						fkerror = TRUE;
+						if (ref_pk_nrows > 0)
+							fkerror = TRUE;
+						else {
+							/* not an error, only the referenced table is not present
+							 * in the meta store, which is possible if the meta
+							 * store was only partially updated */
+						}
 					}
 					else {
 						gint n;
@@ -938,7 +1227,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 						     g_value_get_string (iname),
 						     g_value_get_string (fk_catalog), 
 						     g_value_get_string (fk_schema), 
-						     g_value_get_string (fk_name));
+						     g_value_get_string (fk_tname));
 					goto onfkerror;
 				}
 				if (fk_cols)
@@ -961,6 +1250,8 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 			mt->fk_list = g_slist_reverse (mt->fk_list);
 			g_object_unref (model);
 			/* Note: mt->reverse_fk_list is not determined here */
+
+			add_declared_foreign_keys (mstruct, mt, NULL);
 		}
 		
 		break;
@@ -1031,7 +1322,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 
  onerror:
 	if (dbo)
-		gda_meta_db_object_free (dbo);
+		dbo->obj_type = GDA_META_DB_UNKNOWN;
 
 	return NULL;
 }
@@ -1082,11 +1373,11 @@ array_type_to_sql (GdaMetaStore *store, const GValue *specific_name)
 
 
 /**
- * gda_meta_struct_complement_schema
+ * gda_meta_struct_complement_schema:
  * @mstruct: a #GdaMetaStruct object
- * @catalog: name of a catalog, or %NULL
- * @schema: name of a schema, or %NULL
- * @error: a place to store errors, or %NULL
+ * @catalog: (allow-none): name of a catalog, or %NULL
+ * @schema: (allow-none): name of a schema, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * This method is similar to gda_meta_struct_complement() but creates #GdaMetaDbObject for all the
  * database object which are in the @schema schema (and in the @catalog catalog).
@@ -1284,13 +1575,13 @@ real_gda_meta_struct_complement_all (GdaMetaStruct *mstruct, gboolean default_on
 }
 
 /**
- * gda_meta_struct_complement_default
+ * gda_meta_struct_complement_default:
  * @mstruct: a #GdaMetaStruct object
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * This method is similar to gda_meta_struct_complement() and gda_meta_struct_complement_all()
  * but creates #GdaMetaDbObject for all the
- * database object which are useable using only their short name (that is which do not need to be prefixed by 
+ * database object which are usable using only their short name (that is which do not need to be prefixed by 
  * the schema in which they are to be used).
  *
  * Please refer to gda_meta_struct_complement() form more information.
@@ -1304,9 +1595,9 @@ gda_meta_struct_complement_default (GdaMetaStruct *mstruct, GError **error)
 }
 
 /**
- * gda_meta_struct_complement_all
+ * gda_meta_struct_complement_all:
  * @mstruct: a #GdaMetaStruct object
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * This method is similar to gda_meta_struct_complement() and gda_meta_struct_complement_default()
  * but creates #GdaMetaDbObject for all the database object.
@@ -1322,10 +1613,10 @@ gda_meta_struct_complement_all (GdaMetaStruct *mstruct, GError **error)
 }
 
 /**
- * gda_meta_struct_complement_depend
+ * gda_meta_struct_complement_depend:
  * @mstruct: a #GdaMetaStruct object
  * @dbo: a #GdaMetaDbObject part of @mstruct
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * This method is similar to gda_meta_struct_complement() but creates #GdaMetaDbObject for all the dependencies
  * of @dbo.
@@ -1431,17 +1722,18 @@ db_object_sort_func (GdaMetaDbObject *dbo1, GdaMetaDbObject *dbo2)
 }
 
 /**
- * gda_meta_struct_sort_db_objects
+ * gda_meta_struct_sort_db_objects:
  * @mstruct: a #GdaMetaStruct object
  * @sort_type: the kind of sorting requested
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Reorders the list of database objects within @mstruct in a way specified by @sort_type.
  *
  * Returns: TRUE if no error occurred
  */ 
 gboolean
-gda_meta_struct_sort_db_objects (GdaMetaStruct *mstruct, GdaMetaSortType sort_type, GError **error)
+gda_meta_struct_sort_db_objects (GdaMetaStruct *mstruct, GdaMetaSortType sort_type,
+				 G_GNUC_UNUSED GError **error)
 {
 	GSList *pass_list;
 	GSList *ordered_list = NULL;
@@ -1534,14 +1826,13 @@ _meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, const
 }
 
 /**
- * gda_meta_struct_get_all_db_objects
+ * gda_meta_struct_get_all_db_objects:
  * @mstruct: a #GdaMetaStruct object
  *
  * Get a list of all the #GdaMetaDbObject structures representing database objects in @mstruct. Note that
  * no #GdaMetaDbObject structure must not be modified.
  *
- * Returns: a new #GSList list of pointers to #GdaMetaDbObject structures which must be destroyed after
- * usage using g_slist_free(). The individual #GdaMetaDbObject must not be modified.
+ * Returns: (transfer container): a new #GSList list of pointers to #GdaMetaDbObject structures which must be destroyed after usage using g_slist_free(). The individual #GdaMetaDbObject must not be modified.
  */
 GSList *
 gda_meta_struct_get_all_db_objects (GdaMetaStruct *mstruct)
@@ -1554,10 +1845,10 @@ gda_meta_struct_get_all_db_objects (GdaMetaStruct *mstruct)
 }
 
 /**
- * gda_meta_struct_get_db_object
+ * gda_meta_struct_get_db_object:
  * @mstruct: a #GdaMetaStruct object
- * @catalog: the catalog the object belongs to (as a G_TYPE_STRING GValue), or %NULL
- * @schema: the schema the object belongs to (as a G_TYPE_STRING GValue), or %NULL
+ * @catalog: (allow-none): the catalog the object belongs to (as a G_TYPE_STRING GValue), or %NULL
+ * @schema: (allow-none): the schema the object belongs to (as a G_TYPE_STRING GValue), or %NULL
  * @name: the object's name (as a G_TYPE_STRING GValue), not %NULL
  *
  * Tries to locate the #GdaMetaDbObject structure representing the database object named after
@@ -1566,7 +1857,7 @@ gda_meta_struct_get_all_db_objects (GdaMetaStruct *mstruct)
  * If one or both of @catalog and @schema are %NULL, and more than one database object matches the name, then
  * the return value is also %NULL.
  *
- * Returns: the #GdaMetaDbObject or %NULL if not found
+ * Returns: (transfer none): the #GdaMetaDbObject or %NULL if not found
  */
 GdaMetaDbObject *
 gda_meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, const GValue *schema, const GValue *name)
@@ -1598,14 +1889,14 @@ gda_meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, co
 }
 
 /**
- * gda_meta_struct_get_table_column
+ * gda_meta_struct_get_table_column: (skip):
  * @mstruct: a #GdaMetaStruct object
  * @table: the #GdaMetaTable structure to find the column for
  * @col_name: the name of the column to find (as a G_TYPE_STRING GValue)
  *
  * Tries to find the #GdaMetaTableColumn representing the column named @col_name in @table.
  *
- * Returns: the #GdaMetaTableColumn or %NULL if not found
+ * Returns: (transfer none): the #GdaMetaTableColumn or %NULL if not found
  */
 GdaMetaTableColumn *
 gda_meta_struct_get_table_column (GdaMetaStruct *mstruct, GdaMetaTable *table, const GValue *col_name)
@@ -1627,17 +1918,17 @@ gda_meta_struct_get_table_column (GdaMetaStruct *mstruct, GdaMetaTable *table, c
 }
 
 /**
- * gda_meta_struct_dump_as_graph
+ * gda_meta_struct_dump_as_graph:
  * @mstruct: a #GdaMetaStruct object
  * @info: informs what kind of information to show in the resulting graph
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Creates a new graph (in the GraphViz syntax) representation of @mstruct.
  *
- * Returns: a new string, or %NULL if an error occurred.
+ * Returns: (transfer full): a new string, or %NULL if an error occurred.
  */
 gchar *
-gda_meta_struct_dump_as_graph (GdaMetaStruct *mstruct, GdaMetaGraphInfo info, GError **error)
+gda_meta_struct_dump_as_graph (GdaMetaStruct *mstruct, GdaMetaGraphInfo info, G_GNUC_UNUSED GError **error)
 {
 	GString *string;
 	gchar *result;
@@ -1781,6 +2072,7 @@ gda_meta_db_object_free_contents (GdaMetaDbObject *dbo)
 		TO_IMPLEMENT;
 	}
 	g_slist_free (dbo->depend_list);
+	memset (dbo, 0, sizeof (GdaMetaDbObject));
 }
 
 static void
@@ -1830,6 +2122,7 @@ gda_meta_table_foreign_key_free (GdaMetaTableForeignKey *tfk)
 	g_free (tfk->fk_names_array);
 	g_free (tfk->ref_pk_cols_array);
 	g_free (tfk->ref_pk_names_array);
+	g_free (tfk->fk_name);
 	g_free (tfk);
 }
 
@@ -2200,7 +2493,10 @@ determine_db_object_from_missing_type (GdaMetaStruct *mstruct,
 		g_object_unref (model);
 		return TRUE;
 	}
-	if (model) g_object_unref (model);
+	if (model) {
+		g_object_unref (model);
+		model = NULL;
+	}
 	
 copyerror:
 	if (model)
@@ -2223,17 +2519,17 @@ copyerror:
 }
 
 /**
- * gda_meta_struct_add_db_object
+ * gda_meta_struct_add_db_object:
  * @mstruct: a #GdaMetaStruct object
  * @dbo: a #GdaMetaDbObject structure
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Adds @dbo to the database objects known to @mstruct. In any case (whether an error occured or not)
- * @dbo's responsability is then transferred to @smtruct and should
+ * @dbo's ownership is then transferred to @smtruct and should
  * not be used after calling this function (it may have been destroyed). If you need a pointer to the #GdaMetaDbObject
  * for a database object, use gda_meta_struct_get_db_object().
  *
- * Returns: a pointer to the #GdaMetaDbObject used in @mstruct to represent the added database object (may be @dbo or not)
+ * Returns: (transfer none): a pointer to the #GdaMetaDbObject used in @mstruct to represent the added database object (may be @dbo or not)
  */
 GdaMetaDbObject *
 gda_meta_struct_add_db_object (GdaMetaStruct *mstruct, GdaMetaDbObject *dbo, GError **error)
@@ -2283,7 +2579,7 @@ gda_meta_struct_add_db_object (GdaMetaStruct *mstruct, GdaMetaDbObject *dbo, GEr
 }
 
 /**
- * gda_meta_table_column_get_attribute
+ * gda_meta_table_column_get_attribute:
  * @tcol: a #GdaMetaTableColumn
  * @attribute: attribute name as a string
  *
@@ -2291,7 +2587,7 @@ gda_meta_struct_add_db_object (GdaMetaStruct *mstruct, GdaMetaDbObject *dbo, GEr
  *
  * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-40-Attributes-manager.synopsis">this section</link>.
  *
- * Returns: a read-only #GValue, or %NULL if not attribute named @attribute has been set for @column
+ * Returns: (transfer none): a read-only #GValue, or %NULL if not attribute named @attribute has been set for @column
  */
 const GValue *
 gda_meta_table_column_get_attribute (GdaMetaTableColumn *tcol, const gchar *attribute)
@@ -2300,11 +2596,11 @@ gda_meta_table_column_get_attribute (GdaMetaTableColumn *tcol, const gchar *attr
 }
 
 /**
- * gda_meta_table_column_set_attribute
+ * gda_meta_table_column_set_attribute:
  * @tcol: a #GdaMetaTableColumn
  * @attribute: attribute name as a static string
- * @value: a #GValue, or %NULL
- * @destroy: function called when @attribute has to be freed, or %NULL
+ * @value: (allow-none): a #GValue, or %NULL
+ * @destroy: (allow-none): function called when @attribute has to be freed, or %NULL
  *
  * Set the value associated to a named attribute.
  *
@@ -2329,10 +2625,10 @@ gda_meta_table_column_set_attribute (GdaMetaTableColumn *tcol, const gchar *attr
 }
 
 /**
- * gda_meta_table_column_foreach_attribute
+ * gda_meta_table_column_foreach_attribute:
  * @tcol: a #GdaMetaTableColumn
- * @func: a #GdaAttributesManagerFunc function
- * @data: user data to be passed as last argument of @func each time it is called
+ * @func: (scope call): a #GdaAttributesManagerFunc function
+ * @data: (closure): user data to be passed as last argument of @func each time it is called
  *
  * Calls @func for each attribute set to tcol
  */
