@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2006 - 2009 The GNOME Foundation.
+ * Copyright (C) 2006 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -28,6 +28,7 @@
 #include <libgda/gda-set.h>
 #include <libgda/gda-holder.h>
 #include <libgda/gda-connection.h>
+#include <libgda/gda-config.h>
 #include <libgda/gda-data-model-private.h>
 #include <libgda/gda-data-model-import.h>
 #include <libgda/gda-data-model-array.h>
@@ -201,7 +202,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 
 static void
 gda_server_operation_init (GdaServerOperation *operation,
-			  GdaServerOperationClass *klass)
+			  G_GNUC_UNUSED GdaServerOperationClass *klass)
 {
 	g_return_if_fail (GDA_IS_SERVER_OPERATION (operation));
 
@@ -256,6 +257,16 @@ gda_server_operation_dispose (GObject *object)
 	parent_class->dispose (object);
 }
 
+/* module error */
+GQuark
+gda_server_operation_error_quark (void)
+{
+        static GQuark quark;
+        if (!quark)
+                quark = g_quark_from_static_string ("gda_server_operation_error");
+        return quark;
+}
+
 GType
 gda_server_operation_get_type (void)
 {
@@ -273,7 +284,8 @@ gda_server_operation_get_type (void)
 			NULL,
 			sizeof (GdaServerOperation),
 			0,
-			(GInstanceInitFunc) gda_server_operation_init
+			(GInstanceInitFunc) gda_server_operation_init,
+			0
 		};
 
 		g_static_mutex_lock (&registering);
@@ -443,7 +455,7 @@ node_find_or_create (GdaServerOperation *op, const gchar *path)
  * Computes the complete path of a node
  */
 static gchar *
-node_get_complete_path (GdaServerOperation *op, Node *node)
+node_get_complete_path (G_GNUC_UNUSED GdaServerOperation *op, Node *node)
 {
 	GString *string;
 	gchar *retval;
@@ -601,7 +613,8 @@ gda_server_operation_set_property (GObject *object,
 			break;
 		}
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 
@@ -638,6 +651,9 @@ gda_server_operation_get_property (GObject *object,
 		case PROP_OP_TYPE:
 			g_value_set_int (value, op->priv->op_type);
 			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -891,13 +907,17 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, G
 			
 			prop = xmlGetProp(node, (xmlChar*)"minitems");
 			if (prop) {
-				opnode->d.seq.min_items = atoi ((gchar*)prop);
+				opnode->d.seq.min_items = atoi ((gchar*)prop); /* Flawfinder: ignore */
+				if (opnode->d.seq.min_items < 0)
+					opnode->d.seq.min_items = 0;
 				xmlFree (prop);
 			}
 
 			prop = xmlGetProp(node, (xmlChar*)"maxitems");
 			if (prop) {
-				opnode->d.seq.max_items = atoi ((gchar*)prop);
+				opnode->d.seq.max_items = atoi ((gchar*)prop); /* Flawfinder: ignore */
+				if (opnode->d.seq.max_items < opnode->d.seq.min_items)
+					opnode->d.seq.max_items = opnode->d.seq.min_items;
 				xmlFree (prop);
 			}
 
@@ -1013,7 +1033,7 @@ _gda_server_operation_new_from_string (GdaServerOperationType op_type,
 
 
 /**
- * gda_server_operation_new
+ * gda_server_operation_new:
  * @op_type: type of operation
  * @xml_file: a file which has the specifications for the GdaServerOperation object to create
  *
@@ -1051,7 +1071,7 @@ gda_server_operation_new (GdaServerOperationType op_type, const gchar *xml_file)
 }
 
 /**
- * gda_server_operation_get_node_info
+ * gda_server_operation_get_node_info: (skip):
  * @op: a #GdaServerOperation object
  * @path_format: a complete path to a node (starting with "/") as a format string, similar to g_strdup_printf()'s argument
  * @...: the arguments to insert into the format string
@@ -1059,7 +1079,7 @@ gda_server_operation_new (GdaServerOperationType op_type, const gchar *xml_file)
  * Get information about the node identified by @path. The returned #GdaServerOperationNode structure can be 
  * copied but not modified; it may change or cease to exist if @op changes
  *
- * Returns: a #GdaServerOperationNode structure, or %NULL if the node was not found
+ * Returns: (transfer none) (allow-none): a #GdaServerOperationNode structure, or %NULL if the node was not found
  */
 GdaServerOperationNode *
 gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_format, ...)
@@ -1169,7 +1189,7 @@ gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_fo
 }
 
 /**
- * gda_server_operation_get_op_type
+ * gda_server_operation_get_op_type:
  * @op: a #GdaServerOperation object
  * 
  * Get the type of operation @op is for
@@ -1186,12 +1206,12 @@ gda_server_operation_get_op_type (GdaServerOperation *op)
 }
 
 /**
- * gda_server_operation_op_type_to_string
+ * gda_server_operation_op_type_to_string:
  * @type: a #GdaServerOperationType value
  * 
  * Get a string version of @type
  *
- * Returns: a non %NULL string (do not free or modify)
+ * Returns: (transfer none): a non %NULL string (do not free or modify)
  */
 const gchar *
 gda_server_operation_op_type_to_string (GdaServerOperationType type)
@@ -1211,14 +1231,24 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 		return "DROP_INDEX";
         case GDA_SERVER_OPERATION_RENAME_TABLE:
 		return "RENAME_TABLE";
+        case GDA_SERVER_OPERATION_COMMENT_TABLE:
+		return "COMMENT_TABLE";
         case GDA_SERVER_OPERATION_ADD_COLUMN:
 		return "ADD_COLUMN";
         case GDA_SERVER_OPERATION_DROP_COLUMN:
 		return "DROP_COLUMN";
+        case GDA_SERVER_OPERATION_COMMENT_COLUMN:
+		return "COMMENT_COLUMN";
 	case GDA_SERVER_OPERATION_CREATE_VIEW:
 		return "CREATE_VIEW";
 	case GDA_SERVER_OPERATION_DROP_VIEW:
 		return "DROP_VIEW";
+	case GDA_SERVER_OPERATION_CREATE_USER:
+		return "CREATE_USER";
+	case GDA_SERVER_OPERATION_DROP_USER:
+		return "DROP_USER";
+	case GDA_SERVER_OPERATION_ALTER_USER:
+		return "ALTER_USER";
 	default:
 		g_error (_("Non handled GdaServerOperationType, please report error to "
 			   "http://bugzilla.gnome.org/ for the \"libgda\" product"));
@@ -1226,21 +1256,74 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 	}
 }
 
+/**
+ * gda_server_operation_string_to_op_type:
+ * @str: a string
+ *
+ * Performs the reverse of gda_server_operation_op_type_to_string()
+ *
+ * Returns: the #GdaServerOperationType represented by @str, or #G_MAXINT if @str is not a valid representation
+ * of a #GdaServerOperationType
+ *
+ * Since: 4.2
+ */
+GdaServerOperationType
+gda_server_operation_string_to_op_type (const gchar *str)
+{
+	GdaServerOperationType operation_type = G_MAXINT;
+	g_return_val_if_fail (str && *str, G_MAXINT);
+
+	if (! g_ascii_strcasecmp (str, "CREATE_DB"))
+		operation_type = GDA_SERVER_OPERATION_CREATE_DB;
+	else if	(! g_ascii_strcasecmp (str, "DROP_DB"))
+		operation_type = GDA_SERVER_OPERATION_DROP_DB;
+	else if (! g_ascii_strcasecmp (str, "CREATE_TABLE"))
+		operation_type = GDA_SERVER_OPERATION_CREATE_TABLE;
+	else if (! g_ascii_strcasecmp (str, "DROP_TABLE"))
+		operation_type = GDA_SERVER_OPERATION_DROP_TABLE;
+	else if (! g_ascii_strcasecmp (str, "CREATE_INDEX"))
+		operation_type = GDA_SERVER_OPERATION_CREATE_INDEX;
+	else if (! g_ascii_strcasecmp (str, "DROP_INDEX"))
+		operation_type = GDA_SERVER_OPERATION_DROP_INDEX;
+	else if (! g_ascii_strcasecmp (str, "RENAME_TABLE"))
+		operation_type = GDA_SERVER_OPERATION_RENAME_TABLE;
+	else if (! g_ascii_strcasecmp (str, "COMMENT_TABLE"))
+		operation_type = GDA_SERVER_OPERATION_COMMENT_TABLE;
+	else if (! g_ascii_strcasecmp (str, "ADD_COLUMN"))
+		operation_type = GDA_SERVER_OPERATION_ADD_COLUMN;
+	else if (! g_ascii_strcasecmp (str, "DROP_COLUMN"))
+		operation_type = GDA_SERVER_OPERATION_DROP_COLUMN;
+	else if (! g_ascii_strcasecmp (str, "COMMENT_COLUMN"))
+		operation_type = GDA_SERVER_OPERATION_COMMENT_COLUMN;
+	else if (! g_ascii_strcasecmp (str, "CREATE_VIEW"))
+		operation_type = GDA_SERVER_OPERATION_CREATE_VIEW;
+	else if (! g_ascii_strcasecmp (str, "DROP_VIEW"))
+		operation_type = GDA_SERVER_OPERATION_DROP_VIEW;
+	else if (! g_ascii_strcasecmp (str, "CREATE_USER"))
+		operation_type = GDA_SERVER_OPERATION_CREATE_USER;
+	else if (! g_ascii_strcasecmp (str, "DROP_USER"))
+		operation_type = GDA_SERVER_OPERATION_DROP_USER;
+	else if (! g_ascii_strcasecmp (str, "ALTER_USER"))
+		operation_type = GDA_SERVER_OPERATION_ALTER_USER;
+
+	return operation_type;
+}
+
 static gboolean node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent);
 
 /**
- * gda_server_operation_save_data_to_xml
+ * gda_server_operation_save_data_to_xml: (skip):
  * @op: a #GdaServerOperation object
- * @error: a place to store errors or %NULL
+ * @error: (allow-none): a place to store errors or %NULL
  * 
  * Creates a new #xmlNodePtr tree which can be used to save the #op object. This
  * XML structure can then be saved to disk if necessary. Use xmlFreeNode to free
  * the associated memory when not needed anymore.
  *
- * Returns: a new #xmlNodePtr structure, or %NULL
+ * Returns: (transfer full): a new #xmlNodePtr structure, or %NULL
  */
 xmlNodePtr
-gda_server_operation_save_data_to_xml (GdaServerOperation *op, GError **error)
+gda_server_operation_save_data_to_xml (GdaServerOperation *op, G_GNUC_UNUSED GError **error)
 {
 	xmlNodePtr topnode = NULL;
 	GSList *list;
@@ -1350,10 +1433,10 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 }
 
 /**
- * gda_server_operation_load_data_from_xml
+ * gda_server_operation_load_data_from_xml:
  * @op: a #GdaServerOperation object
  * @node: a #xmlNodePtr
- * @error: a place to store errors or %NULL
+ * @error: (allow-none): a place to store errors or %NULL
  * 
  * Loads the contents of @node into @op. The XML tree passed through the @node
  * argument must correspond to an XML tree saved using gda_server_operation_save_data_to_xml().
@@ -1514,12 +1597,12 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 }
 
 /**
- * gda_server_operation_get_root_nodes
+ * gda_server_operation_get_root_nodes:
  * @op: a #GdaServerOperation object
  *
  * Get an array of strings containing the paths of nodes situated at the root of @op.
  * 
- * Returns: a new array, which must be freed with g_strfreev().
+ * Returns: (transfer full): a new array, which must be freed with g_strfreev().
  */ 
 gchar**
 gda_server_operation_get_root_nodes (GdaServerOperation *op)
@@ -1539,10 +1622,10 @@ gda_server_operation_get_root_nodes (GdaServerOperation *op)
 }
 
 /**
- * gda_server_operation_get_node_type
+ * gda_server_operation_get_node_type:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a node (starting with "/")
- * @status: a place to store the status of the node, or %NULL
+ * @status: (allow-none): a place to store the status of the node, or %NULL
  *
  * Convenience function to get the type of a node.
  *
@@ -1567,13 +1650,13 @@ gda_server_operation_get_node_type (GdaServerOperation *op, const gchar *path,
 }
 
 /**
- * gda_server_operation_get_node_parent
+ * gda_server_operation_get_node_parent:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a node (starting with "/")
  *
  * Get the complete path to the parent of the node defined by @path
  *
- * Returns: a new string or %NULL if the node does not have any parent or does not exist.
+ * Returns: (transfer full): a new string or %NULL if the node does not have any parent or does not exist.
  */
 gchar *
 gda_server_operation_get_node_parent (GdaServerOperation *op, const gchar *path)
@@ -1608,13 +1691,13 @@ gda_server_operation_get_node_parent (GdaServerOperation *op, const gchar *path)
 }
 
 /**
- * gda_server_operation_get_node_path_portion
+ * gda_server_operation_get_node_path_portion:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a node (starting with "/")
  *
  * Get the last part of @path
  *
- * Returns: a new string, or %NULL if an error occurred
+ * Returns: (transfer full): a new string, or %NULL if an error occurred
  */
 gchar *
 gda_server_operation_get_node_path_portion (GdaServerOperation *op, const gchar *path)
@@ -1649,14 +1732,14 @@ gda_server_operation_get_node_path_portion (GdaServerOperation *op, const gchar
 }
 
 /**
- * gda_server_operation_get_sequence_item_names
+ * gda_server_operation_get_sequence_item_names:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a sequence node (starting with "/")
  *
  * Fetch the contents of a sequence. @path can describe either a sequence (for example "/SEQNAME") or an item in a sequence
  * (for example "/SEQNAME/3")
  *
- * Returns: a array of strings containing the complete paths of the nodes contained at @path (free with g_strfreev())
+ * Returns: (transfer full): a array of strings containing the complete paths of the nodes contained at @path (free with g_strfreev())
  */
 gchar **
 gda_server_operation_get_sequence_item_names (GdaServerOperation *op, const gchar *path)
@@ -1687,11 +1770,11 @@ gda_server_operation_get_sequence_item_names (GdaServerOperation *op, const gcha
 }
 
 /**
- * gda_server_operation_get_sequence_name
+ * gda_server_operation_get_sequence_name:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a sequence node (starting with "/")
  *
- * Returns: the name of the sequence at @path
+ * Returns: (transfer none): the name of the sequence at @path
  */
 const gchar *
 gda_server_operation_get_sequence_name (GdaServerOperation *op, const gchar *path)
@@ -1709,7 +1792,7 @@ gda_server_operation_get_sequence_name (GdaServerOperation *op, const gchar *pat
 }
 
 /**
- * gda_server_operation_get_sequence_size
+ * gda_server_operation_get_sequence_size:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a sequence node (starting with "/")
  *
@@ -1731,7 +1814,7 @@ gda_server_operation_get_sequence_size (GdaServerOperation *op, const gchar *pat
 }
 
 /**
- * gda_server_operation_get_sequence_max_size
+ * gda_server_operation_get_sequence_max_size:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a sequence node (starting with "/")
  *
@@ -1753,7 +1836,7 @@ gda_server_operation_get_sequence_max_size (GdaServerOperation *op, const gchar
 }
 
 /**
- * gda_server_operation_get_sequence_min_size
+ * gda_server_operation_get_sequence_min_size:
  * @op: a #GdaServerOperation object
  * @path: a complete path to a sequence node (starting with "/")
  *
@@ -1800,7 +1883,7 @@ dump (GdaServerOperation *op)
 #endif
 
 /**
- * gda_server_operation_add_item_to_sequence
+ * gda_server_operation_add_item_to_sequence:
  * @op: a #GdaServerOperation object
  * @seq_path: the path to the sequence to which an item must be added (like "/SEQ_NAME" for instance)
  *
@@ -1832,7 +1915,7 @@ gda_server_operation_add_item_to_sequence (GdaServerOperation *op, const gchar *
 }
 
 /**
- * gda_server_operation_del_item_from_sequence
+ * gda_server_operation_del_item_from_sequence:
  * @op: a #GdaServerOperation object
  * @item_path: the path to the sequence's item to remove (like "/SEQ_NAME/5" for instance)
  *
@@ -1955,7 +2038,7 @@ real_gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *pat
 }
 
 /**
- * gda_server_operation_get_value_at
+ * gda_server_operation_get_value_at:
  * @op: a #GdaServerOperation object
  * @path_format: a complete path to a node (starting with "/")
  * @...: arguments to use with @path_format to make a complete path
@@ -1963,7 +2046,7 @@ real_gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *pat
  * Get the value for the node at the path formed using @path_format and ... (the rules are the same as
  * for g_strdup_printf())
  *
- * Returns: a constant #GValue if a value has been defined, or %NULL if the value is undefined or
+ * Returns: (transfer none): a constant #GValue if a value has been defined, or %NULL if the value is undefined or
  * if the @path is not defined or @path does not hold any value.
  */
 const GValue *
@@ -1988,7 +2071,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 }
 
 /**
- * gda_server_operation_get_sql_identifier_at
+ * gda_server_operation_get_sql_identifier_at:
  * @op: a #GdaServerOperation object
  * @cnc: a #GdaConnection, or %NULL
  * @prov: a #GdaServerProvider, or %NULL
@@ -2000,7 +2083,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
  * and will correctly be quoted to be used with @cnc, or @prov if @cnc is %NULL (a generic quoting rule
  * will be applied if both are %NULL).
  *
- * Returns: a new string, or %NULL if the value is undefined or
+ * Returns: (transfer full): a new string, or %NULL if the value is undefined or
  * if the @path is not defined or @path does not hold any value, or if the value held is not a string
  * (in that last case a warning is shown).
  *
@@ -2036,7 +2119,7 @@ gda_server_operation_get_sql_identifier_at (GdaServerOperation *op, GdaConnectio
 }
 
 /**
- * gda_server_operation_set_value_at
+ * gda_server_operation_set_value_at:
  * @op: a #GdaServerOperation object
  * @value: a string
  * @error: a place to store errors or %NULL
@@ -2073,7 +2156,7 @@ gda_server_operation_get_sql_identifier_at (GdaServerOperation *op, GdaConnectio
  *  </para></listitem>
  * </itemizedlist>
  *
- * Returns: TRUE if no error occurred
+ * Returns: %TRUE if no error occurred
  */
 gboolean
 gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, GError **error,
@@ -2224,7 +2307,7 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 }
 
 /**
- * gda_server_operation_is_valid
+ * gda_server_operation_is_valid:
  * @op: a #GdaServerOperation widget
  * @xml_file: an XML specification file (see gda_server_operation_new())
  * @error: a place to store an error, or %NULL
@@ -2232,9 +2315,9 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
  * Tells if all the required values in @op have been defined.
  *
  * if @xml_file is not %NULL, the validity of @op is tested against that specification, 
- * and not againts the current @op's specification.
+ * and not against the current @op's specification.
  *
- * Returns: TRUE if @op is valid
+ * Returns: %TRUE if @op is valid
  */
 gboolean
 gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GError **error)
@@ -2255,7 +2338,7 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 				if (node->type == GDA_SERVER_OPERATION_NODE_PARAM) {
 					const GValue *value;
 					gchar *path;
-					
+
 					path = node_get_complete_path (op, node);
 					value = gda_server_operation_get_value_at (op, path);
 					if (!value) {
@@ -2295,3 +2378,437 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 
 	return valid;
 }
+
+/**
+ * gda_server_operation_prepare_create_database:
+ * @provider: the database provider to use
+ * @db_name: the name of the database to create, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Creates a new #GdaServerOperation object which contains the specifications required
+ * to create a database. Once these specifications provided, use
+ * gda_server_operation_perform_create_database() to perform the database creation.
+ *
+ * If @db_name is left %NULL, then the name of the database to create will have to be set in the
+ * returned #GdaServerOperation using gda_server_operation_set_value_at().
+ *
+ * Returns: (transfer full) (allow-none): new #GdaServerOperation object, or %NULL if the provider does not support database
+ * creation
+ *
+ * Since: 4.2.3
+ */
+GdaServerOperation *
+gda_server_operation_prepare_create_database (const gchar *provider, const gchar *db_name, GError **error)
+{
+	GdaServerProvider *prov;
+
+	g_return_val_if_fail (provider && *provider, NULL);
+
+	prov = gda_config_get_provider (provider, error);
+	if (prov) {
+		GdaServerOperation *op;
+		op = gda_server_provider_create_operation (prov, NULL, GDA_SERVER_OPERATION_CREATE_DB,
+							   NULL, error);
+		if (op) {
+			g_object_set_data_full (G_OBJECT (op), "_gda_provider_obj", g_object_ref (prov), g_object_unref);
+			if (db_name)
+				gda_server_operation_set_value_at (op, db_name, NULL, "/DB_DEF_P/DB_NAME");
+		}
+		return op;
+	}
+	else
+		return NULL;
+}
+
+/**
+ * gda_server_operation_perform_create_database:
+ * @provider: the database provider to use, or %NULL if @op has been created using gda_server_operation_prepare_create_database()
+ * @op: a #GdaServerOperation object obtained using gda_server_operation_prepare_create_database()
+ * @error: a place to store en error, or %NULL
+ *
+ * Creates a new database using the specifications in @op. @op can be obtained using
+ * gda_server_provider_create_operation(), or gda_server_operation_prepare_create_database().
+ *
+ * Returns: TRUE if no error occurred and the database has been created, FALSE otherwise
+ *
+ * Since: 4.2.3
+ */
+gboolean
+gda_server_operation_perform_create_database (GdaServerOperation *op, const gchar *provider, GError **error)
+{
+	GdaServerProvider *prov;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+	if (provider)
+		prov = gda_config_get_provider (provider, error);
+	else
+		prov = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
+	if (prov)
+		return gda_server_provider_perform_operation (prov, NULL, op, error);
+	else {
+		g_warning ("Could not find operation's associated provider, "
+			   "did you use gda_server_operation_prepare_create_database() ?");
+		return FALSE;
+	}
+}
+
+/**
+ * gda_server_operation_prepare_drop_database:
+ * @provider: the database provider to use
+ * @db_name: the name of the database to drop, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Creates a new #GdaServerOperation object which contains the specifications required
+ * to drop a database. Once these specifications provided, use
+ * gda_server_operation_perform_drop_database() to perform the database creation.
+ *
+ * If @db_name is left %NULL, then the name of the database to drop will have to be set in the
+ * returned #GdaServerOperation using gda_server_operation_set_value_at().
+ *
+ * Returns: (transfer full) (allow-none): new #GdaServerOperation object, or %NULL if the provider does not support database
+ * destruction
+ *
+ * Since: 4.2.3
+ */
+GdaServerOperation *
+gda_server_operation_prepare_drop_database (const gchar *provider, const gchar *db_name, GError **error)
+{
+	GdaServerProvider *prov;
+
+	g_return_val_if_fail (provider && *provider, NULL);
+
+	prov = gda_config_get_provider (provider, error);
+	if (prov) {
+		GdaServerOperation *op;
+		op = gda_server_provider_create_operation (prov, NULL, GDA_SERVER_OPERATION_DROP_DB,
+							   NULL, error);
+		if (op) {
+			g_object_set_data_full (G_OBJECT (op), "_gda_provider_obj", g_object_ref (prov), g_object_unref);
+			if (db_name)
+				gda_server_operation_set_value_at (op, db_name, NULL, "/DB_DESC_P/DB_NAME");
+		}
+		return op;
+	}
+	else
+		return NULL;
+}
+
+/**
+ * gda_server_operation_perform_drop_database:
+ * @provider: the database provider to use, or %NULL if @op has been created using gda_server_operation_prepare_drop_database()
+ * @op: a #GdaServerOperation object obtained using gda_prepare_drop_database()
+ * @error: a place to store en error, or %NULL
+ *
+ * Destroys an existing database using the specifications in @op. @op can be obtained using
+ * gda_server_provider_create_operation(), or gda_server_operation_prepare_drop_database().
+ *
+ * Returns: TRUE if no error occurred and the database has been destroyed
+ *
+ * Since: 4.2.3
+ */
+gboolean
+gda_server_operation_perform_drop_database (GdaServerOperation *op, const gchar *provider, GError **error)
+{
+	GdaServerProvider *prov;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+	if (provider)
+		prov = gda_config_get_provider (provider, error);
+	else
+		prov = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
+	if (prov)
+		return gda_server_provider_perform_operation (prov, NULL, op, error);
+	else {
+		g_warning ("Could not find operation's associated provider, "
+			   "did you use gda_server_operation_prepare_drop_database() ?");
+		return FALSE;
+	}
+}
+
+/**
+ * gda_server_operation_prepare_create_table:
+ * @cnc: an opened connection
+ * @table_name: name of the table to create
+ * @error: a place to store errors, or %NULL
+ * @...: group of three arguments for column's name, column's #GType
+ * and a #GdaEasyCreateTableFlag flag, finished with %NULL
+ *
+ * Add more arguments if the flag needs them:
+ *
+ * GDA_SERVER_OPERATION_CREATE_TABLE_FKEY_FLAG:
+ * <itemizedlist>
+ *   <listitem><para>string with the table's name referenced</para></listitem>
+ *   <listitem><para>an integer with the number pairs "local_field", "referenced_field"
+ *   used in the reference</para></listitem>
+ *   <listitem><para>Pairs of "local_field", "referenced_field" to use, must match
+ *    the number specified above.</para></listitem>
+ *   <listitem><para>a string with the action for ON DELETE; can be: "RESTRICT", "CASCADE",
+ *    "NO ACTION", "SET NULL" and "SET DEFAULT". Example: "ON UPDATE CASCADE".</para></listitem>
+ *   <listitem><para>a string with the action for ON UPDATE (see above).</para></listitem>
+ * </itemizedlist>
+ *
+ * Create a #GdaServerOperation object using an opened connection, taking three
+ * arguments, a column's name the column's GType and #GdaEasyCreateTableFlag
+ * flag, you need to finish the list using %NULL.
+ *
+ * You'll be able to modify the #GdaServerOperation object to add custom options * to the operation. When finished call #gda_server_operation_perform_create_table
+ * or #gda_server_provider_perform_operation
+ * in order to execute the operation.
+ *
+ * Returns: (transfer full) (allow-none): a #GdaServerOperation if no errors; NULL and set @error otherwise
+ *
+ * Since: 4.2.3
+ */
+G_GNUC_NULL_TERMINATED
+GdaServerOperation*
+gda_server_operation_prepare_create_table (GdaConnection *cnc, const gchar *table_name, GError **error, ...)
+{
+	GdaServerOperation *op;
+	GdaServerProvider *server;
+
+	g_return_val_if_fail (gda_connection_is_opened (cnc), FALSE);
+
+	server = gda_connection_get_provider (cnc);
+
+	if (!table_name) {
+		g_set_error (error, GDA_SERVER_OPERATION_ERROR, GDA_SERVER_OPERATION_OBJECT_NAME_ERROR,
+			     "%s", _("Unspecified table name"));
+		return NULL;
+	}
+
+	if (gda_server_provider_supports_operation (server, cnc, GDA_SERVER_OPERATION_CREATE_TABLE, NULL)) {
+		va_list  args;
+		gchar   *arg;
+		GType    type;
+		gchar   *dbms_type;
+		GdaServerOperationCreateTableFlag flag;
+		gint i;
+		gint refs;
+
+		op = gda_server_provider_create_operation (server, cnc,
+							   GDA_SERVER_OPERATION_CREATE_TABLE, NULL, error);
+		if (!GDA_IS_SERVER_OPERATION(op))
+			return NULL;
+		if(!gda_server_operation_set_value_at (op, table_name, error, "/TABLE_DEF_P/TABLE_NAME")) {
+			g_object_unref (op);
+			return NULL;
+		}
+
+		va_start (args, error);
+		type = 0;
+		arg = NULL;
+		i = 0;
+		refs = -1;
+
+		while ((arg = va_arg (args, gchar*))) {
+			/* First argument for Column's name */
+			if(!gda_server_operation_set_value_at (op, arg, error, "/FIELDS_A/@COLUMN_NAME/%d", i)){
+				g_object_unref (op);
+				return NULL;
+			}
+
+			/* Second to Define column's type */
+			type = va_arg (args, GType);
+			if (type == 0) {
+				g_set_error (error, GDA_SERVER_OPERATION_ERROR, GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
+					     "%s", _("Invalid type"));
+				g_object_unref (op);
+				return NULL;
+			}
+			dbms_type = (gchar *) gda_server_provider_get_default_dbms_type (server,
+											 cnc, type);
+			if (!gda_server_operation_set_value_at (op, dbms_type, error, "/FIELDS_A/@COLUMN_TYPE/%d", i)){
+				g_object_unref (op);
+				return NULL;
+			}
+
+			/* Third for column's flags */
+			flag = va_arg (args, GdaServerOperationCreateTableFlag);
+			if (flag & GDA_SERVER_OPERATION_CREATE_TABLE_PKEY_FLAG)
+				if(!gda_server_operation_set_value_at (op, "TRUE", error, "/FIELDS_A/@COLUMN_PKEY/%d", i)){
+					g_object_unref (op);
+					return NULL;
+				}
+			if (flag & GDA_SERVER_OPERATION_CREATE_TABLE_NOT_NULL_FLAG)
+				if(!gda_server_operation_set_value_at (op, "TRUE", error, "/FIELDS_A/@COLUMN_NNUL/%d", i)){
+					g_object_unref (op);
+					return NULL;
+				}
+			if (flag & GDA_SERVER_OPERATION_CREATE_TABLE_AUTOINC_FLAG)
+				if (!gda_server_operation_set_value_at (op, "TRUE", error, "/FIELDS_A/@COLUMN_AUTOINC/%d", i)){
+					g_object_unref (op);
+					return NULL;
+				}
+			if (flag & GDA_SERVER_OPERATION_CREATE_TABLE_UNIQUE_FLAG)
+				if(!gda_server_operation_set_value_at (op, "TRUE", error, "/FIELDS_A/@COLUMN_UNIQUE/%d", i)){
+					g_object_unref (op);
+					return NULL;
+				}
+			if (flag & GDA_SERVER_OPERATION_CREATE_TABLE_FKEY_FLAG) {
+				gint j;
+				gint fields;
+				gchar *fkey_table;
+				gchar *fkey_ondelete;
+				gchar *fkey_onupdate;
+
+				refs++;
+
+				fkey_table = va_arg (args, gchar*);
+				if (!gda_server_operation_set_value_at (op, fkey_table, error,
+								   "/FKEY_S/%d/FKEY_REF_TABLE", refs)){
+					g_object_unref (op);
+					return NULL;
+				}
+
+				fields = va_arg (args, gint);
+
+				for (j = 0; j < fields; j++) {
+					gchar *field, *rfield;
+
+					field = va_arg (args, gchar*);
+					if(!gda_server_operation_set_value_at (op, field, error,
+									   "/FKEY_S/%d/FKEY_FIELDS_A/@FK_FIELD/%d", refs, j)){
+						g_object_unref (op);
+						return NULL;
+					}
+
+					rfield = va_arg (args, gchar*);
+					if(!gda_server_operation_set_value_at (op, rfield, error,
+									   "/FKEY_S/%d/FKEY_FIELDS_A/@FK_REF_PK_FIELD/%d", refs, j)){
+						g_object_unref (op);
+						return NULL;
+					}
+				}
+
+				fkey_ondelete = va_arg (args, gchar*);
+				if (!gda_server_operation_set_value_at (op, fkey_ondelete, error,
+								   "/FKEY_S/%d/FKEY_ONDELETE", refs)){
+					g_object_unref (op);
+					return NULL;
+				}
+				fkey_onupdate = va_arg (args, gchar*);
+				if(!gda_server_operation_set_value_at (op, fkey_onupdate, error,
+								   "/FKEY_S/%d/FKEY_ONUPDATE", refs)){
+					g_object_unref (op);
+					return NULL;
+				}
+			}
+
+			i++;
+		}
+
+		va_end (args);
+
+		g_object_set_data_full (G_OBJECT (op), "_gda_connection", g_object_ref (cnc), g_object_unref);
+
+		return op;
+	}
+	else {
+		g_set_error (error, GDA_SERVER_OPERATION_ERROR, GDA_SERVER_OPERATION_OBJECT_NAME_ERROR,
+			     "%s", _("CREATE TABLE operation is not supported by the database server"));
+		return NULL;
+	}
+}
+
+
+/**
+ * gda_server_operation_perform_create_table:
+ * @op: a valid #GdaServerOperation
+ * @error: a place to store errors, or %NULL
+ *
+ * Performs a prepared #GdaServerOperation to create a table. This could perform
+ * an operation created by #gda_server_operation_prepare_create_table or any other using the
+ * the #GdaServerOperation API.
+ *
+ * Returns: TRUE if the table was created; FALSE and set @error otherwise
+ *
+ * Since: 4.2.3
+ */
+gboolean
+gda_server_operation_perform_create_table (GdaServerOperation *op, GError **error)
+{
+	GdaConnection *cnc;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+	g_return_val_if_fail (gda_server_operation_get_op_type (op) == GDA_SERVER_OPERATION_CREATE_TABLE, FALSE);
+
+	cnc = g_object_get_data (G_OBJECT (op), "_gda_connection");
+	if (cnc)
+		return gda_server_provider_perform_operation (gda_connection_get_provider (cnc), cnc, op, error);
+	else {
+		g_warning ("Could not find operation's associated connection, "
+			   "did you use gda_connection_prepare_create_table() ?");
+		return FALSE;
+	}
+}
+
+/**
+ * gda_server_operation_prepare_drop_table:
+ * @cnc: an opened connection
+ * @table_name: name of the table to drop
+ * @error: a place to store errors, or %NULL
+ *
+ * This is just a convenient function to create a #GdaServerOperation to drop a
+ * table in an opened connection.
+ *
+ * Returns: (transfer full) (allow-none): a new #GdaServerOperation or %NULL if couldn't create the opereration.
+ *
+ * Since: 4.2.3
+ */
+GdaServerOperation*
+gda_server_operation_prepare_drop_table (GdaConnection *cnc, const gchar *table_name, GError **error)
+{
+	GdaServerOperation *op;
+	GdaServerProvider *server;
+
+	server = gda_connection_get_provider (cnc);
+
+	op = gda_server_provider_create_operation (server, cnc,
+						   GDA_SERVER_OPERATION_DROP_TABLE, NULL, error);
+
+	if (GDA_IS_SERVER_OPERATION (op)) {
+		g_return_val_if_fail (table_name != NULL
+				      || GDA_IS_CONNECTION (cnc)
+				      || !gda_connection_is_opened (cnc), NULL);
+
+		if (gda_server_operation_set_value_at (op, table_name,
+						       error, "/TABLE_DESC_P/TABLE_NAME")) {
+			g_object_set_data_full (G_OBJECT (op), "_gda_connection", g_object_ref (cnc), g_object_unref);
+			return op;
+		}
+		else
+			return NULL;
+	}
+	else
+		return NULL;
+}
+
+
+/**
+ * gda_server_operation_perform_drop_table:
+ * @op: a #GdaServerOperation object
+ * @error: a place to store errors, or %NULL
+ *
+ * This is just a convenient function to perform a drop a table operation.
+ *
+ * Returns: TRUE if the table was dropped
+ *
+ * Since: 4.2.3
+ */
+gboolean
+gda_server_operation_perform_drop_table (GdaServerOperation *op, GError **error)
+{
+	GdaConnection *cnc;
+
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+	g_return_val_if_fail (gda_server_operation_get_op_type (op) == GDA_SERVER_OPERATION_DROP_TABLE, FALSE);
+
+	cnc = g_object_get_data (G_OBJECT (op), "_gda_connection");
+	if (cnc)
+		return gda_server_provider_perform_operation (gda_connection_get_provider (cnc), cnc, op, error);
+	else {
+		g_warning ("Could not find operation's associated connection, "
+			   "did you use gda_server_operation_prepare_drop_table() ?");
+		return FALSE;
+	}
+}
