@@ -108,7 +108,8 @@ gda_statement_get_type (void)
 			NULL,
 			sizeof (GdaStatement),
 			0,
-			(GInstanceInitFunc) gda_statement_init
+			(GInstanceInitFunc) gda_statement_init,
+			0
 		};
 		
 		g_static_mutex_lock (&registering);
@@ -192,6 +193,11 @@ gda_statement_new (void)
 	return GDA_STATEMENT (obj);
 }
 
+const GdaSqlStatement *
+_gda_statement_get_internal_struct (GdaStatement *stmt)
+{
+	return stmt->priv->internal_struct;
+}
 
 /**
  * gda_statement_copy:
@@ -199,7 +205,7 @@ gda_statement_new (void)
  * 
  * Copy constructor
  *
- * Returns: a the new copy of @orig
+ * Returns: (transfer full): a the new copy of @orig
  */
 GdaStatement *
 gda_statement_copy (GdaStatement *orig)
@@ -688,7 +694,7 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 		if (context->provider)
 			dh = gda_server_provider_get_data_handler_g_type (context->provider, context->cnc, G_VALUE_TYPE (value));
 		else  			
-			dh = gda_get_default_handler (G_VALUE_TYPE (value));
+			dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 
 		if (!dh) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
@@ -704,8 +710,8 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 /**
  * gda_statement_to_sql_extended:
  * @stmt: a #GdaStatement object
- * @cnc: a #GdaConnection object, or %NULL
- * @params: parameters contained in a single #GdaSet object
+ * @cnc: (allow-none): a #GdaConnection object, or %NULL
+ * @params: (allow-none): parameters contained in a single #GdaSet object, or %NULL
  * @flags: a set of flags to control the rendering
  * @params_used: (element-type GdaHolder) (out) (transfer container) (allow-none):a place to store the list of actual #GdaHolder objects in @params used to do the rendering, or %NULL
  * @error: a place to store errors, or %NULL
@@ -716,7 +722,7 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
  * it may include some SQL tweaks and/or proprietary extensions specific to the database engine used by @cnc):
  * in this case the result is similar to calling gda_connection_statement_to_sql().
  *
- * Returns: a new string if no error occurred
+ * Returns: (transfer full): a new string if no error occurred
  */
 gchar *
 gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *params,
@@ -861,8 +867,10 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 			}
 			g_string_append_c (string, ')');
 		}
+
+		if (!stmt->fields_list && !stmt->values_list)
+			g_string_append (string, " DEFAULT VALUES");
 	}
-	
 
 	str = string->str;
 	g_string_free (string, FALSE);
@@ -1416,17 +1424,25 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->cond) {
-		str = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
-		if (!str) goto err;
+		gchar *tmp;
+		tmp = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
+		if (!tmp) goto err;
+		str = g_strconcat ("(", tmp, ")", NULL);
+		g_free (tmp);
 	}
 	else if (expr->select) {
 		gchar *str1;
-		str1 = context->render_select (GDA_SQL_ANY_PART (expr->select), context, error);
+		if (GDA_SQL_ANY_PART (expr->select)->type == GDA_SQL_ANY_STMT_SELECT)
+			str1 = context->render_select (GDA_SQL_ANY_PART (expr->select), context, error);
+		else if (GDA_SQL_ANY_PART (expr->select)->type == GDA_SQL_ANY_STMT_COMPOUND)
+			str1 = context->render_compound (GDA_SQL_ANY_PART (expr->select), context, error);
+		else
+			g_assert_not_reached ();
 		if (!str1) goto err;
 
 		if (! GDA_SQL_ANY_PART (expr)->parent ||
 		    (GDA_SQL_ANY_PART (expr)->parent->type != GDA_SQL_ANY_SQL_FUNCTION)) {
-			str = g_strdup_printf ("(%s)", str1);
+			str = g_strconcat ("(", str1, ")", NULL);
 			g_free (str1);
 		}
 		else
