@@ -1,5 +1,5 @@
-/* GDA common library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+/* GDA library
+ * Copyright (C) 1998 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *	Michael Lausch <michael@lausch.at>
@@ -73,7 +73,7 @@ set_from_string (GValue *value, const gchar *as_string)
 		}
 		else {
 			gint i;
-			i = atoi (as_string);
+			i = atoi (as_string); /* Flawfinder: ignore */
 			g_value_set_boolean (value, i ? TRUE : FALSE);
 			retval = TRUE;
 		}
@@ -308,13 +308,14 @@ gda_binary_get_type (void)
 }
 
 /**
- * gda_binary_copy
+ * gda_binary_copy:
  * @boxed: source to get a copy from.
  *
  * Creates a new #GdaBinary structure from an existing one.
 
- * Returns: a newly allocated #GdaBinary which contains a copy of
- * information in @boxed.
+ * Returns: (transfer full): a newly allocated #GdaBinary which contains a copy of information in @boxed.
+ *
+ * Free-function: gda_binary_free
  */
 gpointer
 gda_binary_copy (gpointer boxed)
@@ -332,8 +333,8 @@ gda_binary_copy (gpointer boxed)
 }
 
 /**
- * gda_binary_free
- * @boxed: #GdaBinary to free.
+ * gda_binary_free:
+ * @boxed: (transfer full): #GdaBinary to free.
  *
  * Deallocates all memory associated to the given #GdaBinary.
  */
@@ -405,13 +406,14 @@ gda_blob_get_type (void)
 }
 
 /**
- * gda_blob_copy
+ * gda_blob_copy:
  * @boxed: source to get a copy from.
  *
  * Creates a new #GdaBlob structure from an existing one.
 
- * Returns: a newly allocated #GdaBlob which contains a copy of
- * information in @boxed.
+ * Returns: (transfer full): a newly allocated #GdaBlob which contains a copy of information in @boxed.
+ *
+ * Free-function: gda_blob_free
  */
 gpointer
 gda_blob_copy (gpointer boxed)
@@ -432,8 +434,8 @@ gda_blob_copy (gpointer boxed)
 }
 
 /**
- * gda_blob_free
- * @boxed: #GdaBlob to free.
+ * gda_blob_free:
+ * @boxed: (transfer full): #GdaBlob to free.
  *
  * Deallocates all memory associated to the given #GdaBlob.
  */
@@ -452,7 +454,7 @@ gda_blob_free (gpointer boxed)
 }
 
 /**
- * gda_blob_set_op 
+ * gda_blob_set_op:
  * @blob: a #GdaBlob value
  * @op: a #GdaBlobOp object, or %NULL
  *
@@ -540,6 +542,11 @@ gda_geometricpoint_get_type (void)
 	return type;
 }
 
+/**
+ * gda_geometricpoint_copy:
+ *
+ * Returns: (transfer full):
+ */
 gpointer 
 gda_geometricpoint_copy (gpointer boxed)
 {
@@ -680,8 +687,13 @@ numeric_to_int (const GValue *src, GValue *dest)
 			  GDA_VALUE_HOLDS_NUMERIC (src));
 
 	numeric = gda_value_get_numeric (src);
-	if (numeric)
-		g_value_set_int (dest, atol (numeric->number));
+	if (numeric) {
+		glong tmp;
+		tmp = atol (numeric->number); /* Flawfinder: ignore */
+		if ((tmp < G_MININT) || (tmp > G_MAXINT))
+			g_warning ("Integer overflow for value %ld", tmp);
+		g_value_set_int (dest, tmp);
+	}
 	else
 		g_value_set_int (dest, 0);
 }
@@ -695,8 +707,13 @@ numeric_to_uint (const GValue *src, GValue *dest)
 			  GDA_VALUE_HOLDS_NUMERIC (src));
 
 	numeric = gda_value_get_numeric (src);
-	if (numeric)
-		g_value_set_uint (dest, atol (numeric->number));
+	if (numeric) {
+		glong tmp;
+		tmp = atol (numeric->number); /* Flawfinder: ignore */
+		if ((tmp < 0) || (tmp > G_MAXUINT))
+			g_warning ("Unsigned integer overflow for value %ld", tmp);
+		g_value_set_uint (dest, tmp);
+	}
 	else
 		g_value_set_uint (dest, 0);
 }
@@ -711,7 +728,7 @@ numeric_to_boolean (const GValue *src, GValue *dest)
 
 	numeric = gda_value_get_numeric (src);
 	if (numeric)
-		g_value_set_boolean (dest, atoi (numeric->number));
+		g_value_set_boolean (dest, atoi (numeric->number)); /* Flawfinder: ignore */
 	else
 		g_value_set_boolean (dest, 0);
 }
@@ -738,13 +755,14 @@ gda_numeric_get_type (void)
 }
 
 /**
- * gda_numeric_copy
+ * gda_numeric_copy:
  * @boxed: source to get a copy from.
  *
  * Creates a new #GdaNumeric structure from an existing one.
 
- * Returns: a newly allocated #GdaNumeric which contains a copy of
- * information in @boxed.
+ * Returns: (transfer full): a newly allocated #GdaNumeric which contains a copy of information in @boxed.
+ *
+ * Free-function: gda_numeric_free
  */
 
 gpointer
@@ -764,8 +782,8 @@ gda_numeric_copy (gpointer boxed)
 }
 
 /**
- * gda_numeric_free
- * @boxed: a #GdaNumeric pointer
+ * gda_numeric_free:
+ * @boxed: (transfer full): a #GdaNumeric pointer
  *
  * Deallocates all memory associated to the given @boxed
  */
@@ -923,6 +941,11 @@ gda_time_get_type(void)
 	return type;
 }
 
+/**
+ * gda_time_copy:
+ *
+ * Returns: (transfer full):
+ */
 gpointer
 gda_time_copy (gpointer boxed)
 {
@@ -949,10 +972,10 @@ gda_time_free (gpointer boxed)
 }
 
 /**
- * gda_time_valid
+ * gda_time_valid:
  * @time: a #GdaTime value to check if it is valid
  *
- * Returns: #TRUE if #GdaTime is valid; FALSE otherwise.
+ * Returns: #TRUE if #GdaTime is valid; %FALSE otherwise.
  *
  * Since: 4.2
  */
@@ -961,9 +984,9 @@ gda_time_valid (const GdaTime *time)
 {
 	g_return_val_if_fail (time, FALSE);
 
-	if (time->hour < 0 || time->hour > 24 ||
-	    time->minute < 0 || time->minute > 59 ||
-	    time->second < 0 || time->second > 59)
+	if (time->hour > 24 ||
+	    time->minute > 59 ||
+	    time->second > 59)
 		return FALSE;
 	else
 		return TRUE;
@@ -978,37 +1001,14 @@ static void
 string_to_timestamp (const GValue *src, GValue *dest)
 {
 	/* FIXME: add more checks*/
-	GdaTimestamp *timestamp;
-	const gchar *as_string;
+	GdaTimestamp timestamp;
 	
 	g_return_if_fail (G_VALUE_HOLDS_STRING (src) &&
 			  GDA_VALUE_HOLDS_TIMESTAMP (dest));
-	
-	timestamp = g_new0 (GdaTimestamp, 1);
-	
-	as_string = g_value_get_string (src);
-	
-	timestamp->year = atoi (as_string);
-	as_string += 5;
-	timestamp->month = atoi (as_string);
-	as_string += 3;
-	timestamp->day = atoi (as_string);
-	as_string += 3;
-	timestamp->hour = atoi (as_string);
-	as_string += 3;
-	timestamp->minute = atoi (as_string);
-	as_string += 3;
-	timestamp->second = atoi (as_string);
-	if (strlen(as_string)>=3) {
-		as_string += 3;
-		timestamp->fraction = atol (as_string) * 10; /* I have only hundredths of second */
-		if (strlen(as_string)>=3) {
-			as_string += 3;
-			timestamp->timezone = atol (as_string) * 60 * 60;
-		}
-	}
-	gda_value_set_timestamp (dest, timestamp);
-	g_free (timestamp);
+
+	if (! gda_parse_iso8601_timestamp (&timestamp, g_value_get_string (src)))
+		g_warning ("Can't convert '%s' to a timestamp", g_value_get_string (src));
+	gda_value_set_timestamp (dest, &timestamp);
 }
 
 static void 
@@ -1056,6 +1056,11 @@ gda_timestamp_get_type (void)
 	return type;
 }
 
+/**
+ * gda_timestamp_copy:
+ *
+ * Returns: (transfer full):
+ */
 gpointer 
 gda_timestamp_copy (gpointer boxed)
 {
@@ -1084,10 +1089,10 @@ gda_timestamp_free (gpointer boxed)
 }
 
 /**
- * gda_timestamp_valid
+ * gda_timestamp_valid:
  * @timestamp: a #GdaTimestamp value to check if it is valid
  *
- * Returns: #TRUE if #GdaTimestamp is valid; FALSE otherwise.
+ * Returns: #TRUE if #GdaTimestamp is valid; %FALSE otherwise.
  *
  * Since: 4.2
  */
@@ -1110,22 +1115,23 @@ gda_timestamp_valid (const GdaTimestamp *timestamp)
 	g_date_free (gdate);
 
 	/* check the time part */
-	if (timestamp->hour < 0 || timestamp->hour > 24 ||
-	    timestamp->minute < 0 || timestamp->minute > 59 ||
-	    timestamp->second < 0 || timestamp->second > 59)
+	if (timestamp->hour > 24 ||
+	    timestamp->minute > 59 ||
+	    timestamp->second > 59)
 		return FALSE;
 	else
 		return TRUE;
 }
 
 /**
- * gda_value_new
+ * gda_value_new:
  * @type: the new value type.
  *
  * Makes a new #GValue of type @type.
  *
- * Returns: the newly created #GValue with the specified @type. 
- *	You need to set the value in the returned GValue.
+ * Returns: (transfer full): the newly created #GValue with the specified @type. You need to set the value in the returned GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new (GType type)
@@ -1139,13 +1145,15 @@ gda_value_new (GType type)
 }
 
 /**
- * gda_value_new_binary
+ * gda_value_new_binary:
  * @val: value to set for the new #GValue.
  * @size: the size of the memory pool pointer to by @val.
  *
  * Makes a new #GValue of type #GDA_TYPE_BINARY with value @val.
  *
- * Returns: the newly created #GValue.
+ * Returns: (transfer full): the newly created #GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_binary (const guchar *val, glong size)
@@ -1166,13 +1174,15 @@ gda_value_new_binary (const guchar *val, glong size)
 }
 
 /**
- * gda_value_new_blob
+ * gda_value_new_blob:
  * @val: value to set for the new #GValue.
  * @size: the size of the memory pool pointer to by @val.
  *
  * Makes a new #GValue of type #GDA_TYPE_BLOB with the data contained by @val.
  *
- * Returns: the newly created #GValue.
+ * Returns: (transfer full): the newly created #GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_blob (const guchar *val, glong size)
@@ -1184,7 +1194,7 @@ gda_value_new_blob (const guchar *val, glong size)
 	blob = g_new0 (GdaBlob, 1);
 	bin = (GdaBinary*)(blob);
 	bin->data = g_new (guchar, size);
-        memcpy ((gpointer) bin->data, (gpointer) val, size);
+        memcpy ((gpointer) bin->data, (gpointer) val, size); /* Flawfinder: ignore */
         bin->binary_length = size;
 	blob->op = NULL;
 
@@ -1196,13 +1206,15 @@ gda_value_new_blob (const guchar *val, glong size)
 }
 
 /**
- * gda_value_new_blob
- * @val: value to set for the new #GValue.
- * @size: the size of the memory pool pointer to by @val.
+ * gda_value_new_blob_from_file:
+ * @filename: name of the file to manipulate
  *
- * Makes a new #GValue of type #GDA_TYPE_BLOB with the data contained by @val.
+ * Makes a new #GValue of type #GDA_TYPE_BLOB interfacing with the contents of the file
+ * named @filename
  *
- * Returns: the newly created #GValue.
+ * Returns: (transfer full): the newly created #GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_blob_from_file (const gchar *filename)
@@ -1221,13 +1233,15 @@ gda_value_new_blob_from_file (const gchar *filename)
 }
 
 /**
- * gda_value_new_timestamp_from_timet
+ * gda_value_new_timestamp_from_timet:
  * @val: value to set for the new #GValue.
  *
  * Makes a new #GValue of type #GDA_TYPE_TIMESTAMP with value @val
  * (of type time_t).
  *
- * Returns: the newly created #GValue.
+ * Returns: (transfer full): the newly created #GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_timestamp_from_timet (time_t val)
@@ -1255,7 +1269,7 @@ gda_value_new_timestamp_from_timet (time_t val)
 }
 
 /**
- * gda_value_new_from_string
+ * gda_value_new_from_string:
  * @as_string: stringified representation of the value.
  * @type: the new value type.
  *
@@ -1266,8 +1280,9 @@ gda_value_new_timestamp_from_timet (time_t val)
  * This function is typically used when reading configuration files or other non-user input that should be locale 
  * independent.
  *
- * Returns: the newly created #GValue or %NULL if the string representation
- * cannot be converted to the specified @type.
+ * Returns: (transfer full): the newly created #GValue or %NULL if the string representation cannot be converted to the specified @type.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_from_string (const gchar *as_string, GType type)
@@ -1286,7 +1301,7 @@ gda_value_new_from_string (const gchar *as_string, GType type)
 }
 
 /**
- * gda_value_new_from_xml
+ * gda_value_new_from_xml:
  * @node: an XML node representing the value.
  *
  * Creates a GValue from an XML representation of it. That XML
@@ -1298,7 +1313,9 @@ gda_value_new_from_string (const gchar *as_string, GType type)
  * This function is typically used when reading configuration files or other non-user input that should be locale 
  * independent.
  *
- * Returns: the newly created #GValue.
+ * Returns: (transfer full): the newly created #GValue.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
 gda_value_new_from_xml (const xmlNodePtr node)
@@ -1327,8 +1344,8 @@ gda_value_new_from_xml (const xmlNodePtr node)
 }
 
 /**
- * gda_value_free
- * @value: the resource to free.
+ * gda_value_free:
+ * @value: (transfer full): the resource to free.
  *
  * Deallocates all memory associated to a #GValue.
  */
@@ -1366,13 +1383,12 @@ gda_value_reset_with_type (GValue *value, GType type)
 
 
 /**
- * gda_value_is_null
+ * gda_value_is_null:
  * @value: value to test.
  *
  * Tests if a given @value is of type #GDA_TYPE_NULL.
  *
- * Returns: a boolean that says whether or not @value is of type
- * #GDA_TYPE_NULL.
+ * Returns: a boolean that says whether or not @value is of type #GDA_TYPE_NULL.
  */
 gboolean
 gda_value_is_null (const GValue *value)
@@ -1382,11 +1398,10 @@ gda_value_is_null (const GValue *value)
 }
 
 /**
- * gda_value_is_number
+ * gda_value_is_number:
  * @value: a #GValue.
  *
- * Gets whether the value stored in the given #GValue is of
- * numeric type or not.
+ * Gets whether the value stored in the given #GValue is of numeric type or not.
  *
  * Returns: %TRUE if a number, %FALSE otherwise.
  */
@@ -1406,15 +1421,17 @@ gda_value_is_number (const GValue *value)
 }
 
 /**
- * gda_value_copy
+ * gda_value_copy:
  * @value: value to get a copy from.
  *
  * Creates a new #GValue from an existing one.
  * 
- * Returns: a newly allocated #GValue with a copy of the data in @value.
+ * Returns: (transfer full): a newly allocated #GValue with a copy of the data in @value.
+ *
+ * Free-function: gda_value_free
  */
 GValue *
-gda_value_copy (const  GValue *value)
+gda_value_copy (const GValue *value)
 {
 	GValue *copy;
 	
@@ -1431,10 +1448,10 @@ gda_value_copy (const  GValue *value)
 }
 
 /**
- * gda_value_get_binary
+ * gda_value_get_binary:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 G_CONST_RETURN GdaBinary *
 gda_value_get_binary (const GValue *value)
@@ -1451,7 +1468,7 @@ gda_value_get_binary (const GValue *value)
 
 
 /**
- * gda_value_set_binary
+ * gda_value_set_binary:
  * @value: a #GValue that will store @val.
  * @binary: a #GdaBinary structure with the data and its size to be stored in @value.
  *
@@ -1473,9 +1490,9 @@ gda_value_set_binary (GValue *value, const GdaBinary *binary)
 }
 
 /**
- * gda_value_take_binary
+ * gda_value_take_binary:
  * @value: a #GValue that will store @val.
- * @binary: a #GdaBinary structure with the data and its size to be stored in @value.
+ * @binary: (transfer full): a #GdaBinary structure with the data and its size to be stored in @value.
  *
  * Stores @val into @value, but on the contrary to gda_value_set_binary(), the @binary
  * argument is not copied, but used as-is and it should be considered owned by @value.
@@ -1492,7 +1509,7 @@ gda_value_take_binary (GValue *value, GdaBinary *binary)
 }
 
 /**
- * gda_value_set_blob
+ * gda_value_set_blob:
  * @value: a #GValue that will store @val.
  * @blob: a #GdaBlob structure with the data and its size to be stored in @value.
  *
@@ -1510,10 +1527,10 @@ gda_value_set_blob (GValue *value, const GdaBlob *blob)
 }
 
 /**
- * gda_value_get_blob
+ * gda_value_get_blob:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 G_CONST_RETURN GdaBlob *
 gda_value_get_blob (const GValue *value)
@@ -1529,9 +1546,9 @@ gda_value_get_blob (const GValue *value)
 }
 
 /**
- * gda_value_take_blob
+ * gda_value_take_blob:
  * @value: a #GValue that will store @val.
- * @blob: a #GdaBlob structure with the data and its size to be stored in @value.
+ * @blob: (transfer full): a #GdaBlob structure with the data and its size to be stored in @value.
  *
  * Stores @val into @value, but on the contrary to gda_value_set_blob(), the @blob
  * argument is not copied, but used as-is and it should be considered owned by @value.
@@ -1548,10 +1565,10 @@ gda_value_take_blob (GValue *value, GdaBlob *blob)
 }
 
 /**
- * gda_value_get_geometric_point
+ * gda_value_get_geometric_point:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 G_CONST_RETURN GdaGeometricPoint *
 gda_value_get_geometric_point (const GValue *value)
@@ -1562,7 +1579,7 @@ gda_value_get_geometric_point (const GValue *value)
 }
 
 /**
- * gda_value_set_geometric_point
+ * gda_value_set_geometric_point:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1580,10 +1597,10 @@ gda_value_set_geometric_point (GValue *value, const GdaGeometricPoint *val)
 }
 
 /**
- * gda_value_get_list
+ * gda_value_get_list:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 G_CONST_RETURN GdaValueList *
 gda_value_get_list (const GValue *value)
@@ -1594,7 +1611,7 @@ gda_value_get_list (const GValue *value)
 }
 
 /**
- * gda_value_set_list
+ * gda_value_set_list:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1614,7 +1631,7 @@ gda_value_set_list (GValue *value, const GdaValueList *val)
 }
 
 /**
- * gda_value_set_null
+ * gda_value_set_null:
  * @value: a #GValue that will store a value of type #GDA_TYPE_NULL.
  *
  * Sets the type of @value to #GDA_TYPE_NULL.
@@ -1631,7 +1648,7 @@ gda_value_set_null (GValue *value)
  * gda_value_get_numeric
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 G_CONST_RETURN GdaNumeric *
 gda_value_get_numeric (const GValue *value)
@@ -1642,7 +1659,7 @@ gda_value_get_numeric (const GValue *value)
 }
 
 /**
- * gda_value_set_numeric
+ * gda_value_set_numeric:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1660,7 +1677,7 @@ gda_value_set_numeric (GValue *value, const GdaNumeric *val)
 }
 
 /**
- * gda_value_get_short
+ * gda_value_get_short:
  * @value: a #GValue whose value we want to get.
  *
  * Returns: the value stored in @value.
@@ -1674,7 +1691,7 @@ gda_value_get_short (const GValue *value)
 }
 
 /**
- * gda_value_set_short
+ * gda_value_set_short:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1691,7 +1708,7 @@ gda_value_set_short (GValue *value, gshort val)
 }
 
 /**
- * gda_value_get_ushort
+ * gda_value_get_ushort:
  * @value: a #GValue whose value we want to get.
  *
  * Returns: the value stored in @value.
@@ -1705,7 +1722,7 @@ gda_value_get_ushort (const GValue *value)
 }
 
 /**
- * gda_value_set_ushort
+ * gda_value_set_ushort:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1723,10 +1740,10 @@ gda_value_set_ushort (GValue *value, gushort val)
 
 
 /**
- * gda_value_get_time
+ * gda_value_get_time:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 const GdaTime *
 gda_value_get_time (const GValue *value)
@@ -1737,7 +1754,7 @@ gda_value_get_time (const GValue *value)
 }
 
 /**
- * gda_value_set_time
+ * gda_value_set_time:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1755,10 +1772,10 @@ gda_value_set_time (GValue *value, const GdaTime *val)
 }
 
 /**
- * gda_value_get_timestamp
+ * gda_value_get_timestamp:
  * @value: a #GValue whose value we want to get.
  *
- * Returns: the value stored in @value.
+ * Returns: (transfer none): the value stored in @value.
  */
 const GdaTimestamp *
 gda_value_get_timestamp (const GValue *value)
@@ -1769,7 +1786,7 @@ gda_value_get_timestamp (const GValue *value)
 }
 
 /**
- * gda_value_set_timestamp
+ * gda_value_set_timestamp:
  * @value: a #GValue that will store @val.
  * @val: value to be stored in @value.
  *
@@ -1787,7 +1804,7 @@ gda_value_set_timestamp (GValue *value, const GdaTimestamp *val)
 }
 
 /**
- * gda_value_set_from_string
+ * gda_value_set_from_string:
  * @value: a #GValue that will store @val.
  * @as_string: the stringified representation of the value.
  * @type: the type of the value
@@ -1825,7 +1842,7 @@ gda_value_set_from_string (GValue *value,
 }
 
 /**
- * gda_value_set_from_value
+ * gda_value_set_from_value:
  * @value: a #GValue.
  * @from: the value to copy from.
  *
@@ -1858,7 +1875,7 @@ gda_value_set_from_value (GValue *value, const GValue *from)
 }
 
 /**
- * gda_value_stringify
+ * gda_value_stringify:
  * @value: a #GValue.
  *
  * Converts a GValue to its string representation which is a human readable value. Note that the
@@ -1868,8 +1885,7 @@ gda_value_set_from_value (GValue *value, const GValue *from)
  *
  * Dates are converted in a YYYY-MM-DD format.
  *
- * Returns: a new string, or %NULL if the conversion cannot be done. Free the value with a g_free() when you've finished
- * using it. 
+ * Returns: (transfer full): a new string, or %NULL if the conversion cannot be done. Free the value with a g_free() when you've finished using it. 
  */
 gchar *
 gda_value_stringify (const GValue *value)
@@ -1932,7 +1948,7 @@ gda_value_stringify (const GValue *value)
 }
 	
 /**
- * gda_value_differ
+ * gda_value_differ:
  * @value1: a #GValue to compare.
  * @value2: the other #GValue to be compared to @value1.
  *
@@ -2098,7 +2114,7 @@ gda_value_differ (const GValue *value1, const GValue *value2)
 }
 
 /**
- * gda_value_compare
+ * gda_value_compare:
  * @value1: a #GValue to compare (not %NULL)
  * @value2: the other #GValue to be compared to @value1 (not %NULL)
  *
@@ -2411,12 +2427,12 @@ to_string (const GValue *value)
 
 
 /**
- * gda_value_to_xml
+ * gda_value_to_xml: (skip):
  * @value: a #GValue.
  *
  * Serializes the given #GValue to an XML node string.
  *
- * Returns: the XML node. Once not needed anymore, you should free it.
+ * Returns: (transfer full): the XML node. Once not needed anymore, you should free it.
  */
 xmlNodePtr
 gda_value_to_xml (const GValue *value)
@@ -2554,9 +2570,9 @@ gda_ushort_get_type (void) {
 #define MYMIN(a,b) (((b) > 0) ? ((a) < (b) ? (a) : (b)) : (a))
 
 /**
- * gda_binary_to_string
+ * gda_binary_to_string:
  * @bin: a correctly filled @GdaBinary structure
- * @maxlen: a maximum len used to truncate, or 0 for no maximum length
+ * @maxlen: a maximum len used to truncate, or %0 for no maximum length
  *
  * Converts all the non printable characters of bin->data into the "\xyz" representation
  * where "xyz" is the octal representation of the byte, and the '\' (backslash) character
@@ -2569,7 +2585,7 @@ gda_ushort_get_type (void) {
  * chunk. Note that this function is internally called when transforming a binary value to
  * a string for example when using g_value_transform() or gda_value_stringify().
  *
- * Returns: a new string from @bin
+ * Returns: (transfer full): a new string from @bin
  */
 gchar *
 gda_binary_to_string (const GdaBinary *bin, guint maxlen)
@@ -2579,7 +2595,7 @@ gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 	gulong realsize = MYMIN (bin->binary_length, maxlen);
 
 	gchar *retval;
-	glong offset = 0;
+	gulong offset = 0;
 
 	if (!bin->data || (bin->binary_length == 0))
 		return g_strdup ("");
@@ -2637,14 +2653,14 @@ gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 }
 
 /**
- * gda_string_to_binary
+ * gda_string_to_binary:
  * @str: a string to convert
  *
  * Performs the reverse of gda_binary_to_string() (note that for any "\xyz" succession
  * of 4 characters where "xyz" represents a valid octal value, the resulting read value will
  * be modulo 256)
  *
- * Returns: a new #GdaBinary if no error were found in @str, or NULL otherwise
+ * Returns: (transfer full): a new #GdaBinary if no error were found in @str, or NULL otherwise
  */
 GdaBinary *
 gda_string_to_binary (const gchar *str)
@@ -2705,7 +2721,7 @@ gda_string_to_binary (const gchar *str)
 }
 
 /**
- * gda_blob_to_string
+ * gda_blob_to_string:
  * @blob: a correctly filled @GdaBlob structure
  * @maxlen: a maximum len used to truncate, or 0 for no maximum length
  *
@@ -2713,7 +2729,7 @@ gda_string_to_binary (const gchar *str)
  * where xxx is the octal representation of the byte, and the '\' (backslash) character
  * is converted to "\\".
  *
- * Returns: a new string from @blob
+ * Returns: (transfer full): a new string from @blob
  */
 gchar *
 gda_blob_to_string (GdaBlob *blob, guint maxlen)
@@ -2726,12 +2742,12 @@ gda_blob_to_string (GdaBlob *blob, guint maxlen)
 }
 
 /**
- * gda_string_to_blob
+ * gda_string_to_blob:
  * @str: a string to convert
  *
  * Performs the reverse of gda_blob_to_string().
  *
- * Returns: a new #gdaBlob if no error were found in @str, or NULL otherwise
+ * Returns: (transfer full): a new #gdaBlob if no error were found in @str, or NULL otherwise
  */
 GdaBlob *
 gda_string_to_blob (const gchar *str)
