@@ -43,7 +43,7 @@ static gboolean     gda_handler_bin_accepts_g_type       (GdaDataHandler * dh, G
 static const gchar *gda_handler_bin_get_descr              (GdaDataHandler *dh);
 
 struct  _GdaHandlerBinPriv {
-	gchar             *detailled_descr;
+	gchar             *detailed_descr;
 	guint              nb_g_types;
 	GType             *valid_g_types;
 };
@@ -67,7 +67,8 @@ gda_handler_bin_get_type (void)
 			NULL,
 			sizeof (GdaHandlerBin),
 			0,
-			(GInstanceInitFunc) gda_handler_bin_init
+			(GInstanceInitFunc) gda_handler_bin_init,
+			0
 		};		
 
 		static const GInterfaceInfo data_entry_info = {
@@ -114,7 +115,7 @@ gda_handler_bin_init (GdaHandlerBin * hdl)
 {
 	/* Private structure */
 	hdl->priv = g_new0 (GdaHandlerBinPriv, 1);
-	hdl->priv->detailled_descr = _("Binary handler");
+	hdl->priv->detailed_descr = _("Binary handler");
 	hdl->priv->nb_g_types = 2;
 	hdl->priv->valid_g_types = g_new0 (GType, hdl->priv->nb_g_types);
 	hdl->priv->valid_g_types[0] = GDA_TYPE_BINARY;
@@ -147,11 +148,11 @@ gda_handler_bin_dispose (GObject   * object)
 }
 
 /**
- * gda_handler_bin_new
+ * gda_handler_bin_new:
  *
  * Creates a data handler for binary values
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_bin_new (void)
