@@ -1,6 +1,6 @@
 /* gda-handler-time.c
  *
- * Copyright (C) 2003 - 2008 Vivien Malerba
+ * Copyright (C) 2003 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -20,6 +20,7 @@
 
 #include "gda-handler-time.h"
 #include <string.h>
+#include <ctype.h>
 #include <glib/gi18n-lib.h>
 
 static void gda_handler_time_class_init (GdaHandlerTimeClass *class);
@@ -51,8 +52,6 @@ static GValue      *gda_handler_time_get_value_from_str     (GdaDataHandler *dh,
 
 static GValue      *gda_handler_time_get_sane_init_value    (GdaDataHandler * dh, GType type);
 
-static guint        gda_handler_time_get_nb_g_types       (GdaDataHandler *dh);
-static GType        gda_handler_time_get_g_type_index     (GdaDataHandler *dh, guint index);
 static gboolean     gda_handler_time_accepts_g_type       (GdaDataHandler * dh, GType type);
 
 static const gchar *gda_handler_time_get_descr              (GdaDataHandler *dh);
@@ -67,7 +66,7 @@ typedef struct _LocaleSetting {
 static gchar *render_date_locale (const GDate *date, LocaleSetting *locale);
 
 struct  _GdaHandlerTimePriv {
-	gchar          *detailled_descr;
+	gchar          *detailed_descr;
 	guint           nb_g_types;
 	GType          *valid_g_types;
 
@@ -95,7 +94,8 @@ gda_handler_time_get_type (void)
 			NULL,
 			sizeof (GdaHandlerTime),
 			0,
-			(GInstanceInitFunc) gda_handler_time_init
+			(GInstanceInitFunc) gda_handler_time_init,
+			0
 		};		
 
 		static const GInterfaceInfo data_entry_info = {
@@ -122,9 +122,7 @@ gda_handler_time_data_handler_init (GdaDataHandlerIface *iface)
 	iface->get_value_from_sql = gda_handler_time_get_value_from_sql;
 	iface->get_value_from_str = gda_handler_time_get_value_from_str;
 	iface->get_sane_init_value = gda_handler_time_get_sane_init_value;
-	iface->get_nb_g_types = gda_handler_time_get_nb_g_types;
 	iface->accepts_g_type = gda_handler_time_accepts_g_type;
-	iface->get_g_type_index = gda_handler_time_get_g_type_index;
 	iface->get_descr = gda_handler_time_get_descr;
 }
 
@@ -144,7 +142,7 @@ gda_handler_time_init (GdaHandlerTime *hdl)
 {
 	/* Private structure */
 	hdl->priv = g_new0 (GdaHandlerTimePriv, 1);
-	hdl->priv->detailled_descr = _("Time and Date handler");
+	hdl->priv->detailed_descr = _("Time and Date handler");
 	hdl->priv->nb_g_types = 3;
 	hdl->priv->valid_g_types = g_new0 (GType, 7);
 	hdl->priv->valid_g_types[0] = G_TYPE_DATE;
@@ -198,11 +196,11 @@ gda_handler_time_dispose (GObject *object)
 }
 
 /**
- * gda_handler_time_new
+ * gda_handler_time_new:
  *
  * Creates a data handler for time values
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new (void)
@@ -216,12 +214,12 @@ gda_handler_time_new (void)
 }
 
 /**
- * gda_handler_time_new_no_locale
+ * gda_handler_time_new_no_locale:
  *
  * Creates a data handler for time values, but using the default C locale
  * instead of the current user locale.
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new_no_locale (void)
@@ -236,18 +234,18 @@ gda_handler_time_new_no_locale (void)
 /**
  * gda_handler_time_set_sql_spec
  * @dh: a #GdaHandlerTime object
- * @first:
- * @sec:
- * @third:
- * @separator:
+ * @first: what comes first in the date representation
+ * @sec: what comes second in the date representation
+ * @third: what comes third in the date representation
+ * @separator: separator character used between year, month and day
  * @twodigits_years: TRUE if year part of date must be rendered on 2 digits
  *
  * Specifies the SQL output style of the @dh data handler. The general format is "FIRSTsSECsTHIRD"
  * where FIRST, SEC and THIRD are specified by @first, @sec and @trird and 's' is the separator,
  * specified by @separator.
  *
- * The default implementation is FIRST=G_DATE_MONTH, SEC=G_DATE_DAY and THIRD=G_DATE_YEAR (the year is
- * rendered on 4 digits) and the separator is '-'
+ * The default implementation is @first=G_DATE_MONTH, @sec=G_DATE_DAY and @third=G_DATE_YEAR
+ * (the year is rendered on 4 digits) and the separator is '-'
  */
 void
 gda_handler_time_set_sql_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec,
@@ -266,6 +264,41 @@ gda_handler_time_set_sql_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec
 	dh->priv->sql_locale->separator = separator;
 }
 
+/**
+ * gda_handler_time_set_str_spec
+ * @dh: a #GdaHandlerTime object
+ * @first: what comes first in the date representation
+ * @sec: what comes second in the date representation
+ * @third: what comes third in the date representation
+ * @separator: separator character used between year, month and day
+ * @twodigits_years: TRUE if year part of date must be rendered on 2 digits
+ *
+ * Specifies the human readable output style of the @dh data handler.
+ * The general format is "FIRSTsSECsTHIRD"
+ * where FIRST, SEC and THIRD are specified by @first, @sec and @trird and 's' is the separator,
+ * specified by @separator.
+ *
+ * The default implementation depends on the current locale, except if @dh was created
+ * using gda_handler_time_new_no_locale().
+ *
+ * Since: 4.2.1
+ */
+void
+gda_handler_time_set_str_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec,
+				GDateDMY third, gchar separator, gboolean twodigits_years)
+{
+	g_return_if_fail (GDA_IS_HANDLER_TIME (dh));
+	g_return_if_fail (dh->priv);
+	g_return_if_fail (first != sec);
+	g_return_if_fail (sec != third);
+	g_return_if_fail (first != third);
+
+	dh->priv->str_locale->dmy_order[0] = first;
+	dh->priv->str_locale->dmy_order[1] = sec;
+	dh->priv->str_locale->dmy_order[2] = third;
+	dh->priv->str_locale->twodigit_years = twodigits_years;
+	dh->priv->str_locale->separator = separator;
+}
 
 static void
 handler_compute_locale (GdaHandlerTime *hdl)
@@ -277,6 +310,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 
 	date = g_date_new_dmy (4, 7, 1976); /* Same date used by GLib */
 	g_date_strftime (buf, 127, "%x", date);
+	g_date_free (date);
 
 	/* 1st number */
 	ptr = buf;
@@ -286,9 +320,10 @@ handler_compute_locale (GdaHandlerTime *hdl)
 	if (*ptr) {
 		hdl->priv->str_locale->separator = *ptr;
 		*ptr = 0;
-		nums[0] = atoi (numstart);
+		nums[0] = atoi (numstart); /* Flawfinder: ignore */
 	}
-	else error = TRUE;
+	else
+		error = TRUE;
 
 	/* 2nd number */
 	if (!error) {
@@ -298,9 +333,10 @@ handler_compute_locale (GdaHandlerTime *hdl)
 			ptr++;
 		if (*ptr) {
 			*ptr = 0;
-			nums[1] = atoi (numstart);
+			nums[1] = atoi (numstart); /* Flawfinder: ignore */
 		}
-		else error = TRUE;
+		else
+			error = TRUE;
 	}
 
 	/* 3rd number */
@@ -310,12 +346,12 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		while (*ptr && g_ascii_isdigit (*ptr))
 			ptr++;
 		*ptr = 0;
-		nums[2] = atoi (numstart);
+		nums[2] = atoi (numstart); /* Flawfinder: ignore */
 	}
 	
 	/* computations */
 	if (!error) {
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 		gchar *strings[3];
 #endif
 		gint i;
@@ -344,7 +380,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		now_tm = localtime (&now);
 		hdl->priv->str_locale->current_offset = ((now_tm->tm_year + 1900) / 100) * 100;
 
-#ifdef GDA_DEBUG		
+#ifdef GDA_DEBUG_NO		
 		for (i=0; i<3; i++) {
 			switch (hdl->priv->str_locale->dmy_order[i]) {
 			case G_DATE_MONTH:
@@ -450,36 +486,36 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *hdl, const GValue
 
 /**
  * gda_handler_time_get_format
- * @hdl: a #GdaHandlerTime object
+ * @dh: a #GdaHandlerTime object
  * @type: the type of data being handled
  *
- * Get a string representing the locale-dependant way to enter a date/time/datetime, using
+ * Get a string representing the locale-dependent way to enter a date/time/datetime, using
  * a syntax suitable for the #GnomeDbFormatEntry widget
  *
  * Returns: a new string
  */
 gchar *
-gda_handler_time_get_format (GdaHandlerTime *hdl, GType type)
+gda_handler_time_get_format (GdaHandlerTime *dh, GType type)
 {
 	gchar *str;
 	GString *string;
 	gint i;
 
-	g_return_val_if_fail (GDA_IS_HANDLER_TIME (hdl), NULL);
-	g_return_val_if_fail (hdl->priv, NULL);
+	g_return_val_if_fail (GDA_IS_HANDLER_TIME (dh), NULL);
+	g_return_val_if_fail (dh->priv, NULL);
 
 	string = g_string_new ("");
 	if ((type == G_TYPE_DATE) || (type == GDA_TYPE_TIMESTAMP)) {
 		for (i=0; i<3; i++) {
 			if (i > 0)
-				g_string_append_c (string, hdl->priv->str_locale->separator);
-			switch (hdl->priv->str_locale->dmy_order[i]) {
+				g_string_append_c (string, dh->priv->str_locale->separator);
+			switch (dh->priv->str_locale->dmy_order[i]) {
 			case G_DATE_DAY:
 			case G_DATE_MONTH:
 				g_string_append (string, "00");
 				break;
 			case G_DATE_YEAR:
-				if (hdl->priv->str_locale->twodigit_years)
+				if (dh->priv->str_locale->twodigit_years)
 					g_string_append (string, "00");
 				else
 					g_string_append (string, "0000");
@@ -744,7 +780,8 @@ gda_handler_time_get_value_from_str (GdaDataHandler *iface, const gchar *sql, GT
 
 static gboolean make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, 
 				const gchar *value, LocaleSetting *locale);
-static gboolean make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *locale);
+static gboolean make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value,
+			   LocaleSetting *locale, const gchar **out_endptr);
 static gboolean make_time (GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value);
 static GValue *
 gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql, 
@@ -762,7 +799,7 @@ gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql,
 	g_return_val_if_fail (hdl->priv, NULL);
 
 	if (type == G_TYPE_DATE) {
-		if (make_date (hdl, &date, sql, locale)) {
+		if (make_date (hdl, &date, sql, locale, NULL)) {
 			value = g_value_init (g_new0 (GValue, 1), G_TYPE_DATE);
 			g_value_set_boxed (value, (gconstpointer) &date);
 		}
@@ -793,47 +830,69 @@ static gboolean
 make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, const gchar *value, LocaleSetting *locale)
 {
 	gboolean retval;
-	gchar *str, *ptr;
+	const gchar *end_ptr;
 	GDate vdate;
 	GdaTime vtime;
-	char *buff;
+	memset (&vtime, 0, sizeof (GdaTime));
+	vtime.timezone = GDA_TIMEZONE_INVALID;
+
+	retval = make_date (hdl, &vdate, value, locale, &end_ptr);
+	timestamp->day = vdate.day;
+	timestamp->month = vdate.month;
+	timestamp->year = vdate.year;
 
-	str = g_strdup (value);
-	ptr = strtok_r (str, " ", &buff);
-	retval = make_date (hdl, &vdate, ptr, locale);
 	if (retval) {
-		ptr = strtok_r (NULL, " ", &buff);
-		retval = make_time (hdl, &vtime, ptr);
-		if (retval) {
-			timestamp->day = vdate.day;
-			timestamp->month = vdate.month;
-			timestamp->year = vdate.year;
-
-			timestamp->hour = vtime.hour;
-			timestamp->minute = vtime.minute;
-			timestamp->second = vtime.second;
-			timestamp->fraction = vtime.fraction;
-			timestamp->timezone = vtime.timezone;
-		}
+		if (*end_ptr != ' ')
+			retval = FALSE;
+		else
+			retval = make_time (hdl, &vtime, end_ptr + 1);
 	}
-	g_free (str);
+
+	timestamp->hour = vtime.hour;
+	timestamp->minute = vtime.minute;
+	timestamp->second = vtime.second;
+	timestamp->fraction = vtime.fraction;
+	timestamp->timezone = vtime.timezone;
 
 	/*g_print ("Value #%s# => %d\n", value, retval);*/
 
 	return retval;
 }
 
-
-/* Makes a GDate from a string like "24-12-2003" */
 static gboolean
-make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *locale)
+get_uint_from_string (const gchar *str, guint16 *out_int)
+{
+	long int li;
+	char *endptr = NULL;
+	li = strtol (str, &endptr, 10);
+	if (!*endptr && (li >= 0) && (li <= G_MAXUINT16)) {
+		*out_int = (guint16) li;
+		return TRUE;
+	}
+	else {
+		*out_int = 0;
+		return FALSE;
+	}
+}
+
+/* Makes a GDate from a string like "24-12-2003"
+ * If @out_endptr is %NULL, then all the @value has to be consumed and there must not
+ * be any character left. If it's not %NULL, then it will point on the first unused character
+ * of @value.
+ */
+static gboolean
+make_date (G_GNUC_UNUSED GdaHandlerTime *hdl, GDate *date, const gchar *value,
+	   LocaleSetting *locale, const gchar **out_endptr)
 {
 	gboolean retval = TRUE;
-	gushort nums[3];
+	guint16 nums[3];
 	gboolean error = FALSE;
 	gchar *ptr, *numstart, *tofree;
 	gint i;
 
+	if (out_endptr)
+		*out_endptr = NULL;
+
 	if (!value)
 		return FALSE;
 
@@ -848,47 +907,70 @@ make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *
 		ptr++;
 	if ((ptr != numstart) && *ptr) {
 		*ptr = 0;
-		nums[0] = atoi (numstart);
+		if (! get_uint_from_string (numstart, &(nums[0])))
+			error = TRUE;
 	}
-	else error = TRUE;
+	else
+		error = TRUE;
 
 	/* 2nd number */
 	if (!error) {
-		ptr++;
-		numstart = ptr;
-		while (*ptr && g_ascii_isdigit (*ptr))
+		if (value [ptr-tofree] != locale->separator)
+			error = TRUE;
+		else {
 			ptr++;
-		if ((ptr != numstart) && *ptr) {
-			*ptr = 0;
-			nums[1] = atoi (numstart);
+			numstart = ptr;
+			while (*ptr && g_ascii_isdigit (*ptr))
+				ptr++;
+			if ((ptr != numstart) && *ptr) {
+				*ptr = 0;
+				if (! get_uint_from_string (numstart, &(nums[1])))
+					error = TRUE;
+			}
+			else
+				error = TRUE;
 		}
-		else error = TRUE;
 	}
 
 	/* 3rd number */
 	if (!error) {
-		ptr++;
-		numstart = ptr;
-		while (*ptr && g_ascii_isdigit (*ptr))
+		if (value [ptr-tofree] != locale->separator)
+			error = TRUE;
+		else {
 			ptr++;
-		*ptr = 0;
-		if (ptr != numstart)
-			nums[2] = atoi (numstart);
-		else
+			numstart = ptr;
+			while (*ptr && g_ascii_isdigit (*ptr))
+				ptr++;
+			*ptr = 0;
+			if (ptr != numstart) {
+				if (! get_uint_from_string (numstart, &(nums[2])))
+					error = TRUE;
+			}
+			else
+				error = TRUE;
+		}
+	}
+
+	/* test if there are some characters left */
+	if (!error) {
+		if (out_endptr)
+			*out_endptr = value + (ptr-tofree);
+		else if (value [ptr-tofree])
 			error = TRUE;
 	}
 
+	/* analyse what's parsed */
 	if (!error) {
 		for (i=0; i<3; i++) {
 			switch (locale->dmy_order[i]) {
 			case G_DATE_DAY:
-				if (g_date_valid_day (nums[i]))
+				if ((nums[i] <= G_MAXUINT8) && g_date_valid_day ((GDateDay) nums[i]))
 					g_date_set_day (date, nums[i]);
 				else
 					retval = FALSE;
 				break;
 			case G_DATE_MONTH:
-				if (g_date_valid_month (nums[i]))
+				if ((nums[i] <= 12) && g_date_valid_month ((GDateMonth) nums[i]))
 					g_date_set_month (date, nums[i]);
 				else
 					retval = FALSE;
@@ -922,86 +1004,103 @@ make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *
  * 123015-02
  * 123015.123
  * taken from libgda/gda-value.h
+ *
+ * Also works if there is only 0 or 1 digit instead of 2
  */
 static gboolean
-make_time (GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value)
+make_time (G_GNUC_UNUSED GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value)
 {
 	const gchar *ptr;
 
+	memset (timegda, 0, sizeof (GdaTime));
+	timegda->timezone = GDA_TIMEZONE_INVALID;
+
 	if (!value)
 		return FALSE;
 
 	/* hour */
-	timegda->fraction = 0;
-	timegda->timezone = GDA_TIMEZONE_INVALID;
 	ptr = value;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9'))
-		timegda->hour = (*ptr - '0') * 10 + *(ptr+1) - '0';
-	else
+	if (!isdigit (*ptr))
+		return FALSE;
+
+	timegda->hour = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->hour = timegda->hour * 10 + (*ptr - '0');
+		ptr++;
+	}
+	if (timegda->hour >= 24)
+		return FALSE;
+
+	if (*ptr == ':')
+		ptr++;
+	else if (!isdigit (*ptr))
 		return FALSE;
 
 	/* minute */
-	ptr += 2;
-	if (! *ptr)
+	timegda->minute = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->minute = timegda->minute * 10 + (*ptr - '0');
+		ptr++;
+	}
+	if (timegda->minute >= 60)
 		return FALSE;
 	if (*ptr == ':')
 		ptr++;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9'))
-		timegda->minute = (*ptr - '0') * 10 + *(ptr+1) - '0';
-	else
+	else if (!isdigit (*ptr))
 		return FALSE;
 
 	/* second */
-	ptr += 2;
-	timegda->second = 0;
-	if (! *ptr) {
-		if ((timegda->hour > 24) || (timegda->minute > 60))
-			return FALSE;
-		else
-			return TRUE;
-	}
-	if (*ptr == ':')
+	timegda->second = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->second = timegda->second * 10 + (*ptr - '0');
 		ptr++;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9'))
-		timegda->second = (*ptr - '0') * 10 + *(ptr+1) - '0';
-	
-	/* extra */
-	ptr += 2;
-	if (! *ptr) {
-		if ((timegda->hour > 24) || (timegda->minute > 60) || 
-		    (timegda->second > 60))
-			return FALSE;
-		else
-			return TRUE;
 	}
-
-	if (*ptr == '.') {
-		ptr ++;
-		while (*ptr && (*ptr >= '0') && (*ptr <= '9')) {
-			timegda->fraction = timegda->fraction * 10 + *ptr - '0';
-			ptr++;
-		}
-	}
-
-	if ((*ptr == '+') || (*ptr == '-')) {
-		glong sign = (*ptr == '+') ? 1 : -1;
-		ptr ++;
-		timegda->timezone = 0;
-		while (*ptr && (*ptr >= '0') && (*ptr <= '9')) {
-			timegda->timezone = timegda->timezone * 10 + sign * ((*ptr) - '0');
-			ptr++;
-		}
-		timegda->timezone *= 3600;
-	}
-	
-	/* checks */
-	if ((timegda->hour > 24) || (timegda->minute > 60) || (timegda->second > 60))
+	if (timegda->second >= 60)
 		return FALSE;
-	else
-		return TRUE;
+
+	/* fraction */
+	if (*ptr && (*ptr != '.') && (*ptr != '+') && (*ptr != '-'))
+		return FALSE;
+	if (*ptr == '.') {
+		ptr++;
+		if (!isdigit (*ptr))
+			return FALSE;
+		while (isdigit (*ptr)) {
+			unsigned long long lu;
+			lu = timegda->fraction * 10 + (*ptr - '0');
+			if (lu < G_MAXULONG)
+				timegda->fraction = lu;
+			else
+				return FALSE;
+			ptr++;
+		}
+	}
+	
+	/* timezone */
+	if ((*ptr == '-') || (*ptr == '+')) {
+		gint sign = (*ptr == '+') ? 1 : -1;
+		ptr++;
+		if (!isdigit (*ptr))
+			return FALSE;
+		timegda->timezone = sign * (*ptr - '0');
+		ptr++;
+		if (isdigit (*ptr)) {
+			timegda->timezone = timegda->minute * 10 + sign * (*ptr - '0');
+			ptr++;
+		}
+		if (*ptr)
+			return FALSE;
+		if ((timegda->timezone >= -24) && (timegda->timezone <= 24))
+			timegda->timezone *= 60*60;
+		else {
+			timegda->timezone = 0;
+			return FALSE;
+		}
+	}
+	return TRUE;
 }
 
 
@@ -1058,20 +1157,6 @@ gda_handler_time_get_sane_init_value (GdaDataHandler *iface, GType type)
 	return value;
 }
 
-
-static guint
-gda_handler_time_get_nb_g_types (GdaDataHandler *iface)
-{
-	GdaHandlerTime *hdl;
-
-	g_return_val_if_fail (iface && GDA_IS_HANDLER_TIME (iface), 0);
-	hdl = GDA_HANDLER_TIME (iface);
-	g_return_val_if_fail (hdl->priv, 0);
-
-	return hdl->priv->nb_g_types;
-}
-
-
 static gboolean
 gda_handler_time_accepts_g_type (GdaDataHandler *iface, GType type)
 {
@@ -1093,19 +1178,6 @@ gda_handler_time_accepts_g_type (GdaDataHandler *iface, GType type)
 	return found;
 }
 
-static GType
-gda_handler_time_get_g_type_index (GdaDataHandler *iface, guint index)
-{
-	GdaHandlerTime *hdl;
-
-	g_return_val_if_fail (iface && GDA_IS_HANDLER_TIME (iface), G_TYPE_INVALID);
-	hdl = GDA_HANDLER_TIME (iface);
-	g_return_val_if_fail (hdl->priv, G_TYPE_INVALID);
-	g_return_val_if_fail (index < hdl->priv->nb_g_types, G_TYPE_INVALID);
-
-	return hdl->priv->valid_g_types[index];
-}
-
 static const gchar *
 gda_handler_time_get_descr (GdaDataHandler *iface)
 {
