@@ -1,5 +1,5 @@
 /* GDA Capi provider
- * Copyright (C) 2008 The GNOME Foundation.
+ * Copyright (C) 2008 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      TO_ADD: your name and email
@@ -85,7 +85,7 @@ static gboolean            gda_capi_provider_rollback_savepoint (GdaServerProvid
 static gboolean            gda_capi_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
 							       const gchar *name, GError **error);
 
-/* information retreival */
+/* information retrieval */
 static const gchar        *gda_capi_provider_get_version (GdaServerProvider *provider);
 static gboolean            gda_capi_provider_supports_feature (GdaServerProvider *provider, GdaConnection *cnc,
 							       GdaConnectionFeature feature);
@@ -111,6 +111,9 @@ static GObject             *gda_capi_provider_statement_execute (GdaServerProvid
 								 GType *col_types, GdaSet **last_inserted_row, 
 								 guint *task_id, GdaServerProviderExecCallback async_cb, 
 								 gpointer cb_data, GError **error);
+static GdaSqlStatement     *gda_capi_statement_rewrite          (GdaServerProvider *provider, GdaConnection *cnc,
+								 GdaStatement *stmt, GdaSet *params, GError **error);
+
 
 /* distributed transactions */
 static gboolean gda_capi_provider_xa_start    (GdaServerProvider *provider, GdaConnection *cnc, 
@@ -185,9 +188,13 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
         provider_class->delete_savepoint = gda_capi_provider_delete_savepoint;
 
 	provider_class->create_parser = gda_capi_provider_create_parser;
-	provider_class->statement_to_sql = gda_capi_provider_statement_to_sql;
+	provider_class->statement_to_sql = NULL; /* don't use gda_capi_provider_statement_to_sql()
+						  * because it only calls gda_statement_to_sql_extended().
+						  * Set it to gda_capi_provider_statement_to_sql() if it does
+						  * not call calls gda_statement_to_sql_extended() */
 	provider_class->statement_prepare = gda_capi_provider_statement_prepare;
 	provider_class->statement_execute = gda_capi_provider_statement_execute;
+	provider_class->statement_rewrite = gda_capi_statement_rewrite;
 
 	provider_class->is_busy = NULL;
 	provider_class->cancel = NULL;
@@ -236,6 +243,10 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
 	provider_class->meta_funcs.routine_col = _gda_capi_meta_routine_col;
 	provider_class->meta_funcs._routine_par = _gda_capi_meta__routine_par;
 	provider_class->meta_funcs.routine_par = _gda_capi_meta_routine_par;
+	provider_class->meta_funcs._indexes_tab = _gda_capi_meta__indexes_tab;
+        provider_class->meta_funcs.indexes_tab = _gda_capi_meta_indexes_tab;
+        provider_class->meta_funcs._index_cols = _gda_capi_meta__index_cols;
+        provider_class->meta_funcs.index_cols = _gda_capi_meta_index_cols;
 
 	/* distributed transactions: if not supported, then provider_class->xa_funcs should be set to NULL */
 	provider_class->xa_funcs = g_new0 (GdaServerProviderXa, 1);
@@ -248,7 +259,7 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
 }
 
 static void
-gda_capi_provider_init (GdaCapiProvider *capi_prv, GdaCapiProviderClass *klass)
+gda_capi_provider_init (GdaCapiProvider *capi_prv, G_GNUC_UNUSED GdaCapiProviderClass *klass)
 {
 	InternalStatementItem i;
 	GdaSqlParser *parser;
@@ -282,7 +293,8 @@ gda_capi_provider_get_type (void)
 			NULL, NULL,
 			sizeof (GdaCapiProvider),
 			0,
-			(GInstanceInitFunc) gda_capi_provider_init
+			(GInstanceInitFunc) gda_capi_provider_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -298,7 +310,7 @@ gda_capi_provider_get_type (void)
  * Get provider name request
  */
 static const gchar *
-gda_capi_provider_get_name (GdaServerProvider *provider)
+gda_capi_provider_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return CAPI_PROVIDER_NAME;
 }
@@ -307,7 +319,7 @@ gda_capi_provider_get_name (GdaServerProvider *provider)
  * Get provider's version, no need to change this
  */
 static const gchar *
-gda_capi_provider_get_version (GdaServerProvider *provider)
+gda_capi_provider_get_version (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return PACKAGE_VERSION;
 }
@@ -321,12 +333,13 @@ gda_capi_provider_get_version (GdaServerProvider *provider)
  *   - open the real connection to the database using the parameters previously checked
  *   - create a CapiConnectionData structure and associate it to @cnc
  *
- * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR gonnection event must be added to @cnc)
+ * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR connection event must be added to @cnc)
  */
 static gboolean
 gda_capi_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-				   GdaQuarkList *params, GdaQuarkList *auth,
-				   guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
+				   GdaQuarkList *params, G_GNUC_UNUSED GdaQuarkList *auth,
+				   G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
+				   G_GNUC_UNUSED gpointer cb_data)
 {
 	g_return_val_if_fail (GDA_IS_CAPI_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -372,7 +385,7 @@ gda_capi_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
  *   - Actually close the connection to the database using @cnc's associated CapiConnectionData structure
  *   - Free the CapiConnectionData structure and its contents
  *
- * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR gonnection event must be added to @cnc)
+ * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR connection event must be added to @cnc)
  */
 static gboolean
 gda_capi_provider_close_connection (GdaServerProvider *provider, GdaConnection *cnc)
@@ -448,7 +461,7 @@ gda_capi_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
  */
 static gboolean
 gda_capi_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				      GdaServerOperationType type, GdaSet *options)
+				      GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options)
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -481,11 +494,12 @@ gda_capi_provider_supports_operation (GdaServerProvider *provider, GdaConnection
  * Create operation request
  *
  * Creates a #GdaServerOperation. The following code is generic and should only be changed
- * if some further initialization is required, or if operation's contents is dependant on @cnc
+ * if some further initialization is required, or if operation's contents is dependent on @cnc
  */
 static GdaServerOperation *
 gda_capi_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				    GdaServerOperationType type, GdaSet *options, GError **error)
+				    GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options,
+				    GError **error)
 {
         gchar *file;
         GdaServerOperation *op;
@@ -585,8 +599,9 @@ gda_capi_provider_render_operation (GdaServerProvider *provider, GdaConnection *
  */
 static gboolean
 gda_capi_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				     GdaServerOperation *op, guint *task_id, 
-				     GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+				     GdaServerOperation *op, G_GNUC_UNUSED guint *task_id,
+				     GdaServerProviderAsyncCallback async_cb, G_GNUC_UNUSED gpointer cb_data,
+				     GError **error)
 {
         GdaServerOperationType optype;
 
@@ -616,8 +631,9 @@ gda_capi_provider_perform_operation (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_begin_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-				     const gchar *name, GdaTransactionIsolation level,
-				     GError **error)
+				     G_GNUC_UNUSED const gchar *name,
+				     G_GNUC_UNUSED GdaTransactionIsolation level,
+				     G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -638,7 +654,7 @@ gda_capi_provider_begin_transaction (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_commit_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-				      const gchar *name, GError **error)
+				      G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -659,7 +675,7 @@ gda_capi_provider_commit_transaction (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_rollback_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-					const gchar *name, GError **error)
+					G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -680,7 +696,7 @@ gda_capi_provider_rollback_transaction (GdaServerProvider *provider, GdaConnecti
  */
 static gboolean
 gda_capi_provider_add_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				 const gchar *name, GError **error)
+				 G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -701,7 +717,7 @@ gda_capi_provider_add_savepoint (GdaServerProvider *provider, GdaConnection *cnc
  */
 static gboolean
 gda_capi_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				      const gchar *name, GError **error)
+				      G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -722,7 +738,7 @@ gda_capi_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				    const gchar *name, GError **error)
+				    G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -869,7 +885,7 @@ gda_capi_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnect
  * by the database. See the PostgreSQL provider implementation for an example.
  */
 static GdaSqlParser *
-gda_capi_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc)
+gda_capi_provider_create_parser (G_GNUC_UNUSED GdaServerProvider *provider, G_GNUC_UNUSED GdaConnection *cnc)
 {
 	TO_IMPLEMENT;
 	return NULL;
@@ -883,6 +899,9 @@ gda_capi_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc
  * The implementation show here simply calls gda_statement_to_sql_extended() but the rendering
  * can be specialized to the database's SQL dialect, see the implementation of gda_statement_to_sql_extended()
  * and SQLite's specialized rendering for more details
+ *
+ * NOTE: This implementation MUST NOT call gda_statement_to_sql_extended() if it is
+ *       the GdaServerProvider::statement_to_sql() virtual method's implementation
  */
 static gchar *
 gda_capi_provider_statement_to_sql (GdaServerProvider *provider, GdaConnection *cnc,
@@ -1062,7 +1081,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 		
 		/* find requested parameter */
 		if (!params) {
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+			event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 			gda_connection_event_set_description (event, _("Missing parameter(s) to execute query"));
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
@@ -1082,7 +1101,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			if (! allow_noparam) {
 				gchar *str;
 				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
-				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 				gda_connection_event_set_description (event, str);
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
@@ -1101,7 +1120,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			if (! allow_noparam) {
 				gchar *str;
 				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
-				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 				gda_connection_event_set_description (event, str);
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
@@ -1115,9 +1134,42 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
                                 continue;
                         }
 		}
+		else if (gda_holder_value_is_default (h) && !gda_holder_get_value (h)) {
+			/* create a new GdaStatement to handle all default values and execute it instead
+			 * needs to be adapted to take into account how the database server handles default
+			 * values (some accept the DEFAULT keyword), changing the 3rd argument of the
+			 * gda_statement_rewrite_for_default_values() call
+			 */
+			GdaSqlStatement *sqlst;
+			GError *lerror = NULL;
+			sqlst = gda_statement_rewrite_for_default_values (stmt, params, TRUE, &lerror);
+			if (!sqlst) {
+				event = gda_connection_point_available_event (cnc,
+									      GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, lerror && lerror->message ? 
+								      lerror->message :
+								      _("Can't rewrite statement handle default values"));
+				g_propagate_error (error, lerror);
+				break;
+			}
+			
+			GdaStatement *rstmt;
+			GObject *res;
+			rstmt = g_object_new (GDA_TYPE_STATEMENT, "structure", sqlst, NULL);
+			gda_sql_statement_free (sqlst);
+			res = gda_capi_provider_statement_execute (provider, cnc,
+								   rstmt, params,
+								   model_usage,
+								   col_types, last_inserted_row,
+								   task_id,
+								   async_cb, cb_data, error);
+			g_object_unref (rstmt);
+			return res;
+		}
+
 
 		/* actual binding using the C API, for parameter at position @i */
-		const GValue *value = gda_holder_get_value (h);
+		/* const GValue *value = gda_holder_get_value (h); */
 		TO_IMPLEMENT;
 	}
 		
@@ -1128,7 +1180,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	}
 	
 	/* add a connection event for the execution */
-	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+	event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_COMMAND);
         gda_connection_event_set_description (event, _GDA_PSTMT (ps)->sql);
         gda_connection_add_event (cnc, event);
 
@@ -1191,12 +1243,29 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	}
 }
 
+/*
+ * Rewrites a statement in case some parameters in @params are set to DEFAULT, for INSERT or UPDATE statements
+ *
+ * Usually it uses the DEFAULT keyword or removes any default value inserted or updated, see
+ * gda_statement_rewrite_for_default_values()
+ */
+static GdaSqlStatement *
+gda_capi_statement_rewrite (GdaServerProvider *provider, GdaConnection *cnc,
+			    GdaStatement *stmt, GdaSet *params, GError **error)
+{
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
+	}
+	return gda_statement_rewrite_for_default_values (stmt, params, TRUE, error);
+}
+
 /*
  * starts a distributed transaction: put the XA transaction in the ACTIVE state
  */
 static gboolean
 gda_capi_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc, 
-				const GdaXaTransactionId *xid, GError **error)
+				const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1218,7 +1287,7 @@ gda_capi_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc, 
-			      const GdaXaTransactionId *xid, GError **error)
+			      const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1239,7 +1308,7 @@ gda_capi_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_prepare (GdaServerProvider *provider, GdaConnection *cnc, 
-				  const GdaXaTransactionId *xid, GError **error)
+				  const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1261,7 +1330,7 @@ gda_capi_provider_xa_prepare (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_commit (GdaServerProvider *provider, GdaConnection *cnc, 
-				 const GdaXaTransactionId *xid, GError **error)
+				 const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1282,7 +1351,7 @@ gda_capi_provider_xa_commit (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_rollback (GdaServerProvider *provider, GdaConnection *cnc, 
-				   const GdaXaTransactionId *xid, GError **error)
+				   const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1305,7 +1374,7 @@ gda_capi_provider_xa_rollback (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static GList *
 gda_capi_provider_xa_recover (GdaServerProvider *provider, GdaConnection *cnc,
-				  GError **error)
+				  G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
