@@ -62,7 +62,7 @@ static GObjectClass *parent_class = NULL;
  */
 static void
 gda_web_recordset_init (GdaWebRecordset *recset,
-			   GdaWebRecordsetClass *klass)
+			   G_GNUC_UNUSED GdaWebRecordsetClass *klass)
 {
 	g_return_if_fail (GDA_IS_WEB_RECORDSET (recset));
 	recset->priv = g_new0 (GdaWebRecordsetPrivate, 1);
@@ -137,7 +137,8 @@ gda_web_recordset_get_type (void)
 			NULL,
 			sizeof (GdaWebRecordset),
 			0,
-			(GInstanceInitFunc) gda_web_recordset_init
+			(GInstanceInitFunc) gda_web_recordset_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -343,10 +344,10 @@ create_table (GdaWebRecordset *rs, GError **error)
 		g_string_append_printf (string, "%s %s", colname,
 					gda_g_type_to_string (gda_column_get_g_type (column)));
 
-		gda_sql_builder_add_field_id (ib, gda_sql_builder_add_id (ib, colname),
+		gda_sql_builder_add_field_value_id (ib, gda_sql_builder_add_id (ib, colname),
 					   gda_sql_builder_add_param (ib, colname,
 								      gda_column_get_g_type (column), TRUE));
-		gda_sql_builder_add_field_id (sb, gda_sql_builder_add_id (sb, colname), 0);
+		gda_sql_builder_add_field_value_id (sb, gda_sql_builder_add_id (sb, colname), 0);
 
 		g_free (colname);
 	}
