@@ -115,7 +115,8 @@ gda_ddl_creator_get_type (void) {
 			NULL,
 			sizeof (GdaDDLCreator),
 			0,
-			(GInstanceInitFunc) gda_ddl_creator_init
+			(GInstanceInitFunc) gda_ddl_creator_init,
+			0
 		};
 		
 		g_static_rec_mutex_lock (&init_mutex);
@@ -266,7 +267,7 @@ static void
 gda_ddl_creator_set_property (GObject *object,
 			      guint param_id,
 			      const GValue *value,
-			      GParamSpec *pspec) 
+			      GParamSpec *pspec)
 {
 	GdaDDLCreator *creator;
 	
@@ -311,6 +312,9 @@ gda_ddl_creator_set_property (GObject *object,
 								  FALSE, FALSE);
 			}
 			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -319,7 +323,7 @@ static void
 gda_ddl_creator_get_property (GObject *object,
 			     guint param_id,
 			     GValue *value,
-			     GParamSpec *pspec) 
+			     GParamSpec *pspec)
 {
 	GdaDDLCreator *creator;
 	creator = GDA_DDL_CREATOR (object);
@@ -341,6 +345,9 @@ gda_ddl_creator_get_property (GObject *object,
 			else
 				g_value_set_string (value, NULL);
 			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -632,7 +639,7 @@ create_server_operation_for_table (GdaDDLCreator *ddlc, GdaServerProvider *prov,
 }
 
 static GdaServerOperation *
-create_server_operation_for_view (GdaDDLCreator *ddlc, GdaServerProvider *prov, GdaConnection *cnc, 
+create_server_operation_for_view (G_GNUC_UNUSED GdaDDLCreator *ddlc, GdaServerProvider *prov, GdaConnection *cnc,
 				  GdaMetaDbObject *dbobj, GError **error)
 {
 	GdaServerOperation *op;
