@@ -87,7 +87,8 @@ browser_canvas_table_get_type (void)
 			NULL,
 			sizeof (BrowserCanvasTable),
 			0,
-			(GInstanceInitFunc) browser_canvas_table_init
+			(GInstanceInitFunc) browser_canvas_table_init,
+			0
 		};		
 
 		type = g_type_register_static (TYPE_BROWSER_CANVAS_ITEM, "BrowserCanvasTable", &info, 0);
@@ -229,6 +230,9 @@ browser_canvas_table_set_property (GObject *object,
 	case PROP_MENU_FUNC:
 		ce->priv->popup_menu_func = (GtkWidget *(*) (BrowserCanvasTable *ce)) g_value_get_pointer (value);
 		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
 	}
 }
 
@@ -249,6 +253,9 @@ browser_canvas_table_get_property (GObject *object,
 	case PROP_TABLE:
 		g_value_set_pointer (value, ce->priv->table);
 		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
 	}
 }
 
@@ -403,8 +410,9 @@ create_items (BrowserCanvasTable *ce)
 }
 
 static gboolean
-button_press_event_cb (BrowserCanvasTable *ce, GooCanvasItem  *target_item, GdkEventButton *event,
-		       gpointer unused_data)
+button_press_event_cb (BrowserCanvasTable *ce, G_GNUC_UNUSED GooCanvasItem  *target_item,
+		       GdkEventButton *event,
+		       G_GNUC_UNUSED gpointer data)
 {
 	if ((event->button == 3) && ce->priv->popup_menu_func) {
 		GtkWidget *menu;
@@ -515,8 +523,9 @@ browser_canvas_table_new (GooCanvasItem *parent, GdaMetaStruct *mstruct, GdaMeta
 }
 
 static void
-browser_canvas_table_drag_data_get (BrowserCanvasItem *citem, GdkDragContext *drag_context,
-				    GtkSelectionData *data, guint info, guint time)
+browser_canvas_table_drag_data_get (BrowserCanvasItem *citem, G_GNUC_UNUSED GdkDragContext *drag_context,
+				    GtkSelectionData *data, G_GNUC_UNUSED guint info,
+				    G_GNUC_UNUSED guint time)
 {
 	BrowserCanvasTable *ctable;
 
@@ -535,7 +544,11 @@ browser_canvas_table_drag_data_get (BrowserCanvasItem *citem, GdkDragContext *dr
 	g_free (tmp1);
 	g_free (tmp2);
 	g_free (tmp3);
-	gtk_selection_data_set (data, data->target, 8, str, strlen (str));
+#if GTK_CHECK_VERSION(2,18,0)
+	gtk_selection_data_set (data, gtk_selection_data_get_target (data), 8, (guchar*) str, strlen (str));
+#else
+	gtk_selection_data_set (data, data->target, 8, (guchar*) str, strlen (str));
+#endif
 	g_free (str);
 }
 
@@ -561,7 +574,7 @@ browser_canvas_table_serialize (BrowserCanvasItem *citem)
 	gchar *str;
 
 	dbo = GDA_META_DB_OBJECT (ctable->priv->table);
-	node = xmlNewNode (NULL, "table");
+	node = xmlNewNode (NULL, BAD_CAST "table");
 	xmlSetProp (node, BAD_CAST "schema", BAD_CAST (dbo->obj_schema));
 	xmlSetProp (node, BAD_CAST "name", BAD_CAST (dbo->obj_name));
 	goo_canvas_item_get_bounds (GOO_CANVAS_ITEM (citem), &bounds);
