@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 The GNOME Foundation
+ * Copyright (C) 2009 - 2011 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -32,21 +32,10 @@
 #include "../browser-stock-icons.h"
 #include "query-editor.h"
 #include "query-result.h"
-#include "../common/popup-container.h"
+#include <libgda-ui/internal/popup-container.h>
 #include <libgda/sql-parser/gda-sql-parser.h>
 #include <libgda-ui/libgda-ui.h>
 
-#define VARIABLES_HELP _("<small>This area allows to give values to\n" \
-			 "variables defined in the SQL code\n"		\
-			 "using the following syntax:\n"		\
-			 "<b><tt>##&lt;variable name&gt;::&lt;type&gt;[::null]</tt></b>\n" \
-			 "For example:\n"				\
-			 "<span foreground=\"#4e9a06\"><b><tt>##id::int</tt></b></span>\n      defines <b>id</b> as a non NULL integer\n" \
-			 "<span foreground=\"#4e9a06\"><b><tt>##age::string::null</tt></b></span>\n      defines <b>age</b> as a a string\n\n" \
-			 "Valid types are: <tt>string</tt>, <tt>boolean</tt>, <tt>int</tt>,\n" \
-			 "<tt>date</tt>, <tt>time</tt>, <tt>timestamp</tt>, <tt>guint</tt>, <tt>blob</tt> and\n" \
-			 "<tt>binary</tt></small>")
-
 /*
  * Statement execution structures
  */
@@ -117,7 +106,6 @@ struct _QueryConsolePrivate {
 	GtkWidget   *exec_button;
 	GtkWidget   *indent_button;
 	guint params_compute_id; /* timout ID to compute params */
-	GdaSet *past_params; /* keeps values given to old params */
 	GdaSet *params; /* execution params */
 	GtkWidget *params_popup; /* popup shown when invalid params are required */
 
@@ -134,6 +122,9 @@ struct _QueryConsolePrivate {
 
 	ExecutionBatch *current_exec;
 	guint current_exec_id; /* timout ID to fetch execution results */
+
+	gint fav_id;
+	GtkWidget *favorites_menu;
 };
 
 static void query_console_class_init (QueryConsoleClass *klass);
@@ -148,11 +139,6 @@ static GtkActionGroup      *query_console_page_get_actions_group (BrowserPage *p
 static const gchar         *query_console_page_get_actions_ui (BrowserPage *page);
 static GtkWidget           *query_console_page_get_tab_label (BrowserPage *page, GtkWidget **out_close_button);
 
-enum {
-	LAST_SIGNAL
-};
-
-static guint query_console_signals[LAST_SIGNAL] = { };
 static GObjectClass *parent_class = NULL;
 
 /*
@@ -192,15 +178,15 @@ query_console_page_init (BrowserPageIface *iface)
 }
 
 static void
-query_console_init (QueryConsole *tconsole, QueryConsoleClass *klass)
+query_console_init (QueryConsole *tconsole, G_GNUC_UNUSED QueryConsoleClass *klass)
 {
 	tconsole->priv = g_new0 (QueryConsolePrivate, 1);
 	tconsole->priv->parser = NULL;
 	tconsole->priv->params_compute_id = 0;
-	tconsole->priv->past_params = NULL;
 	tconsole->priv->params = NULL;
 	tconsole->priv->params_popup = NULL;
 	tconsole->priv->agroup = NULL;
+	tconsole->priv->fav_id = -1;
 }
 static void connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy,
 				gchar *reason, QueryConsole *tconsole);
@@ -222,14 +208,16 @@ query_console_dispose (GObject *object)
 		}
 		if (tconsole->priv->parser)
 			g_object_unref (tconsole->priv->parser);
-		if (tconsole->priv->past_params)
-			g_object_unref (tconsole->priv->past_params);
 		if (tconsole->priv->params)
 			g_object_unref (tconsole->priv->params);
 		if (tconsole->priv->params_compute_id)
 			g_source_remove (tconsole->priv->params_compute_id);
 		if (tconsole->priv->params_popup)
 			gtk_widget_destroy (tconsole->priv->params_popup);
+		if (tconsole->priv->agroup)
+			g_object_unref (tconsole->priv->agroup);
+		if (tconsole->priv->favorites_menu)
+			gtk_widget_destroy (tconsole->priv->favorites_menu);
 
 		g_free (tconsole->priv);
 		tconsole->priv = NULL;
@@ -253,7 +241,8 @@ query_console_get_type (void)
 			NULL,
 			sizeof (QueryConsole),
 			0,
-			(GInstanceInitFunc) query_console_init
+			(GInstanceInitFunc) query_console_init,
+			0
 		};
 
 		static GInterfaceInfo page_console = {
@@ -268,9 +257,6 @@ query_console_get_type (void)
 	return type;
 }
 
-static GtkWidget *make_small_button (gboolean is_toggle,
-				     const gchar *label, const gchar *stock_id, const gchar *tooltip);
-
 static void editor_changed_cb (QueryEditor *editor, QueryConsole *tconsole);
 static void editor_execute_request_cb (QueryEditor *editor, QueryConsole *tconsole);
 static void sql_clear_clicked_cb (GtkButton *button, QueryConsole *tconsole);
@@ -367,6 +353,7 @@ query_console_new (BrowserConnection *bcnc)
 
 	wid = gtk_label_new ("");
 	gtk_label_set_markup (GTK_LABEL (wid), VARIABLES_HELP);
+	gtk_misc_set_alignment (GTK_MISC (wid), -1, 0.);
 	gtk_container_add (GTK_CONTAINER (tconsole->priv->params_form_box), wid);
 	tconsole->priv->params_form = wid;
 	
@@ -374,32 +361,36 @@ query_console_new (BrowserConnection *bcnc)
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 5);
 
-	button = make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear the editor"));
+	button = browser_make_small_button (FALSE, FALSE, _("Clear"),
+					    GTK_STOCK_CLEAR, _("Clear the editor's\ncontents"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_clear_clicked_cb), tconsole);
 
-	button = make_small_button (TRUE, _("Variables"), NULL, _("Show variables needed\nto execute SQL"));
+	button = browser_make_small_button (TRUE, FALSE, _("Variables"), NULL,
+					    _("Show variables needed\nto execute SQL"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	tconsole->priv->params_toggle = GTK_TOGGLE_BUTTON (button);
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (sql_variables_clicked_cb), tconsole);
 
-	button = make_small_button (FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute SQL in editor"));
+	button = browser_make_small_button (FALSE, FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute SQL in editor"));
 	tconsole->priv->exec_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_execute_clicked_cb), tconsole);
 	
-	button = make_small_button (FALSE, _("Indent"), GTK_STOCK_INDENT, _("Indent SQL in editor\n"
-									    "and make the code more readable\n"
-									    "(removes comments)"));
+	button = browser_make_small_button (FALSE, FALSE, _("Indent"),
+					    GTK_STOCK_INDENT, _("Indent SQL in editor\n"
+								"and make the code more readable\n"
+								"(removes comments)"));
 	tconsole->priv->indent_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_indent_clicked_cb), tconsole);
 
-	button = make_small_button (FALSE, _("Favorite"), STOCK_ADD_BOOKMARK, _("Add SQL to favorite"));
+	button = browser_make_small_button (FALSE, TRUE, _("Favorite"),
+					    STOCK_ADD_BOOKMARK, _("Add SQL to favorite"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_favorite_clicked_cb), tconsole);
@@ -431,14 +422,16 @@ query_console_new (BrowserConnection *bcnc)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 
-	button = make_small_button (FALSE, _("Copy"), GTK_STOCK_COPY, _("Copy selected history\nto editor"));
+	button = browser_make_small_button (FALSE, FALSE, _("Copy"), GTK_STOCK_COPY,
+					    _("Copy selected history\nto editor"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (history_copy_clicked_cb), tconsole);
 	tconsole->priv->history_copy_button = button;
 	gtk_widget_set_sensitive (button, FALSE);
 
-	button = make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear history"));
+	button = browser_make_small_button (FALSE, FALSE, _("Clear"),
+					    GTK_STOCK_CLEAR, _("Clear history"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (history_clear_clicked_cb), tconsole);
@@ -477,7 +470,7 @@ query_console_new (BrowserConnection *bcnc)
 }
 
 static void
-connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, QueryConsole *tconsole)
+connection_busy_cb (G_GNUC_UNUSED BrowserConnection *bcnc, gboolean is_busy, G_GNUC_UNUSED gchar *reason, QueryConsole *tconsole)
 {
 	gtk_widget_set_sensitive (tconsole->priv->exec_button, !is_busy);
 	gtk_widget_set_sensitive (tconsole->priv->indent_button, !is_busy);
@@ -489,47 +482,8 @@ connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, Qu
 	}
 }
 
-static GtkWidget *
-make_small_button (gboolean is_toggle, const gchar *label, const gchar *stock_id, const gchar *tooltip)
-{
-	GtkWidget *button, *hbox = NULL;
-
-	if (is_toggle)
-		button = gtk_toggle_button_new ();
-	else
-		button = gtk_button_new ();
-	if (label && stock_id) {
-		hbox = gtk_hbox_new (FALSE, 0);
-		gtk_container_add (GTK_CONTAINER (button), hbox);
-		gtk_widget_show (hbox);
-	}
-
-	if (stock_id) {
-		GtkWidget *image;
-		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
-		if (hbox)
-			gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-		else
-			gtk_container_add (GTK_CONTAINER (button), image);
-		gtk_widget_show (image);
-	}
-	if (label) {
-		GtkWidget *wid;
-		wid = gtk_label_new (label);
-		if (hbox)
-			gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 5);
-		else
-			gtk_container_add (GTK_CONTAINER (button), wid);
-		gtk_widget_show (wid);
-	}
-
-	if (tooltip)
-		gtk_widget_set_tooltip_text (button, tooltip);
-	return button;
-}
-
 static void
-history_changed_cb (QueryEditor *history, QueryConsole *tconsole)
+history_changed_cb (G_GNUC_UNUSED QueryEditor *history, QueryConsole *tconsole)
 {
 	gboolean act = FALSE;
 	QueryEditor *qe;
@@ -553,13 +507,13 @@ history_changed_cb (QueryEditor *history, QueryConsole *tconsole)
 }
 
 static void
-history_clear_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+history_clear_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
 {
 	query_editor_del_all_history_items (tconsole->priv->history);
 }
 
 static void
-history_copy_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+history_copy_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
 {
 	QueryEditorHistoryItem *qih;
 	QueryEditor *qe;
@@ -584,9 +538,11 @@ history_copy_clicked_cb (GtkButton *button, QueryConsole *tconsole)
         }
 
 	query_editor_set_text (tconsole->priv->editor, string->str);
+	tconsole->priv->fav_id = -1;
 	g_string_free (string, TRUE);
 }
 
+
 static gboolean
 compute_params (QueryConsole *tconsole)
 {
@@ -594,16 +550,11 @@ compute_params (QueryConsole *tconsole)
 	GdaBatch *batch;
 
 	if (tconsole->priv->params) {
-		if (tconsole->priv->past_params) {
-			gda_set_merge_with_set (tconsole->priv->params, tconsole->priv->past_params);
-			g_object_unref (tconsole->priv->past_params);
-			tconsole->priv->past_params = tconsole->priv->params;
-		}
-		else
-			tconsole->priv->past_params = tconsole->priv->params;
+		browser_connection_keep_variables (tconsole->priv->bcnc, tconsole->priv->params);
+		g_object_unref (tconsole->priv->params);
 	}
-
 	tconsole->priv->params = NULL;
+
 	if (tconsole->priv->params_form) {
 		gtk_widget_destroy (tconsole->priv->params_form);
 		tconsole->priv->params_form = NULL;		
@@ -621,6 +572,9 @@ compute_params (QueryConsole *tconsole)
 
 		if (gda_batch_get_parameters (batch, &(tconsole->priv->params), &error)) {
 			if (tconsole->priv->params) {
+				browser_connection_define_ui_plugins_for_batch (tconsole->priv->bcnc,
+										batch,
+										tconsole->priv->params);
 				show_variables = TRUE;
 				tconsole->priv->params_form = gdaui_basic_form_new (tconsole->priv->params);
 				g_object_set ((GObject*) tconsole->priv->params_form,
@@ -640,35 +594,7 @@ compute_params (QueryConsole *tconsole)
 		gtk_widget_show (tconsole->priv->params_form);
 		g_object_unref (batch);
 
-		if (tconsole->priv->past_params && tconsole->priv->params) {
-			/* copy the values from tconsole->priv->past_params to tconsole->priv->params */
-			GSList *list;
-			for (list = tconsole->priv->params->holders; list; list = list->next) {
-				GdaHolder *oldh, *newh;
-				newh = GDA_HOLDER (list->data);
-				oldh = gda_set_get_holder (tconsole->priv->past_params, gda_holder_get_id  (newh));
-				if (oldh) {
-					GType otype, ntype;
-					otype = gda_holder_get_g_type (oldh);
-					ntype = gda_holder_get_g_type (newh);
-					if (otype == ntype) {
-						const GValue *ovalue;
-						ovalue = gda_holder_get_value (oldh);
-						gda_holder_set_value (newh, ovalue, NULL);
-					}
-					else if (g_value_type_transformable (otype, ntype)) {
-						const GValue *ovalue;
-						GValue *nvalue;
-						ovalue = gda_holder_get_value (oldh);
-						nvalue = gda_value_new (ntype);
-						if (g_value_transform (ovalue, nvalue))
-							gda_holder_take_value (newh, nvalue, NULL);
-						else
-							gda_value_free  (nvalue);
-					}
-				}
-			}
-		}
+		browser_connection_load_variables (tconsole->priv->bcnc, tconsole->priv->params);
 		if (tconsole->priv->params && show_variables &&
 		    gda_set_is_valid (tconsole->priv->params, NULL))
 			show_variables = FALSE;
@@ -688,7 +614,7 @@ compute_params (QueryConsole *tconsole)
 }
 
 static void
-editor_changed_cb (QueryEditor *editor, QueryConsole *tconsole)
+editor_changed_cb (G_GNUC_UNUSED QueryEditor *editor, QueryConsole *tconsole)
 {
 	if (tconsole->priv->params_compute_id)
 		g_source_remove (tconsole->priv->params_compute_id);
@@ -696,7 +622,7 @@ editor_changed_cb (QueryEditor *editor, QueryConsole *tconsole)
 }
 
 static void
-editor_execute_request_cb (QueryEditor *editor, QueryConsole *tconsole)
+editor_execute_request_cb (G_GNUC_UNUSED QueryEditor *editor, QueryConsole *tconsole)
 {
 	gboolean sensitive;
 	g_object_get (tconsole->priv->exec_button, "sensitive", &sensitive, NULL);
@@ -714,13 +640,15 @@ sql_variables_clicked_cb (GtkToggleButton *button, QueryConsole *tconsole)
 }
 
 static void
-sql_clear_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+sql_clear_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
 {
 	query_editor_set_text (tconsole->priv->editor, NULL);
+	tconsole->priv->fav_id = -1;
+	gtk_widget_grab_focus (GTK_WIDGET (tconsole->priv->editor));
 }
 
 static void
-sql_indent_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+sql_indent_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
 {
 	gchar *sql;
 	GdaBatch *batch;
@@ -754,18 +682,144 @@ sql_indent_clicked_cb (GtkButton *button, QueryConsole *tconsole)
 	}
 }
 
+static void sql_favorite_new_mitem_cb (G_GNUC_UNUSED GtkMenuItem *mitem, QueryConsole *tconsole);
+static void sql_favorite_modify_mitem_cb (G_GNUC_UNUSED GtkMenuItem *mitem, QueryConsole *tconsole);
+
 static void
-sql_favorite_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+sql_favorite_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
+{
+	GtkWidget *menu, *mitem;
+	BrowserFavorites *bfav;
+
+	if (tconsole->priv->favorites_menu)
+		gtk_widget_destroy (tconsole->priv->favorites_menu);
+
+	menu = gtk_menu_new ();
+	tconsole->priv->favorites_menu = menu;
+	mitem = gtk_menu_item_new_with_label (_("New favorite"));
+	g_signal_connect (mitem, "activate",
+			  G_CALLBACK (sql_favorite_new_mitem_cb), tconsole);
+	gtk_widget_show (mitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
+
+	bfav = browser_connection_get_favorites (tconsole->priv->bcnc);
+	if (tconsole->priv->fav_id >= 0) {
+		BrowserFavoritesAttributes fav;
+		if (browser_favorites_get (bfav, tconsole->priv->fav_id, &fav, NULL)) {
+			gchar *str;
+			str = g_strdup_printf (_("Modify favorite '%s'"), fav.name);
+			mitem = gtk_menu_item_new_with_label (str);
+			g_free (str);
+			g_signal_connect (mitem, "activate",
+					  G_CALLBACK (sql_favorite_modify_mitem_cb), tconsole);
+			g_object_set_data_full (G_OBJECT (mitem), "favname",
+						g_strdup (fav.name), g_free);
+			g_object_set_data (G_OBJECT (mitem), "favid",
+					   GINT_TO_POINTER (tconsole->priv->fav_id));
+			gtk_widget_show (mitem);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
+			browser_favorites_reset_attributes (&fav);
+		}
+	}
+
+	GSList *allfav;
+	allfav = browser_favorites_list (bfav, 0, BROWSER_FAVORITES_QUERIES, ORDER_KEY_QUERIES, NULL);
+	if (allfav) {
+		GtkWidget *submenu;
+		GSList *list;
+
+		mitem = gtk_menu_item_new_with_label (_("Modify a favorite"));
+		gtk_widget_show (mitem);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
+
+		submenu = gtk_menu_new ();
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), submenu);
+		for (list = allfav; list; list = list->next) {
+			BrowserFavoritesAttributes *fav;
+			fav = (BrowserFavoritesAttributes*) list->data;
+			if (fav->id == tconsole->priv->fav_id)
+				continue;
+			gchar *str;
+			str = g_strdup_printf (_("Modify favorite '%s'"), fav->name);
+			mitem = gtk_menu_item_new_with_label (str);
+			g_free (str);
+			g_signal_connect (mitem, "activate",
+					  G_CALLBACK (sql_favorite_modify_mitem_cb), tconsole);
+			g_object_set_data_full (G_OBJECT (mitem), "favname",
+						g_strdup (fav->name), g_free);
+			g_object_set_data (G_OBJECT (mitem), "favid",
+					   GINT_TO_POINTER (fav->id));
+			gtk_widget_show (mitem);
+			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mitem);
+		}
+		browser_favorites_free_list (allfav);
+	}
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			NULL, NULL, 0,
+			gtk_get_current_event_time ());
+}
+
+static void
+sql_favorite_new_mitem_cb (GtkMenuItem *mitem, QueryConsole *tconsole)
+{
+	BrowserFavorites *bfav;
+	BrowserFavoritesAttributes fav;
+	GError *error = NULL;
+
+	GdaSet *set;
+	GtkWidget *dlg;
+	gint response;
+	const GValue *cvalue;
+	set = gda_set_new_inline (1, _("Favorite's name"),
+				  G_TYPE_STRING, _("Unnamed query"));
+	dlg = gdaui_basic_form_new_in_dialog (set,
+					      (GtkWindow*) gtk_widget_get_toplevel (GTK_WIDGET (tconsole)),
+					      _("Name of the favorite to create"),
+					      _("Enter the name of the favorite to create"));
+	response = gtk_dialog_run (GTK_DIALOG (dlg));
+	if (response == GTK_RESPONSE_REJECT) {
+		g_object_unref (set);
+		gtk_widget_destroy (dlg);
+		return;
+	}
+
+	memset (&fav, 0, sizeof (fav));
+	fav.id = -1;
+	fav.type = BROWSER_FAVORITES_QUERIES;
+	fav.contents = query_editor_get_all_text (tconsole->priv->editor);
+	cvalue = gda_set_get_holder_value (set, _("Favorite's name"));
+	fav.name = (gchar*) g_value_get_string (cvalue);
+
+	bfav = browser_connection_get_favorites (tconsole->priv->bcnc);
+
+	if (! browser_favorites_add (bfav, 0, &fav, ORDER_KEY_QUERIES, G_MAXINT, &error)) {
+		browser_show_error ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) tconsole),
+                                    _("Could not add favorite: %s"),
+                                    error && error->message ? error->message : _("No detail"));
+                if (error)
+                        g_error_free (error);
+	}
+	else
+		tconsole->priv->fav_id = fav.id;
+	g_free (fav.contents);
+
+	g_object_unref (set);
+	gtk_widget_destroy (dlg);
+}
+
+static void
+sql_favorite_modify_mitem_cb (G_GNUC_UNUSED GtkMenuItem *mitem, QueryConsole *tconsole)
 {
 	BrowserFavorites *bfav;
 	BrowserFavoritesAttributes fav;
 	GError *error = NULL;
 
 	memset (&fav, 0, sizeof (fav));
-	fav.id = -1;
+	fav.id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (mitem), "favid"));
 	fav.type = BROWSER_FAVORITES_QUERIES;
 	fav.contents = query_editor_get_all_text (tconsole->priv->editor);
-	fav.name = _("Unnamed query");
+	fav.name = g_object_get_data (G_OBJECT (mitem), "favname");
 
 	bfav = browser_connection_get_favorites (tconsole->priv->bcnc);
 
@@ -813,7 +867,8 @@ popup_container_position_func (PopupContainer *cont, gint *out_x, gint *out_y)
 }
 
 static void
-params_form_changed_cb (GdauiBasicForm *form, GdaHolder *param, gboolean is_user_modif, QueryConsole *tconsole)
+params_form_changed_cb (GdauiBasicForm *form, G_GNUC_UNUSED GdaHolder *param,
+			G_GNUC_UNUSED gboolean is_user_modif, QueryConsole *tconsole)
 {
 	/* if all params are valid => authorize the execute button */
 	GtkWidget *button;
@@ -826,7 +881,7 @@ params_form_changed_cb (GdauiBasicForm *form, GdaHolder *param, gboolean is_user
 static gboolean query_exec_fetch_cb (QueryConsole *tconsole);
 
 static void
-sql_execute_clicked_cb (GtkButton *button, QueryConsole *tconsole)
+sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsole *tconsole)
 {
 	gchar *sql;
 	const gchar *remain;
@@ -1025,14 +1080,15 @@ query_exec_fetch_cb (QueryConsole *tconsole)
 				if (! history->within_transaction &&
 				    browser_connection_get_transaction_status (tconsole->priv->bcnc) &&
 				    gda_statement_get_statement_type (estmt->stmt) != GDA_SQL_STATEMENT_BEGIN) {
-					browser_show_notice (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-							     "QueryExecTransactionStarted",
-							     "%s", _("A transaction has automatically been started\n"
-								     "during this statement's execution, this usually\n"
-								     "happens when blobs are selected (and the transaction\n"
-								     "will have to remain opened while the blobs are still\n"
-								     "accessible, clear the corresponding history item before\n"
-								     "closing the transaction)."));
+					browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+									   GTK_MESSAGE_INFO,
+									   "QueryExecTransactionStarted",
+									   "%s", _("A transaction has automatically been started\n"
+										   "during this statement's execution, this usually\n"
+										   "happens when blobs are selected (and the transaction\n"
+										   "will have to remain opened while the blobs are still\n"
+										   "accessible, clear the corresponding history item before\n"
+										   "closing the transaction)."));
 				}
 				
 				query_editor_add_history_item (tconsole->priv->history, history);
@@ -1085,15 +1141,17 @@ query_exec_fetch_cb (QueryConsole *tconsole)
 
 /**
  * query_console_set_text
- * @console:
- * @text:
+ * @console: a #QueryConsole
+ * @text: the new text
+ * @fav_id: the favorite ID or -1 if not a favorite.
  *
  * Replaces the edited SQL with @text in @console
  */
 void
-query_console_set_text (QueryConsole *console, const gchar *text)
+query_console_set_text (QueryConsole *console, const gchar *text, gint fav_id)
 {
 	g_return_if_fail (IS_QUERY_CONSOLE (console));
+	console->priv->fav_id = fav_id;
 	query_editor_set_text (console->priv->editor, text);
 }
 
@@ -1101,14 +1159,14 @@ query_console_set_text (QueryConsole *console, const gchar *text)
  * UI actions
  */
 static void
-query_execute_cb (GtkAction *action, QueryConsole *tconsole)
+query_execute_cb (G_GNUC_UNUSED GtkAction *action, QueryConsole *tconsole)
 {
 	sql_execute_clicked_cb (NULL, tconsole);
 }
 
 #ifdef HAVE_GTKSOURCEVIEW
 static void
-editor_undo_cb (GtkAction *action, QueryConsole *tconsole)
+editor_undo_cb (G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED QueryConsole *tconsole)
 {
 	TO_IMPLEMENT;
 }
@@ -1142,6 +1200,7 @@ query_console_page_get_actions_group (BrowserPage *page)
 	tconsole = QUERY_CONSOLE (page);
 	if (! tconsole->priv->agroup) {
 		tconsole->priv->agroup = gtk_action_group_new ("QueryExecConsoleActions");
+		gtk_action_group_set_translation_domain (tconsole->priv->agroup, GETTEXT_PACKAGE);
 		gtk_action_group_add_actions (tconsole->priv->agroup,
 					      ui_actions, G_N_ELEMENTS (ui_actions), page);
 
@@ -1153,7 +1212,7 @@ query_console_page_get_actions_group (BrowserPage *page)
 }
 
 static const gchar *
-query_console_page_get_actions_ui (BrowserPage *page)
+query_console_page_get_actions_ui (G_GNUC_UNUSED BrowserPage *page)
 {
 	return ui_actions_console;
 }
