@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007 - 2009 The GNOME Foundation.
+ * Copyright (C) 2007 - 2011 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -28,6 +28,7 @@
 #include <readline/history.h>
 #endif
 #include <sql-parser/gda-statement-struct-util.h>
+#include "tools-utils.h"
 
 /*
  *  gda_internal_command_arg_remove_quotes
@@ -53,12 +54,12 @@ gda_internal_command_arg_remove_quotes (gchar *str)
 
         total = strlen (str);
         if (str[total-1] == delim) {
-                /* string is correclty terminated by a double quote */
+                /* string is correctly terminated by a double quote */
                 g_memmove (str, str+1, total-2);
                 total -=2;
         }
         else {
-                /* string is _not_ correclty terminated by a double quote */
+                /* string is _not_ correctly terminated by a double quote */
                 g_memmove (str, str+1, total-1);
                 total -=1;
         }
@@ -117,6 +118,11 @@ gda_internal_command_exec_result_free (GdaInternalCommandResult *res)
 	case GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL:
 		if (res->u.model)
 			g_object_unref (res->u.model);
+		if (! res->was_in_transaction_before_exec &&
+		    res->cnc &&
+		    gda_connection_get_transaction_status (res->cnc))
+			gda_connection_rollback_transaction (res->cnc, NULL, NULL);
+		    
 		break;
 	case GDA_INTERNAL_COMMAND_RESULT_SET:
 		if (res->u.set)
@@ -138,6 +144,8 @@ gda_internal_command_exec_result_free (GdaInternalCommandResult *res)
 	default:
 		break;
 	}
+	if (res->cnc)
+		g_object_unref (res->cnc);
 	g_free (res);
 }
 
@@ -244,8 +252,8 @@ default_gda_internal_commandargs_func (const gchar *string)
 }
 
 GdaInternalCommandResult *
-gda_internal_command_help (SqlConsole *console, GdaConnection *cnc, const gchar **args, 
-			   GError **error,
+gda_internal_command_help (SqlConsole *console, G_GNUC_UNUSED GdaConnection *cnc,
+			   G_GNUC_UNUSED const gchar **args, G_GNUC_UNUSED GError **error,
 			   GdaInternalCommandsList *clist)
 {
 	GdaInternalCommandResult *res;
@@ -289,7 +297,8 @@ gda_internal_command_help (SqlConsole *console, GdaConnection *cnc, const gchar
 }
 
 GdaInternalCommandResult *
-gda_internal_command_history (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_history (SqlConsole *console, G_GNUC_UNUSED GdaConnection *cnc, const gchar **args,
+			      GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 
@@ -334,7 +343,8 @@ gda_internal_command_history (SqlConsole *console, GdaConnection *cnc, const gch
 }
 
 GdaInternalCommandResult *
-gda_internal_command_dict_sync (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_dict_sync (G_GNUC_UNUSED SqlConsole *console, GdaConnection *cnc, const gchar **args,
+				GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 
@@ -346,7 +356,21 @@ gda_internal_command_dict_sync (SqlConsole *console, GdaConnection *cnc, const g
 	res = g_new0 (GdaInternalCommandResult, 1);
 	res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
 
-	if (!gda_connection_update_meta_store (cnc, NULL, error)) {
+	if (args[0] && *args[0]) {
+		GdaMetaContext context;
+		memset (&context, 0, sizeof (context));
+		if (*args[0] == '_')
+			context.table_name = (gchar*) args[0];
+		else
+			context.table_name = g_strdup_printf ("_%s", args[0]);
+		if (!gda_connection_update_meta_store (cnc, &context, error)) {
+			g_free (res);
+			res = NULL;
+		}
+		if (*args[0] != '_')
+			g_free (context.table_name);
+	}
+	else if (!gda_connection_update_meta_store (cnc, NULL, error)) {
 		g_free (res);
 		res = NULL;
 	}
@@ -355,7 +379,8 @@ gda_internal_command_dict_sync (SqlConsole *console, GdaConnection *cnc, const g
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_tables (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_tables (G_GNUC_UNUSED SqlConsole *console, GdaConnection *cnc, const gchar **args,
+				  GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
@@ -398,7 +423,8 @@ gda_internal_command_list_tables (SqlConsole *console, GdaConnection *cnc, const
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_views (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_views (G_GNUC_UNUSED SqlConsole *console, GdaConnection *cnc, const gchar **args,
+				 GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
@@ -440,7 +466,8 @@ gda_internal_command_list_views (SqlConsole *console, GdaConnection *cnc, const
 }
 
 GdaInternalCommandResult *
-gda_internal_command_list_schemas (SqlConsole *console, GdaConnection *cnc, const gchar **args, GError **error, gpointer data)
+gda_internal_command_list_schemas (G_GNUC_UNUSED SqlConsole *console, GdaConnection *cnc, const gchar **args,
+				   GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
@@ -572,8 +599,8 @@ meta_table_column_foreach_attribute_func (const gchar *att_name, const GValue *v
 }
 
 GdaInternalCommandResult *
-gda_internal_command_detail (SqlConsole *console, GdaConnection *cnc, const gchar **args,
-			     GError **error, gpointer data)
+gda_internal_command_detail (G_GNUC_UNUSED SqlConsole *console, GdaConnection *cnc, const gchar **args,
+			     GError **error, G_GNUC_UNUSED gpointer data)
 {
 	GdaInternalCommandResult *res;
 	GdaDataModel *model;
@@ -763,6 +790,7 @@ gda_internal_command_detail (SqlConsole *console, GdaConnection *cnc, const gcha
 			const gchar *sql = "SELECT constraint_type, constraint_name  "
 				"FROM _table_constraints WHERE table_catalog = ##tc::string "
 				"AND table_schema = ##ts::string AND table_name = ##tname::string "
+				"AND constraint_type != 'FOREIGN KEY' "
 				"ORDER BY constraint_type DESC, constraint_name";
 
 			g_value_set_string ((catalog = gda_value_new (G_TYPE_STRING)), dbo->obj_catalog);
@@ -827,84 +855,7 @@ gda_internal_command_detail (SqlConsole *console, GdaConnection *cnc, const gcha
 					}
 				}
 				else if (*str == 'F') {
-					/* foreign key */
-					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i, error);
-					if (!cvalue) {
-						gda_internal_command_exec_result_free (res);
-						res = NULL;
-						goto out;
-					}
-					string = g_string_new (_("Foreign key"));
-					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
-
-					str = "SELECT ref_table_schema, ref_table_name, fk_column, ref_column "
-						"FROM _detailled_fk WHERE fk_table_catalog = ##tc::string "
-						"AND fk_table_schema = ##ts::string AND fk_table_name = ##tname::string AND "
-						"fk_constraint_name = ##cname::string "
-						"ORDER BY ordinal_position";
-					
-					cols = gda_meta_store_extract (gda_connection_get_meta_store (cnc), str, error, 
-								       "tc", catalog, "ts", schema, "tname", name, "cname", cvalue, 
-								       NULL);
-					if (cols) {
-						gint j, cnrows;
-						cnrows = gda_data_model_get_n_rows (cols);
-						if (cnrows > 0) {
-							g_string_append (string, " (");
-							for (j = 0; j < cnrows; j++) {
-								if (j > 0)
-									g_string_append (string, ", ");
-								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
-								if (!cvalue) {
-									gda_internal_command_exec_result_free (res);
-									res = NULL;
-									g_object_unref (cols);
-									g_string_free (string, TRUE);
-									goto out;
-								}
-								g_string_append (string, g_value_get_string (cvalue));
-							}
-							g_string_append (string, ") references ");
-
-							const GValue *v1, *v2;
-							
-							v1 = gda_data_model_get_value_at (cols, 0, 0, error);
-							if (!v1) {
-								gda_internal_command_exec_result_free (res);
-								res = NULL;
-								g_object_unref (cols);
-								g_string_free (string, TRUE);
-								goto out;
-							}
-							v2 = gda_data_model_get_value_at (cols, 1, 0, error);
-							if (!v2) {
-								gda_internal_command_exec_result_free (res);
-								res = NULL;
-								g_object_unref (cols);
-								g_string_free (string, TRUE);
-								goto out;
-							}
-							g_string_append_printf (string, " %s.%s (", 
-										g_value_get_string (v1),
-										g_value_get_string (v2));
-							for (j = 0; j < cnrows; j++) {
-								if (j > 0)
-									g_string_append (string, ", ");
-								cvalue = gda_data_model_get_value_at (cols, 3, j, error);
-								if (!cvalue) {
-									gda_internal_command_exec_result_free (res);
-									res = NULL;
-									g_object_unref (cols);
-									g_string_free (string, TRUE);
-									goto out;
-								}
-								g_string_append (string, g_value_get_string (cvalue));
-							}
-							g_string_append_c (string, ')');
-						}
-						g_object_unref (cols);
-					}
+					/* foreign key, not handled here */
 				}
 				else if (*str == 'U') {
 					/* Unique constraint */
@@ -963,6 +914,62 @@ gda_internal_command_detail (SqlConsole *console, GdaConnection *cnc, const gcha
 			gda_value_free (name);
 
 			g_object_unref (model);
+
+			/* foreign key constraints */
+			GSList *list;
+			for (list = mt->fk_list; list; list = list->next) {
+				GString *string;
+				GdaMetaTableForeignKey *fk;
+				gint i;
+				fk = (GdaMetaTableForeignKey*) list->data;
+				if (GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED (fk))
+					string = g_string_new (_("Declared foreign key"));
+				else
+					string = g_string_new (_("Foreign key"));
+				g_string_append_printf (string, " '%s' (", fk->fk_name);
+				for (i = 0; i < fk->cols_nb; i++) {
+					if (i != 0)
+						g_string_append (string, ", ");
+					g_string_append (string, fk->fk_names_array [i]);
+				}
+				g_string_append (string, ") ");
+				if (fk->depend_on->obj_short_name)
+					/* To translators: the term "references" is the verb
+					 * "to reference" in the context of foreign keys where
+					 * "table A REFERENCES table B" */
+					g_string_append_printf (string, _("references %s"),
+								fk->depend_on->obj_short_name);
+				else
+					/* To translators: the term "references" is the verb
+					 * "to reference" in the context of foreign keys where
+					 * "table A REFERENCES table B" */
+					g_string_append_printf (string, _("references %s.%s"),
+								fk->depend_on->obj_schema,
+								fk->depend_on->obj_name);
+				g_string_append (string, " (");
+				for (i = 0; i < fk->cols_nb; i++) {
+					if (i != 0)
+						g_string_append (string, ", ");
+					g_string_append (string, fk->ref_pk_names_array [i]);
+				}
+				g_string_append (string, ")");
+
+				g_string_append (string, "\n  ");
+				g_string_append (string, _("Policy on UPDATE"));
+				g_string_append (string, ": ");
+				g_string_append (string, tools_utils_fk_policy_to_string (GDA_META_TABLE_FOREIGN_KEY_ON_UPDATE_POLICY (fk)));
+				g_string_append (string, "\n  ");
+				g_string_append (string, _("Policy on DELETE"));
+				g_string_append (string, ": ");
+				g_string_append (string, tools_utils_fk_policy_to_string (GDA_META_TABLE_FOREIGN_KEY_ON_DELETE_POLICY (fk)));
+
+				subres = g_new0 (GdaInternalCommandResult, 1);
+				subres->type = GDA_INTERNAL_COMMAND_RESULT_TXT;
+				subres->u.txt = string;
+				res->u.multiple_results = g_slist_append (res->u.multiple_results,
+									  subres);
+			}
+
 			return res;
 		}
 	}
